@@ -33,6 +33,7 @@
 #include <gtkmm/accelmap.h>
 
 #include <pbd/error.h>
+#include <pbd/basename.h>
 #include <pbd/compose.h>
 #include <pbd/misc.h>
 #include <pbd/pathscanner.h>
@@ -74,6 +75,7 @@
 #include "add_route_dialog.h"
 #include "new_session_dialog.h"
 #include "about.h"
+#include "splash.h"
 #include "utils.h"
 #include "gui_thread.h"
 #include "theme_manager.h"
@@ -115,7 +117,7 @@ ARDOUR_UI::ARDOUR_UI (int *argcp, char **argvp[])
 
 	  /* big clock */
 
-	  big_clock (X_("bigclock"), false, "BigClockNonRecording", false, false, true),
+	  big_clock (X_("bigclock"), false, "BigClockNonRecording", true, false, true),
 
 	  /* transport */
 
@@ -165,6 +167,12 @@ ARDOUR_UI::ARDOUR_UI (int *argcp, char **argvp[])
 	}
 	
 	about = 0;
+	splash = 0;
+
+	if (ARDOUR_COMMAND_LINE::session_name.length()) {	
+		/* only show this if we're not going to post the new session dialog */
+		show_splash ();
+	}
 
 	if (theArdourUI == 0) {
 		theArdourUI = this;
@@ -187,12 +195,12 @@ ARDOUR_UI::ARDOUR_UI (int *argcp, char **argvp[])
 	route_params = 0;
 	option_editor = 0;
 	location_ui = 0;
+	key_editor = 0;
 	open_session_selector = 0;
 	have_configure_timeout = false;
 	have_disk_speed_dialog_displayed = false;
 	_will_create_new_session_automatically = false;
 	session_loaded = false;
-	loading_dialog = 0;
 	last_speed_displayed = -1.0f;
 
 	keybindings_path = ARDOUR::find_config_file ("ardour.bindings");
@@ -223,10 +231,17 @@ ARDOUR_UI::ARDOUR_UI (int *argcp, char **argvp[])
 
 	ARDOUR::Session::AskAboutPendingState.connect (mem_fun(*this, &ARDOUR_UI::pending_state_dialog));
 
+	/* handle sr mismatch with a dialog */
+
+	ARDOUR::Session::AskAboutSampleRateMismatch.connect (mem_fun(*this, &ARDOUR_UI::sr_mismatch_dialog));
+
 	/* lets get this party started */
 
     	try {
-		ARDOUR::init (ARDOUR_COMMAND_LINE::use_vst, ARDOUR_COMMAND_LINE::try_hw_optimization);
+		if (ARDOUR::init (ARDOUR_COMMAND_LINE::use_vst, ARDOUR_COMMAND_LINE::try_hw_optimization)) {
+			throw failed_constructor ();
+		}
+
 		setup_gtk_ardour_enums ();
 		Config->set_current_owner (ConfigVariableBase::Interface);
 		setup_profile ();
@@ -234,7 +249,7 @@ ARDOUR_UI::ARDOUR_UI (int *argcp, char **argvp[])
 	} catch (failed_constructor& err) {
 		error << _("could not initialize Ardour.") << endmsg;
 		// pass it on up
-		throw err;
+		throw;
 	} 
 
 	/* we like keyboards */
@@ -243,33 +258,27 @@ ARDOUR_UI::ARDOUR_UI (int *argcp, char **argvp[])
 
 	starting.connect (mem_fun(*this, &ARDOUR_UI::startup));
 	stopping.connect (mem_fun(*this, &ARDOUR_UI::shutdown));
+
+	platform_setup ();
 }
 
-gint
-ARDOUR_UI::start_backend_audio ()
-{
-	if (new_session_dialog->engine_control.start_engine ()) {
-		return -1;
-	} 
-
-	return 0;
-}
-
-void
+int
 ARDOUR_UI::create_engine ()
 {
 	// this gets called every time by new_session()
 
 	if (engine) {
-		return;
+		return 0;
 	}
 
+	loading_message (_("Starting audio engine"));
+
 	try { 
 		engine = new ARDOUR::AudioEngine (ARDOUR_COMMAND_LINE::jack_client_name);
-	} catch (AudioEngine::NoBackendAvailable& err) {
-		backend_audio_error ();
-		error << string_compose (_("Could not connect to JACK server as  \"%1\""), ARDOUR_COMMAND_LINE::jack_client_name) <<  endmsg;
-		exit (1);
+
+	} catch (...) {
+
+		return -1;
 	}
 
 	engine->Stopped.connect (mem_fun(*this, &ARDOUR_UI::engine_stopped));
@@ -278,6 +287,8 @@ ARDOUR_UI::create_engine ()
 	engine->SampleRateChanged.connect (mem_fun(*this, &ARDOUR_UI::update_sample_rate));
 
 	post_engine ();
+
+	return 0;
 }
 
 void
@@ -333,6 +344,7 @@ ARDOUR_UI::post_engine ()
 	/* start the time-of-day-clock */
 	
 #ifndef GTKOSX
+	/* OS X provides an always visible wallclock, so don't be stupid */
 	update_wall_clock ();
 	Glib::signal_timeout().connect (mem_fun(*this, &ARDOUR_UI::update_wall_clock), 60000);
 #endif
@@ -341,6 +353,8 @@ ARDOUR_UI::post_engine ()
 	update_cpu_load ();
 	update_sample_rate (engine->frame_rate());
 
+	platform_specific ();
+
 	/* now start and maybe save state */
 
 	if (do_engine_start () == 0) {
@@ -378,6 +392,15 @@ ARDOUR_UI::~ARDOUR_UI ()
 	}
 }
 
+void
+ARDOUR_UI::pop_back_splash ()
+{
+	if (Splash::instance()) {
+		// Splash::instance()->pop_back();
+		Splash::instance()->hide ();
+	}
+}
+
 gint
 ARDOUR_UI::configure_timeout ()
 {
@@ -494,6 +517,7 @@ ARDOUR_UI::save_ardour_state ()
 		}
 	}
 	Config->save_state();
+	ui_config->save_state ();
 
 	XMLNode enode(static_cast<Stateful*>(editor)->get_state());
 	XMLNode mnode(mixer->get_state());
@@ -543,29 +567,52 @@ ARDOUR_UI::update_autosave ()
 }
 
 void
-ARDOUR_UI::backend_audio_error ()
+ARDOUR_UI::backend_audio_error (bool we_set_params, Gtk::Window* toplevel)
 {
-	MessageDialog win (_("Ardour could not connect to JACK."),
-		     false,
-		     Gtk::MESSAGE_INFO,
-		     (Gtk::ButtonsType)(Gtk::BUTTONS_NONE));
-win.set_secondary_text(_("There are several possible reasons:\n\
+	string title;
+	if (we_set_params) {
+		title = _("Ardour could not start JACK");
+	} else {
+		title = _("Ardour could not connect to JACK.");
+	}
+
+	MessageDialog win (title,
+			   false,
+			   Gtk::MESSAGE_INFO,
+			   Gtk::BUTTONS_NONE);
+	
+	if (we_set_params) {
+		win.set_secondary_text(_("There are several possible reasons:\n\
+\n\
+1) You requested audio parameters that are not supported..\n\
+2) JACK is running as another user.\n\
+\n\
+Please consider the possibilities, and perhaps try different parameters."));
+	} else {
+		win.set_secondary_text(_("There are several possible reasons:\n\
 \n\
 1) JACK is not running.\n\
 2) JACK is running as another user, perhaps root.\n\
 3) There is already another client called \"ardour\".\n\
 \n\
 Please consider the possibilities, and perhaps (re)start JACK."));
+	}
+
+	if (toplevel) {
+		win.set_transient_for (*toplevel);
+	}
+
+	if (we_set_params) {
+		win.add_button (Stock::OK, RESPONSE_CLOSE);
+	} else {
+		win.add_button (Stock::QUIT, RESPONSE_CLOSE);
+	}
 
-	win.add_button (Stock::QUIT, RESPONSE_CLOSE);
 	win.set_default_response (RESPONSE_CLOSE);
 	
 	win.show_all ();
 	win.set_position (Gtk::WIN_POS_CENTER);
-
-	if (!ARDOUR_COMMAND_LINE::no_splash) {
-		hide_splash ();
-	}
+	pop_back_splash ();
 
 	/* we just don't care about the result, but we want to block */
 
@@ -576,88 +623,20 @@ void
 ARDOUR_UI::startup ()
 {
 	string name, path;
-	bool isnew;
 
 	new_session_dialog = new NewSessionDialog();
+
+	bool backend_audio_is_running = EngineControl::engine_running();
+	XMLNode* audio_setup = Config->extra_xml ("AudioSetup");
 	
-	/* If no session name is given: we're not loading a session yet, nor creating a new one */
-	
-	if (ARDOUR_COMMAND_LINE::session_name.length()) {
-	
-		/* Load session or start the new session dialog */
-		
-		if (Session::find_session (ARDOUR_COMMAND_LINE::session_name, path, name, isnew)) {
-			error << string_compose(_("could not load command line session \"%1\""), 
-						ARDOUR_COMMAND_LINE::session_name) << endmsg;
-			return;
-		}
-
-		if (!ARDOUR_COMMAND_LINE::new_session) {
-			
-			/* Supposed to be loading an existing session, but the session doesn't exist */
-			
-			if (isnew) {
-				error << string_compose (_("\n\nNo session named \"%1\" exists.\n"
-							   "To create it from the command line, start ardour as \"ardour --new %1"), path) 
-				      << endmsg;
-				return;
-			}
-		}
-		
-		new_session_dialog->set_session_name (name);
-		new_session_dialog->set_session_folder (Glib::path_get_basename (path));
-		_session_is_new = isnew;
-	}
-
-	hide_splash ();
-
-	bool have_backend = EngineControl::engine_running();
-	bool need_nsd;
-	bool load_needed = false;
-
-	if (have_backend) {
-
-		/* backend audio is working */
-
-		if (ARDOUR_COMMAND_LINE::session_name.empty() || ARDOUR_COMMAND_LINE::new_session) {
-			/* need NSD to get session name and other info */
-			need_nsd = true;
-		} else {
-			need_nsd = false;
-		}
-		
-	} else {
-
-		XMLNode* audio_setup = Config->extra_xml ("AudioSetup");
-		
-		if (audio_setup) {
-			new_session_dialog->engine_control.set_state (*audio_setup);
-		}
-
-		/* no backend audio, must bring up NSD to check configuration */
-		
-		need_nsd = true;
-	}
-
-	if (need_nsd) {
-
-		if (!get_session_parameters (ARDOUR_COMMAND_LINE::session_name, have_backend, ARDOUR_COMMAND_LINE::new_session)) {
-			return;
-		}
-
-	} else {
-
-		load_needed = true;
+	if (audio_setup) {
+		new_session_dialog->engine_control.set_state (*audio_setup);
 	}
 	
-	create_engine ();
-
-	if (load_needed) {
-		if (load_session (ARDOUR_COMMAND_LINE::session_name, name)) {
-			return;
-		}
+	if (!get_session_parameters (backend_audio_is_running, ARDOUR_COMMAND_LINE::new_session)) {
+		return;
 	}
-
+	
 	show ();
 }
 
@@ -714,6 +693,8 @@ ARDOUR_UI::check_memory_locking ()
 				hbox.pack_start (cb, true, false);
 				vbox->pack_start (hbox);
 				hbox.show_all ();
+
+				pop_back_splash ();
 				
 				msg.run ();
 			}
@@ -726,32 +707,40 @@ ARDOUR_UI::check_memory_locking ()
 void
 ARDOUR_UI::finish()
 {
-	if (session && session->dirty()) {
-		switch (ask_about_saving_session(_("quit"))) {
-		case -1:
-			return;
-			break;
-		case 1:
-			/* use the default name */
-			if (save_state_canfail ("")) {
-				/* failed - don't quit */
-				MessageDialog msg (*editor, 
-					       _("\
+	if (session) {
+
+		if (session->transport_rolling()) {
+			session->request_stop ();
+			usleep (2500000);
+		}
+
+		if (session->dirty()) {
+			switch (ask_about_saving_session(_("quit"))) {
+			case -1:
+				return;
+				break;
+			case 1:
+				/* use the default name */
+				if (save_state_canfail ("")) {
+					/* failed - don't quit */
+					MessageDialog msg (*editor, 
+							   _("\
 Ardour was unable to save your session.\n\n\
 If you still wish to quit, please use the\n\n\
 \"Just quit\" option."));
-				msg.run ();
-				return;
+					pop_back_splash();
+					msg.run ();
+					return;
+				}
+				break;
+			case 0:
+				break;
 			}
-			break;
-		case 0:
-			break;
 		}
-	}
-
-	if (session) {
+		
 		session->set_deletion_in_progress ();
 	}
+
 	engine->stop (true);
 	save_ardour_state ();
 	quit ();
@@ -1012,6 +1001,13 @@ ARDOUR_UI::redisplay_recent_sessions ()
 			fullpath = fullpath.substr (0, fullpath.length()-1);
 		}
 
+		/* check whether session still exists */
+		if (!Glib::file_test(fullpath.c_str(), Glib::FILE_TEST_EXISTS)) {
+			/* session doesn't exist */
+			cerr << "skipping non-existent session " << fullpath << endl;
+			continue;
+		}		
+		
 		/* now get available states for this session */
 
 		if ((states = Session::possible_states (fullpath)) == 0) {
@@ -1060,8 +1056,7 @@ ARDOUR_UI::build_session_selector ()
 	recent_session_display.set_model (recent_session_model);
 	recent_session_display.append_column (_("Recent Sessions"), recent_session_columns.visible_name);
 	recent_session_display.set_headers_visible (false);
-	recent_session_display.get_selection()->set_mode (SELECTION_SINGLE);
-
+	recent_session_display.get_selection()->set_mode (SELECTION_BROWSE);
 	recent_session_display.signal_row_activated().connect (mem_fun (*this, &ARDOUR_UI::recent_session_row_activated));
 
 	scroller->add (recent_session_display);
@@ -1082,37 +1077,55 @@ ARDOUR_UI::recent_session_row_activated (const TreePath& path, TreeViewColumn* c
 void
 ARDOUR_UI::open_recent_session ()
 {
-	/* popup selector window */
+	bool can_return = (session != 0);
 
 	if (session_selector_window == 0) {
 		build_session_selector ();
 	}
-
+	
 	redisplay_recent_sessions ();
 
-	ResponseType r = (ResponseType) session_selector_window->run ();
+	while (true) {
+		
+		session_selector_window->set_position (WIN_POS_MOUSE);
 
-	session_selector_window->hide();
+		ResponseType r = (ResponseType) session_selector_window->run ();
+		
+		switch (r) {
+		case RESPONSE_ACCEPT:
+			break;
+		default:
+			if (can_return) {
+				session_selector_window->hide();
+				return;
+			} else {
+				exit (1);
+			}
+		}
 
-	switch (r) {
-	case RESPONSE_ACCEPT:
-		break;
-	default:
-		return;
+		if (recent_session_display.get_selection()->count_selected_rows() == 0) {
+			continue;
+		}
+		
+		session_selector_window->hide();
+
+		Gtk::TreeModel::iterator i = recent_session_display.get_selection()->get_selected();
+		
+		if (i == recent_session_model->children().end()) {
+			return;
+		}
+		
+		Glib::ustring path = (*i)[recent_session_columns.fullpath];
+		Glib::ustring state = (*i)[recent_session_columns.visible_name];
+		
+		_session_is_new = false;
+		
+		if (load_session (path, state) == 0) {
+			break;
+		}
+
+		can_return = false;
 	}
-
-	Gtk::TreeModel::iterator i = recent_session_display.get_selection()->get_selected();
-
-	if (i == recent_session_model->children().end()) {
-		return;
-	}
-	
-	Glib::ustring path = (*i)[recent_session_columns.fullpath];
-	Glib::ustring state = (*i)[recent_session_columns.visible_name];
-
-	_session_is_new = false;
-
-	load_session (path, state);
 }
 
 bool
@@ -1149,6 +1162,7 @@ ARDOUR_UI::check_audioengine ()
 		if (!engine->connected()) {
 			MessageDialog msg (_("Ardour is not connected to JACK\n"
 					     "You cannot open or close sessions in this condition"));
+			pop_back_splash ();
 			msg.run ();
 			return false;
 		}
@@ -1265,12 +1279,12 @@ ARDOUR_UI::session_add_audio_route (bool track, int32_t input_channels, int32_t
 	}
 
 	catch (...) {
-		cerr << "About to complain about JACK\n";
 		MessageDialog msg (*editor, 
 				   _("There are insufficient JACK ports available\n\
 to create a new track or bus.\n\
 You should save Ardour, exit and\n\
 restart JACK with more ports."));
+		pop_back_splash ();
 		msg.run ();
 	}
 }
@@ -1381,7 +1395,7 @@ ARDOUR_UI::remove_last_capture()
 }
 
 void
-ARDOUR_UI::transport_record ()
+ARDOUR_UI::transport_record (bool roll)
 {
 	if (session) {
 		switch (session->record_status()) {
@@ -1392,8 +1406,18 @@ ARDOUR_UI::transport_record ()
 				return;
 			}
 			session->maybe_enable_record ();
+			if (roll) {
+				transport_roll ();
+			}
 			break;
 		case Session::Recording:
+			if (roll) {
+				session->request_stop();
+			} else {
+				session->disable_record (false, true);
+			}
+			break;
+
 		case Session::Enabled:
 			session->disable_record (false, true);
 		}
@@ -1637,6 +1661,7 @@ JACK has either been shutdown or it\n\
 disconnected Ardour because Ardour\n\
 was not fast enough. You can save the\n\
 session and/or try to reconnect to JACK ."));
+	pop_back_splash ();
 	msg.run ();
 }
 
@@ -1668,7 +1693,7 @@ void
 ARDOUR_UI::update_clocks ()
 {
 	if (!editor || !editor->dragging_playhead()) {
-		Clock (session->audible_frame(), false, editor->edit_cursor_position(false)); /* EMIT_SIGNAL */
+		Clock (session->audible_frame(), false, editor->get_preferred_edit_position()); /* EMIT_SIGNAL */
 	}
 }
 
@@ -1853,6 +1878,14 @@ ARDOUR_UI::primary_clock_value_changed ()
 	}
 }
 
+void
+ARDOUR_UI::big_clock_value_changed ()
+{
+	if (session) {
+		session->request_locate (big_clock.current_time ());
+	}
+}
+
 void
 ARDOUR_UI::secondary_clock_value_changed ()
 {
@@ -1956,277 +1989,396 @@ ARDOUR_UI::save_template ()
 	}
 }
 
-bool
-ARDOUR_UI::get_session_parameters (Glib::ustring predetermined_path, bool have_engine, bool should_be_new)
+void
+ARDOUR_UI::fontconfig_dialog ()
 {
-	string session_name;
-	string session_path;
-	string template_name;
-
-	if (!loading_dialog) {
-		loading_dialog = new MessageDialog (*new_session_dialog, 
-						    _("Starting audio engine"),
-						    false,
-						    Gtk::MESSAGE_INFO,
-						    Gtk::BUTTONS_NONE);
+#ifdef GTKOSX
+	/* X11 users will always have fontconfig info around, but new GTK-OSX users 
+	   may not and it can take a while to build it. Warn them.
+	*/
+	
+	Glib::ustring fontconfig = Glib::build_filename (Glib::get_home_dir(), ".fontconfig");
+	
+	if (!Glib::file_test (fontconfig, Glib::FILE_TEST_EXISTS|Glib::FILE_TEST_IS_DIR)) {
+		MessageDialog msg (*new_session_dialog,
+				   _("Welcome to Ardour.\n\n"
+				     "The program will take a bit longer to start up\n"
+				     "while the system fonts are checked.\n\n"
+				     "This will only be done once, and you will\n"
+				     "not see this message again\n"),
+				   true,
+				   Gtk::MESSAGE_INFO,
+				   Gtk::BUTTONS_OK);
+		pop_back_splash ();
+		msg.show_all ();
+		msg.present ();
+		msg.run ();
 	}
+#endif
+}
+
+void
+ARDOUR_UI::parse_cmdline_path (const Glib::ustring& cmdline_path, Glib::ustring& session_name, Glib::ustring& session_path, bool& existing_session)
+{
+	existing_session = false;
+
+	if (Glib::file_test (cmdline_path, Glib::FILE_TEST_IS_DIR)) {
+		session_path = cmdline_path;
+		existing_session = true;
+	} else if (Glib::file_test (cmdline_path, Glib::FILE_TEST_IS_REGULAR)) {
+		session_path = Glib::path_get_dirname (string (cmdline_path));
+		existing_session = true;
+	} else {
+		/* it doesn't exist, assume the best */
+		session_path = Glib::path_get_dirname (string (cmdline_path));
+	}
+	
+	session_name = basename_nosuffix (string (cmdline_path));
+}
+
+int
+ARDOUR_UI::load_cmdline_session (const Glib::ustring& session_name, const Glib::ustring& session_path, bool& existing_session)
+{
+	/* when this is called, the backend audio system must be running */
+
+	/* the main idea here is to deal with the fact that a cmdline argument for the session
+	   can be interpreted in different ways - it could be a directory or a file, and before
+	   we load, we need to know both the session directory and the snapshot (statefile) within it
+	   that we are supposed to use.
+	*/
+
+	if (session_name.length() == 0 || session_path.length() == 0) {
+		return false;
+	}
+	
+	if (Glib::file_test (session_path, Glib::FILE_TEST_IS_DIR)) {
+
+		Glib::ustring predicted_session_file;
 		
+		predicted_session_file = session_path;
+		predicted_session_file += '/';
+		predicted_session_file += session_name;
+		predicted_session_file += Session::statefile_suffix();
+		
+		if (Glib::file_test (predicted_session_file, Glib::FILE_TEST_EXISTS)) {
+			existing_session = true;
+		}
+		
+	} else if (Glib::file_test (session_path, Glib::FILE_TEST_EXISTS)) {
+		
+		if (session_path.find (Session::statefile_suffix()) == session_path.length() - 7) {
+			/* existing .ardour file */
+			existing_session = true;
+		}
+
+	} else {
+		existing_session = false;
+	}
+	
+	/* lets just try to load it */
+	
+	if (create_engine ()) {
+		backend_audio_error (false, new_session_dialog);
+		return -1;
+	}
+	
+	return load_session (session_path, session_name);
+}
+
+bool
+ARDOUR_UI::ask_about_loading_existing_session (const Glib::ustring& session_path)
+{
+	Glib::ustring str = string_compose (_("This session\n%1\nalready exists. Do you want to open it?"), session_path);
+	
+	MessageDialog msg (str,
+			   false,
+			   Gtk::MESSAGE_WARNING,
+			   Gtk::BUTTONS_YES_NO,
+			   true);
+	
+	
+	msg.set_name (X_("CleanupDialog"));
+	msg.set_wmclass (X_("existing_session"), "Ardour");
+	msg.set_position (Gtk::WIN_POS_MOUSE);
+	pop_back_splash ();
+
+	switch (msg.run()) {
+	case RESPONSE_YES:
+		return true;
+		break;
+	}
+	return false;
+}
+
+int
+ARDOUR_UI::build_session_from_nsd (const Glib::ustring& session_path, const Glib::ustring& session_name)
+{
+	
+	uint32_t cchns;
+	uint32_t mchns;
+	AutoConnectOption iconnect;
+	AutoConnectOption oconnect;
+	uint32_t nphysin;
+	uint32_t nphysout;
+	
+	if (Profile->get_sae()) {
+		
+		cchns = 0;
+		mchns = 2;
+		iconnect = AutoConnectPhysical;
+		oconnect = AutoConnectMaster;
+		nphysin = 0; // use all available
+		nphysout = 0; // use all available
+		
+	} else {
+		
+		/* get settings from advanced section of NSD */
+		
+		if (new_session_dialog->create_control_bus()) {
+			cchns = (uint32_t) new_session_dialog->control_channel_count();
+		} else {
+			cchns = 0;
+		}
+		
+		if (new_session_dialog->create_master_bus()) {
+			mchns = (uint32_t) new_session_dialog->master_channel_count();
+		} else {
+			mchns = 0;
+		}
+		
+		if (new_session_dialog->connect_inputs()) {
+			iconnect = AutoConnectPhysical;
+		} else {
+			iconnect = AutoConnectOption (0);
+		}
+		
+		/// @todo some minor tweaks.
+		
+		if (new_session_dialog->connect_outs_to_master()) {
+			oconnect = AutoConnectMaster;
+		} else if (new_session_dialog->connect_outs_to_physical()) {
+			oconnect = AutoConnectPhysical;
+		} else {
+			oconnect = AutoConnectOption (0);
+		} 
+		
+		nphysin = (uint32_t) new_session_dialog->input_limit_count();
+		nphysout = (uint32_t) new_session_dialog->output_limit_count();
+	}
+	
+	if (build_session (session_path,
+			   session_name,
+			   cchns,
+			   mchns,
+			   iconnect,
+			   oconnect,
+			   nphysin,
+			   nphysout, 
+			   engine->frame_rate() * 60 * 5)) {
+		
+		return -1;
+	}
+
+	return 0;
+}
+
+void
+ARDOUR_UI::end_loading_messages ()
+{
+	// hide_splash ();
+}
+
+void
+ARDOUR_UI::loading_message (const std::string& msg)
+{
+	show_splash ();
+	splash->message (msg);
+	flush_pending ();
+}
+	
+bool
+ARDOUR_UI::get_session_parameters (bool backend_audio_is_running, bool should_be_new)
+{
+	bool existing_session = false;
+	Glib::ustring session_name;
+	Glib::ustring session_path;
+	Glib::ustring template_name;
+
 	int response = Gtk::RESPONSE_NONE;
 
-	new_session_dialog->set_modal(true);
-	new_session_dialog->set_name (predetermined_path);
-	new_session_dialog->reset_recent();
+	if (!ARDOUR_COMMAND_LINE::session_name.empty()) {
+
+		parse_cmdline_path (ARDOUR_COMMAND_LINE::session_name, session_name, session_path, existing_session);
+
+		/* don't ever reuse this */
+
+		ARDOUR_COMMAND_LINE::session_name = string();
+
+		if (existing_session && backend_audio_is_running) {
+
+			/* just load the thing already */
+
+			if (load_cmdline_session (session_name, session_path, existing_session) == 0) {
+				return true;
+			}
+		}
+
+		/* make the NSD use whatever information we have */
+
+		new_session_dialog->set_session_name (session_name);
+		new_session_dialog->set_session_folder (session_path);
+	}
+
+	/* loading failed, or we need the NSD for something */
+
+	new_session_dialog->set_modal (false);
 	new_session_dialog->set_position (WIN_POS_CENTER);
-	new_session_dialog->show();
-	new_session_dialog->present ();
 	new_session_dialog->set_current_page (0);
+	new_session_dialog->set_existing_session (existing_session);
+	new_session_dialog->reset_recent();
 
 	do {
-
-		new_session_dialog->set_have_engine (have_engine);
-
-	        response = new_session_dialog->run ();
-
+		new_session_dialog->set_have_engine (backend_audio_is_running);
+		new_session_dialog->present ();
+		response = new_session_dialog->run ();
+		
 		_session_is_new = false;
+		
+		/* handle possible negative responses */
 
-		if (response == Gtk::RESPONSE_CANCEL || response == Gtk::RESPONSE_DELETE_EVENT) {
-
+		switch (response) {
+		case Gtk::RESPONSE_CANCEL:
+		case Gtk::RESPONSE_DELETE_EVENT:
 			if (!session) {
 				quit();
 			}
 			new_session_dialog->hide ();
 			return false;
-
-		} else if (response == Gtk::RESPONSE_NONE) {
-
-		        /* Clear was pressed */
-		        new_session_dialog->reset();
-			continue;
+			
+		case Gtk::RESPONSE_NONE:
+			/* "Clear" was pressed */
+			goto try_again;
 		}
 
-		/* first things first ... we need an audio engine running */
+		fontconfig_dialog();
 
-		if (!have_engine) {
-			if (start_backend_audio ()) {
+		if (!backend_audio_is_running) {
+			if (new_session_dialog->engine_control.setup_engine ()) {
 				new_session_dialog->hide ();
 				return false;
-			}
-			have_engine = true;
+			} 
+		}
+		
+		if (create_engine ()) {
+
+			backend_audio_error (!backend_audio_is_running, new_session_dialog);
+			flush_pending ();
+
+			new_session_dialog->set_existing_session (false);
+			new_session_dialog->set_current_page (2);
+
+			response = Gtk::RESPONSE_NONE;
+			goto try_again;
 		}
 
-		loading_dialog->show_all ();
-		flush_pending ();
-		
-		create_engine ();
-
-		/* now handle possible affirmative responses */
-
-		if (response == Gtk::RESPONSE_YES) {
-
-		        /* YES == OPEN from the session selector */
-
-		        session_name = new_session_dialog->session_name();
+		backend_audio_is_running = true;		
 			
+		if (response == Gtk::RESPONSE_OK) {
+
+			session_name = new_session_dialog->session_name();
+
 			if (session_name.empty()) {
 				response = Gtk::RESPONSE_NONE;
-				continue;
+				goto try_again;
 			} 
 
+			/* if the user mistakenly typed path information into the session filename entry,
+			   convert what they typed into a path & a name
+			*/
+			
 			if (session_name[0] == '/' || 
 			    (session_name.length() > 2 && session_name[0] == '.' && session_name[1] == '/') ||
 			    (session_name.length() > 3 && session_name[0] == '.' && session_name[1] == '.' && session_name[2] == '/')) {
-				load_session (Glib::path_get_dirname (session_name), session_name);
-			} else {
-				session_path = new_session_dialog->session_folder();
-				load_session (session_path, session_name);
-			}
-			
-		} else if (response == Gtk::RESPONSE_OK) {
-
-			/* OK == OPEN button */
-
-			session_name = new_session_dialog->session_name();
-		
-			if (session_name.empty()) {
-				response = Gtk::RESPONSE_NONE;
-				continue;
-			} 
 				
-			switch (new_session_dialog->get_current_page()) {
-			case 1: /* recent session selector */
-			case 2: /* audio engine control */
+				session_path = Glib::path_get_dirname (session_name);
+				session_name = Glib::path_get_basename (session_name);
+				
+			} else {
 
-				if (session_name[0] == '/' || 
-				    (session_name.length() > 2 && session_name[0] == '.' && session_name[1] == '/') ||
-				    (session_name.length() > 3 && session_name[0] == '.' && session_name[1] == '.' && session_name[2] == '/')) {
-					load_session (Glib::path_get_dirname (session_name), session_name);
-				} else {
-					session_path = new_session_dialog->session_folder();
-					load_session (session_path, session_name);
-				}
+				session_path = new_session_dialog->session_folder();
+			}
+
+			template_name = Glib::ustring();			
+			switch (new_session_dialog->which_page()) {
+
+			case NewSessionDialog::OpenPage: 
+			case NewSessionDialog::EnginePage:
+				goto loadit;
 				break;
 
-			case 0: /* nominally the "new" session creator, but could be in use for an old session */
-
-				if (new_session_dialog->get_current_page() == 0 && ARDOUR_COMMAND_LINE::session_name.empty()) {
-					should_be_new = true;
-				}
-
-				/* handle what appear to be paths rather than just a name */
-
-				if (session_name[0] == '/' || 
-				    (session_name.length() > 2 && session_name[0] == '.' && session_name[1] == '/') ||
-				    (session_name.length() > 3 && session_name[0] == '.' && session_name[1] == '.' && session_name[2] == '/')) {
-
-					session_path = Glib::path_get_dirname (session_name);
-					session_name = Glib::path_get_basename (session_name);
-
-				} else {
-
-					session_path = new_session_dialog->session_folder();
-
-				}
+			case NewSessionDialog::NewPage: /* nominally the "new" session creator, but could be in use for an old session */
+				
+				should_be_new = true;
 				
 				//XXX This is needed because session constructor wants a 
 				//non-existant path. hopefully this will be fixed at some point.
 				
 				session_path = Glib::build_filename (session_path, session_name);
-				
-				if (!should_be_new) {
 
-					load_session (session_path, session_name);
-					continue; /* leaves while() loop because response != NONE */
+				if (Glib::file_test (session_path, Glib::FileTest (G_FILE_TEST_EXISTS | G_FILE_TEST_IS_DIR))) {
 
-				} else if (Glib::file_test (session_path, Glib::FileTest (G_FILE_TEST_EXISTS | G_FILE_TEST_IS_DIR))) {
-
-					Glib::ustring str = string_compose (_("This session\n%1\nalready exists. Do you want to open it?"), session_path);
-
-					MessageDialog msg (str,
-							   false,
-							   Gtk::MESSAGE_WARNING,
-							   Gtk::BUTTONS_YES_NO,
-							   true);
-
-
-					msg.set_name (X_("CleanupDialog"));
-					msg.set_wmclass (X_("existing_session"), "Ardour");
-					msg.set_position (Gtk::WIN_POS_MOUSE);
-					
-					switch (msg.run()) {
-					case RESPONSE_YES:
-						new_session_dialog->hide ();
-						goto_editor_window ();
-						flush_pending ();
-						load_session (session_path, session_name);
-						goto done;
-						break;
-					default:
+					if (ask_about_loading_existing_session (session_path)) {
+						goto loadit;
+					} else {
 						response = RESPONSE_NONE;
-						new_session_dialog->reset ();
-						continue;
-					}
-				} 
+						goto try_again;
+					} 
+				}
 
 			        _session_is_new = true;
 						
 				if (new_session_dialog->use_session_template()) {
 
 					template_name = new_session_dialog->session_template_name();
-
-					new_session_dialog->hide ();
-					goto_editor_window ();
-					flush_pending ();
-
-					load_session (session_path, session_name, &template_name);
+					goto loadit;
 			  
 				} else {
-
-					uint32_t cchns;
-					uint32_t mchns;
-					AutoConnectOption iconnect;
-					AutoConnectOption oconnect;
-					uint32_t nphysin;
-					uint32_t nphysout;
-					
-					if (Profile->get_sae()) {
-
-						cchns = 0;
-						mchns = 2;
-						iconnect = AutoConnectPhysical;
-						oconnect = AutoConnectMaster;
-						nphysin = 0; // use all available
-						nphysout = 0; // use all available
-
-					} else {
-
-						/* get settings from advanced section of NSD */
-						
-						if (new_session_dialog->create_control_bus()) {
-							cchns = (uint32_t) new_session_dialog->control_channel_count();
-						} else {
-							cchns = 0;
-						}
-						
-						if (new_session_dialog->create_master_bus()) {
-							mchns = (uint32_t) new_session_dialog->master_channel_count();
-						} else {
-							mchns = 0;
-						}
-						
-						if (new_session_dialog->connect_inputs()) {
-							iconnect = AutoConnectPhysical;
-						} else {
-							iconnect = AutoConnectOption (0);
-						}
-						
-						/// @todo some minor tweaks.
-						
-						if (new_session_dialog->connect_outs_to_master()) {
-							oconnect = AutoConnectMaster;
-						} else if (new_session_dialog->connect_outs_to_physical()) {
-							oconnect = AutoConnectPhysical;
-						} else {
-							oconnect = AutoConnectOption (0);
-						} 
-						
-						nphysin = (uint32_t) new_session_dialog->input_limit_count();
-						nphysout = (uint32_t) new_session_dialog->output_limit_count();
-					}
-
-					new_session_dialog->hide ();
-					goto_editor_window ();
-					flush_pending ();
-
-					if (build_session (session_path,
-							   session_name,
-							   cchns,
-							   mchns,
-							   iconnect,
-							   oconnect,
-							   nphysin,
-							   nphysout, 
-							   engine->frame_rate() * 60 * 5)) {
-						
-						response = Gtk::RESPONSE_NONE;
-						new_session_dialog->reset ();
-						continue;
+					if (build_session_from_nsd (session_path, session_name)) {
+						response = RESPONSE_NONE;
+						goto try_again;
 					}
+					goto done;
 				}
 				break;
-
+				
 			default:
 				break;
 			}
+			
+		  loadit:
+			new_session_dialog->hide ();
+			
+			if (load_session (session_path, session_name, template_name)) {
+				/* force a retry */
+				response = Gtk::RESPONSE_NONE;
+			}
+
+		  try_again:
+			if (response == Gtk::RESPONSE_NONE) {
+				new_session_dialog->set_existing_session (false);
+				new_session_dialog->reset ();
+			}
 		}
-		
+
 	} while (response == Gtk::RESPONSE_NONE);
 
   done:
 	show();
-	loading_dialog->hide ();
 	new_session_dialog->hide();
+	new_session_dialog->reset();
+	goto_editor_window ();
 	return true;
-}
+}	
 
 void
 ARDOUR_UI::close_session ()
@@ -2237,11 +2389,11 @@ ARDOUR_UI::close_session ()
 
 	unload_session (true);
 	
-	get_session_parameters ("", true, false);
+	get_session_parameters (true, false);
 }
 
 int
-ARDOUR_UI::load_session (const string & path, const string & snap_name, string* mix_template)
+ARDOUR_UI::load_session (const Glib::ustring& path, const Glib::ustring& snap_name, Glib::ustring mix_template)
 {
 	Session *new_session;
 	int unload_status;
@@ -2264,26 +2416,72 @@ ARDOUR_UI::load_session (const string & path, const string & snap_name, string*
 
 	/* if it already exists, we must have write access */
 
-	if (::access (path.c_str(), F_OK) == 0 && ::access (path.c_str(), W_OK)) {
+	if (Glib::file_test (path.c_str(), Glib::FILE_TEST_EXISTS) && ::access (path.c_str(), W_OK)) {
 		MessageDialog msg (*editor, _("You do not have write access to this session.\n"
 					      "This prevents the session from being loaded."));
+		pop_back_splash ();
 		msg.run ();
 		goto out;
 	}
 
-	if (loading_dialog) {
-		loading_dialog->set_markup (_("Please wait while Ardour loads your session"));
-		flush_pending ();
-	}
-
+	loading_message (_("Please wait while Ardour loads your session"));
 	disable_screen_updates ();
 
 	try {
 		new_session = new Session (*engine, path, snap_name, mix_template);
 	}
 
+	/* this one is special */
+
+	catch (AudioEngine::PortRegistrationFailure& err) {
+
+		MessageDialog msg (err.what(),
+				   true,
+				   Gtk::MESSAGE_INFO,
+				   Gtk::BUTTONS_OK_CANCEL);
+		
+		msg.set_title (_("Loading Error"));
+		msg.set_secondary_text (_("Click the OK button to try again."));
+		msg.set_position (Gtk::WIN_POS_CENTER);
+		pop_back_splash ();
+		msg.present ();
+
+		int response = msg.run ();
+
+		msg.hide ();
+
+		switch (response) {
+		case RESPONSE_CANCEL:
+			exit (1);
+		default:
+			break;
+		}
+		goto out;
+	}
+
 	catch (...) {
-		error << string_compose(_("Session \"%1 (snapshot %2)\" did not load successfully"), path, snap_name) << endmsg;
+
+		MessageDialog msg (string_compose(_("Session \"%1 (snapshot %2)\" did not load successfully"), path, snap_name),
+				   true,
+				   Gtk::MESSAGE_INFO,
+				   Gtk::BUTTONS_OK_CANCEL);
+		
+		msg.set_title (_("Loading Error"));
+		msg.set_secondary_text (_("Click the OK button to try again."));
+		msg.set_position (Gtk::WIN_POS_CENTER);
+		pop_back_splash ();
+		msg.present ();
+
+		int response = msg.run ();
+
+		msg.hide ();
+
+		switch (response) {
+		case RESPONSE_CANCEL:
+			exit (1);
+		default:
+			break;
+		}
 		goto out;
 	}
 
@@ -2299,7 +2497,6 @@ ARDOUR_UI::load_session (const string & path, const string & snap_name, string*
 		session->set_clean ();
 	}
 
-	editor->edit_cursor_position (true);
 	enable_screen_updates ();
 	flush_pending ();
 	retval = 0;
@@ -2309,7 +2506,7 @@ ARDOUR_UI::load_session (const string & path, const string & snap_name, string*
 }
 
 int
-ARDOUR_UI::build_session (const string & path, const string & snap_name, 
+ARDOUR_UI::build_session (const Glib::ustring& path, const Glib::ustring& snap_name, 
 			  uint32_t control_channels,
 			  uint32_t master_channels, 
 			  AutoConnectOption input_connect,
@@ -2345,6 +2542,7 @@ ARDOUR_UI::build_session (const string & path, const string & snap_name,
 	catch (...) {
 
 		MessageDialog msg (string_compose(_("Could not create session in \"%1\""), path));
+		pop_back_splash ();
 		msg.run ();
 		return -1;
 	}
@@ -2370,28 +2568,54 @@ ARDOUR_UI::show ()
 }
 
 void
-ARDOUR_UI::show_splash ()
+ARDOUR_UI::show_about ()
 {
 	if (about == 0) {
-		about = new About();
+		about = new About;
 		about->signal_response().connect(mem_fun (*this, &ARDOUR_UI::about_signal_response) );
 	}
-	about->present();
-	flush_pending ();
+
+	about->show_all ();
+}
+
+void
+ARDOUR_UI::hide_about ()
+{
+	if (about) {
+	        about->get_window()->set_cursor ();
+		about->hide ();
+	}
 }
 
 void
 ARDOUR_UI::about_signal_response(int response)
 {
-	hide_splash();
+	hide_about();
+}
+
+void
+ARDOUR_UI::show_splash ()
+{
+	if (splash == 0) {
+		try {
+			splash = new Splash;
+		} catch (...) {
+			return;
+		}
+	}
+
+	splash->show ();
+	splash->present ();
+	splash->queue_draw ();
+	splash->get_window()->process_updates (true);
+	flush_pending ();
 }
 
 void
 ARDOUR_UI::hide_splash ()
 {
-	if (about) {
-	        about->get_window()->set_cursor ();
-		about->hide();
+	if (splash) {
+		splash->hide();
 	}
 }
 
@@ -2742,7 +2966,9 @@ ARDOUR_UI::disk_speed_dialog_gone (int ignored_response, MessageDialog* msg)
 int
 ARDOUR_UI::pending_state_dialog ()
 {
-	ArdourDialog dialog ("pending state dialog");
+ 	HBox* hbox = new HBox();
+	Image* image = new Image (Stock::DIALOG_QUESTION, ICON_SIZE_DIALOG);
+	ArdourDialog dialog (_("Crash Recovery"), true);
 	Label  message (_("\
 This session appears to have been in\n\
 middle of recording when ardour or\n\
@@ -2751,14 +2977,18 @@ the computer was shutdown.\n\
 Ardour can recover any captured audio for\n\
 you, or it can ignore it. Please decide\n\
 what you would like to do.\n"));
-
-	dialog.get_vbox()->pack_start (message);
-	dialog.add_button (_("Recover from crash"), RESPONSE_ACCEPT);
+	image->set_alignment(ALIGN_CENTER, ALIGN_TOP);
+	hbox->pack_start (*image, PACK_EXPAND_WIDGET, 12);
+	hbox->pack_end (message, PACK_EXPAND_PADDING, 12);
+	dialog.get_vbox()->pack_start(*hbox, PACK_EXPAND_PADDING, 6);
 	dialog.add_button (_("Ignore crash data"), RESPONSE_REJECT);
-
+	dialog.add_button (_("Recover from crash"), RESPONSE_ACCEPT);
+	dialog.set_default_response (RESPONSE_ACCEPT);
 	dialog.set_position (WIN_POS_CENTER);
-	dialog.show_all ();
-	
+	message.show();
+	image->show();
+	hbox->show();
+
 	switch (dialog.run ()) {
 	case RESPONSE_ACCEPT:
 		return 1;
@@ -2766,6 +2996,38 @@ what you would like to do.\n"));
 		return 0;
 	}
 }
+
+int
+ARDOUR_UI::sr_mismatch_dialog (nframes_t desired, nframes_t actual)
+{
+ 	HBox* hbox = new HBox();
+	Image* image = new Image (Stock::DIALOG_QUESTION, ICON_SIZE_DIALOG);
+	ArdourDialog dialog (_("Sample Rate Mismatch"), true);
+	Label  message (string_compose (_("\
+This session was created with a sample rate of %1 Hz\n\
+\n\
+The audioengine is currently running at %2 Hz\n"), desired, actual));
+
+	image->set_alignment(ALIGN_CENTER, ALIGN_TOP);
+	hbox->pack_start (*image, PACK_EXPAND_WIDGET, 12);
+	hbox->pack_end (message, PACK_EXPAND_PADDING, 12);
+	dialog.get_vbox()->pack_start(*hbox, PACK_EXPAND_PADDING, 6);
+	dialog.add_button (_("Do not load session"), RESPONSE_REJECT);
+	dialog.add_button (_("Load session anyway"), RESPONSE_ACCEPT);
+	dialog.set_default_response (RESPONSE_ACCEPT);
+	dialog.set_position (WIN_POS_CENTER);
+	message.show();
+	image->show();
+	hbox->show();
+
+	switch (dialog.run ()) {
+	case RESPONSE_ACCEPT:
+		return 0;
+	default:
+		return 1;
+	}
+}
+
 	
 void
 ARDOUR_UI::disconnect_from_jack ()
@@ -2793,26 +3055,6 @@ ARDOUR_UI::reconnect_to_jack ()
 	}
 }
 
-int
-ARDOUR_UI::cmdline_new_session (string path)
-{
-	if (path[0] != '/') {
-		char buf[PATH_MAX+1];
-		string str;
-
-		getcwd (buf, sizeof (buf));
-		str = buf;
-		str += '/';
-		str += path;
-		path = str;
-	}
-
-	get_session_parameters (path, false, true);
-
-	_will_create_new_session_automatically = false; /* done it */
-	return FALSE; /* don't call it again */
-}
-
 void
 ARDOUR_UI::use_config ()
 {
@@ -2874,13 +3116,13 @@ void
 ARDOUR_UI::update_transport_clocks (nframes_t pos)
 {
 	if (Config->get_primary_clock_delta_edit_cursor()) {
-		primary_clock.set (pos, false, editor->edit_cursor_position(false), 1);
+		primary_clock.set (pos, false, editor->get_preferred_edit_position(), 1);
 	} else {
 		primary_clock.set (pos, 0, true);
 	}
 
 	if (Config->get_secondary_clock_delta_edit_cursor()) {
-		secondary_clock.set (pos, false, editor->edit_cursor_position(false), 2);
+		secondary_clock.set (pos, false, editor->get_preferred_edit_position(), 2);
 	} else {
 		secondary_clock.set (pos);
 	}
@@ -3067,8 +3309,3 @@ ARDOUR_UI::setup_profile ()
 	}
 }
 
-void
-ARDOUR_UI::audioengine_setup ()
-{
-}
-
