@@ -50,6 +50,7 @@
 #include <gtkmm/togglebutton.h>
 #include <gtkmm/treeview.h>
 #include <gtkmm/menubar.h>
+#include <gtkmm/textbuffer.h>
 #include <gtkmm/adjustment.h>
 #include <gtkmm2ext/gtk_ui.h>
 #include <gtkmm2ext/click_box.h>
@@ -67,14 +68,16 @@ class AudioClock;
 class PublicEditor;
 class Keyboard;
 class OptionEditor;
+class KeyEditor;
 class Mixer_UI;
 class ConnectionEditor;
 class RouteParams_UI;
 class About;
+class Splash;
 class AddRouteDialog;
-class NewSessionDialog;
 class LocationUI;
 class ThemeManager;
+class NewSessionDialog;
 
 namespace Gtkmm2ext {
 	class TearOff;
@@ -107,10 +110,13 @@ class ARDOUR_UI : public Gtkmm2ext::UI
 	
 	void show_splash ();
 	void hide_splash ();
+
+	void show_about ();
+	void hide_about ();
 	
-	int load_session (const string & path, const string & snapshot, string* mix_template = 0);
+	int load_session (const Glib::ustring & path, const Glib::ustring& snapshot, Glib::ustring mix_template = Glib::ustring());
 	bool session_loaded;
-	int build_session (const string & path, const string & snapshot, 
+	int build_session (const Glib::ustring& path, const Glib::ustring& snapshot, 
 			   uint32_t ctl_chns, 
 			   uint32_t master_chns,
 			   ARDOUR::AutoConnectOption input_connect,
@@ -130,9 +136,12 @@ class ARDOUR_UI : public Gtkmm2ext::UI
 		_will_create_new_session_automatically = yn;
 	}
 
-	bool new_session(std::string path = string());
-	gint cmdline_new_session (string path);
-	int  unload_session ();
+	bool get_session_parameters (bool have_engine = false, bool should_be_new = false);
+	void parse_cmdline_path (const Glib::ustring& cmdline_path, Glib::ustring& session_name, Glib::ustring& session_path, bool& existing_session);
+	int  load_cmdline_session (const Glib::ustring& session_name, const Glib::ustring& session_path, bool& existing_session);
+	int  build_session_from_nsd (const Glib::ustring& session_name, const Glib::ustring& session_path);
+	bool ask_about_loading_existing_session (const Glib::ustring& session_path);
+	int  unload_session (bool hide_stuff = false);
 	void close_session(); 
 
 	int  save_state_canfail (string state_name = "");
@@ -147,7 +156,10 @@ class ARDOUR_UI : public Gtkmm2ext::UI
 
 	PublicEditor&	  the_editor(){return *editor;}
 	Mixer_UI* the_mixer() { return mixer; }
-	
+
+	ARDOUR::AudioEngine& the_engine() const { return *engine; }
+
+	void toggle_key_editor ();
 	void toggle_location_window ();
 	void toggle_theme_manager ();
 	void toggle_big_clock_window ();
@@ -202,8 +214,8 @@ class ARDOUR_UI : public Gtkmm2ext::UI
 
 	void session_add_midi_track ();
 
-	void set_engine (ARDOUR::AudioEngine&);
-	gint start_engine ();
+	int  create_engine ();
+	void post_engine ();
 
 	gint exit_on_main_window_close (GdkEventAny *);
 
@@ -219,6 +231,8 @@ class ARDOUR_UI : public Gtkmm2ext::UI
 	void setup_profile ();
 	void setup_theme ();
 
+	void set_shuttle_fract (double);
+
   protected:
 	friend class PublicEditor;
 
@@ -262,8 +276,6 @@ class ARDOUR_UI : public Gtkmm2ext::UI
 	ARDOUR::AudioEngine                 *engine;
 	ARDOUR::Session                     *session;
 
-	bool check_audioengine();
-
 	Gtk::Tooltips          _tooltips;
 
 	void                     goto_editor_window ();
@@ -291,6 +303,7 @@ class ARDOUR_UI : public Gtkmm2ext::UI
 
 	static ARDOUR_UI *theArdourUI;
 
+	void backend_audio_error (bool we_set_params, Gtk::Window* toplevel = 0);
 	void startup ();
 	void shutdown ();
 
@@ -322,6 +335,7 @@ class ARDOUR_UI : public Gtkmm2ext::UI
 
 	void about_signal_response(int response);
 
+
   private:
 	Gtk::VBox     top_packer;
 
@@ -436,7 +450,6 @@ class ARDOUR_UI : public Gtkmm2ext::UI
 	gint shuttle_box_expose (GdkEventExpose*);
 	gint mouse_shuttle (double x, bool force);
 	void use_shuttle_fract (bool force);
-	void set_shuttle_fract (double);
 
 	bool   shuttle_grabbed;
 	double shuttle_fract;
@@ -464,6 +477,7 @@ class ARDOUR_UI : public Gtkmm2ext::UI
 	void solo_alert_toggle ();
 	void audition_alert_toggle ();
 
+	void big_clock_value_changed ();
 	void primary_clock_value_changed ();
 	void secondary_clock_value_changed ();
 
@@ -505,6 +519,8 @@ class ARDOUR_UI : public Gtkmm2ext::UI
 	Gtk::EventBox menu_bar_base;
 	Gtk::HBox     menu_hbox;
 
+	void use_menubar_as_top_menubar ();
+
 	void build_menu_bar ();
 	void build_control_surface_menu ();
 
@@ -558,7 +574,7 @@ class ARDOUR_UI : public Gtkmm2ext::UI
 	void transport_goto_end ();
 	void transport_stop ();
 	void transport_stop_and_forget_capture ();
-	void transport_record ();
+	void transport_record (bool roll);
 	void transport_roll ();
 	void transport_play_selection(); 
 	void transport_forward (int option);
@@ -604,6 +620,10 @@ class ARDOUR_UI : public Gtkmm2ext::UI
 	static UIConfiguration *ui_config;
 	ThemeManager *theme_manager;
 
+	/* Key bindings editor */
+
+	KeyEditor *key_editor;
+
 	/* Options window */
 	
 	OptionEditor *option_editor;
@@ -626,7 +646,10 @@ class ARDOUR_UI : public Gtkmm2ext::UI
 	void count_recenabled_streams (ARDOUR::Route&);
 
 	About* about;
+	Splash* splash;
+	void pop_back_splash ();
 	bool shown_flag;
+
 	/* cleanup */
 
 	Gtk::MenuItem *cleanup_item;
@@ -648,6 +671,7 @@ class ARDOUR_UI : public Gtkmm2ext::UI
 	void disk_underrun_handler ();
 
 	int pending_state_dialog ();
+	int sr_mismatch_dialog (nframes_t, nframes_t);
 	
 	void disconnect_from_jack ();
 	void reconnect_to_jack ();
@@ -682,6 +706,7 @@ class ARDOUR_UI : public Gtkmm2ext::UI
 	void set_remote_model (ARDOUR::RemoteModel);
 	void set_denormal_model (ARDOUR::DenormalModel);
 
+	void toggle_sync_order_keys ();
 	void toggle_StopPluginsWithTransport();
 	void toggle_DoNotRunPluginsWhileRecording();
 	void toggle_VerifyRemoveLastCapture();
@@ -695,16 +720,21 @@ class ARDOUR_UI : public Gtkmm2ext::UI
 	void toggle_RegionEquivalentsOverlap ();
 	void toggle_PrimaryClockDeltaEditCursor ();
 	void toggle_SecondaryClockDeltaEditCursor ();
+	void toggle_only_copy_imported_files ();
+	void toggle_ShowTrackMeters ();
+	void toggle_use_narrow_ms();
 
 	void mtc_port_changed ();
 	void map_solo_model ();
 	void map_monitor_model ();
 	void map_denormal_model ();
+	void map_denormal_protection ();
 	void map_remote_model ();
 	void map_file_header_format ();
 	void map_file_data_format ();
 	void map_input_auto_connect ();
 	void map_output_auto_connect ();
+	void map_only_copy_imported_files ();
 	void parameter_changed (const char*);
 
 	void set_meter_hold (ARDOUR::MeterHold);
@@ -721,7 +751,20 @@ class ARDOUR_UI : public Gtkmm2ext::UI
 	void no_memory_warning ();
 	void check_memory_locking ();
 
+	bool check_audioengine();
 	void audioengine_setup ();
+
+	void display_message (const char *prefix, gint prefix_len, 
+			      Glib::RefPtr<Gtk::TextBuffer::Tag> ptag, Glib::RefPtr<Gtk::TextBuffer::Tag> mtag, const char *msg);
+	Gtk::Label status_bar_label;
+	Gtk::ToggleButton error_log_button;
+	
+	void loading_message (const std::string& msg);
+	void end_loading_messages ();
+
+	void platform_specific ();
+	void platform_setup ();
+	void fontconfig_dialog ();
 };
 
 #endif /* __ardour_gui_h__ */
