@@ -30,6 +30,7 @@
 #include "ardour_ui.h"
 #include "actions.h"
 #include "gui_thread.h"
+#include "public_editor.h"
 
 #include "i18n.h"
 
@@ -506,6 +507,18 @@ ARDOUR_UI::toggle_SecondaryClockDeltaEditCursor()
 	ActionManager::toggle_config_state ("options", "SecondaryClockDeltaEditCursor", &Configuration::set_secondary_clock_delta_edit_cursor, &Configuration::get_secondary_clock_delta_edit_cursor);
 }
 
+void
+ARDOUR_UI::toggle_ShowTrackMeters()
+{
+	ActionManager::toggle_config_state ("options", "ShowTrackMeters", &Configuration::set_show_track_meters, &Configuration::get_show_track_meters);
+}
+
+void
+ARDOUR_UI::toggle_use_narrow_ms()
+{
+	ActionManager::toggle_config_state ("options", "DefaultNarrowMS", &Configuration::set_default_narrow_ms, &Configuration::get_default_narrow_ms);
+}
+
 void
 ARDOUR_UI::mtc_port_changed ()
 {
@@ -1107,6 +1120,11 @@ ARDOUR_UI::parameter_changed (const char* parameter_name)
 		ActionManager::map_some_state ("options",  "SecondaryClockDeltaEditCursor", &Configuration::get_secondary_clock_delta_edit_cursor);
 	} else if (PARAM_IS ("only-copy-imported-files")) {
 		map_only_copy_imported_files ();
+	} else if (PARAM_IS ("show-track-meters")) {
+		ActionManager::map_some_state ("options",  "ShowTrackMeters", &Configuration::get_show_track_meters);
+		editor->toggle_meter_updating();
+	} else if (PARAM_IS ("default-narrow_ms")) {
+		ActionManager::map_some_state ("options",  "DefaultNarrowMS", &Configuration::get_default_narrow_ms);
 	}
 			   
 
