@@ -1,7 +1,7 @@
 /*
     Copyright (C) 2001-2006 Paul Davis 
 
-    This program is free software; you can redistribute it and/or modify
+    This program is free software; you can r>edistribute it and/or modify
     it under the terms of the GNU General Public License as published by
     the Free Software Foundation; either version 2 of the License, or
     (at your option) any later version.
@@ -51,7 +51,7 @@
 
 #include "i18n.h"
 
-#define MUTED_ALPHA 0x50
+#define MUTED_ALPHA 10
 
 using namespace sigc;
 using namespace ARDOUR;
@@ -122,7 +122,7 @@ AudioRegionView::init (Gdk::Color& basic_color, bool wfd)
 	// FIXME: Some redundancy here with RegionView::init.  Need to figure out
 	// where order is important and where it isn't...
 	
-	RegionView::init (basic_color, true);
+	RegionView::init (basic_color, wfd);
 	
 	XMLNode *node;
 
@@ -395,6 +395,8 @@ AudioRegionView::region_muted ()
 void
 AudioRegionView::set_height (gdouble height)
 {
+	RegionView::set_height (height);
+
 	uint32_t wcnt = waves.size();
 
 	// FIXME: ick
@@ -496,7 +498,7 @@ AudioRegionView::reset_fade_in_shape_width (nframes_t width)
 	} else {
 		handle_center = 3.0;
 	}
-	
+
 	fade_in_handle->property_x1() =  handle_center - 3.0;
 	fade_in_handle->property_x2() =  handle_center + 3.0;
 	
@@ -761,6 +763,7 @@ AudioRegionView::set_envelope_visible (bool yn)
 void
 AudioRegionView::create_waves ()
 {
+	//cerr << "AudioRegionView::create_waves() called on " << this << endl;//DEBUG
 	RouteTimeAxisView& atv (*(dynamic_cast<RouteTimeAxisView*>(&trackview))); // ick
 
 	if (!atv.get_diskstream()) {
@@ -783,23 +786,25 @@ AudioRegionView::create_waves ()
 		wave_caches.push_back (WaveView::create_cache ());
 
 		if (wait_for_data) {
-
 			if (audio_region()->source(n)->peaks_ready (bind (mem_fun(*this, &AudioRegionView::peaks_ready_handler), n), data_ready_connection)) {
 				create_one_wave (n, true);
 			} else {
 				// we'll get a PeaksReady signal from the source in the future
 				// and will call create_one_wave(n) then.
 			}
-
+			
 		} else {
+
 			create_one_wave (n, true);
 		}
+
 	}
 }
 
 void
 AudioRegionView::create_one_wave (uint32_t which, bool direct)
 {
+	//cerr << "AudioRegionView::create_one_wave() called which: " << which << " this: " << this << endl;//DEBUG
 	RouteTimeAxisView& atv (*(dynamic_cast<RouteTimeAxisView*>(&trackview))); // ick
 	uint32_t nchans = atv.get_diskstream()->n_channels();
 	uint32_t n;
@@ -829,6 +834,7 @@ AudioRegionView::create_one_wave (uint32_t which, bool direct)
 	wave->property_samples_per_unit() =  samples_per_unit;
 	wave->property_amplitude_above_axis() =  _amplitude_above_axis;
 	wave->property_wave_color() = _region->muted() ? UINT_RGBA_CHANGE_A(ARDOUR_UI::config()->canvasvar_WaveForm.get(), MUTED_ALPHA) : ARDOUR_UI::config()->canvasvar_WaveForm.get();
+	wave->property_fill_color() = ARDOUR_UI::config()->canvasvar_WaveFormFill.get();
 	wave->property_clip_color() = ARDOUR_UI::config()->canvasvar_WaveFormClip.get();
 	wave->property_zero_color() = ARDOUR_UI::config()->canvasvar_ZeroLine.get();
 	wave->property_region_start() = _region->start();
@@ -883,6 +889,7 @@ void
 AudioRegionView::peaks_ready_handler (uint32_t which)
 {
 	Gtkmm2ext::UI::instance()->call_slot (bind (mem_fun(*this, &AudioRegionView::create_one_wave), which, false));
+	// cerr << "AudioRegionView::peaks_ready_handler() called on " << which << " this: " << this << endl;
 }
 
 void
@@ -1076,6 +1083,7 @@ AudioRegionView::add_ghost (AutomationTimeAxisView& atv)
 		wave->property_samples_per_unit() =  samples_per_unit;
 		wave->property_amplitude_above_axis() =  _amplitude_above_axis;
 		wave->property_wave_color() = ARDOUR_UI::config()->canvasvar_GhostTrackWave.get();
+		wave->property_fill_color() = ARDOUR_UI::config()->canvasvar_GhostTrackWave.get();
 		wave->property_clip_color() = ARDOUR_UI::config()->canvasvar_GhostTrackWaveClip.get();
 		wave->property_zero_color() = ARDOUR_UI::config()->canvasvar_GhostTrackZeroLine.get();
 		wave->property_region_start() = _region->start();
@@ -1163,3 +1171,44 @@ AudioRegionView::color_handler ()
 	envelope_active_changed();
 }
 
+void
+AudioRegionView::set_frame_color ()
+{
+	if (!frame) {
+		return;
+	}
+
+	if (_region->opaque()) {
+		fill_opacity = 130;
+	} else {
+		fill_opacity = 0;
+	}
+
+	uint32_t r,g,b,a;
+	
+	if (_selected && should_show_selection) {
+		UINT_TO_RGBA(ARDOUR_UI::config()->canvasvar_SelectedFrameBase.get(), &r, &g, &b, &a);
+		frame->property_fill_color_rgba() = RGBA_TO_UINT(r, g, b, fill_opacity ? fill_opacity : a);
+
+		for (vector<ArdourCanvas::WaveView*>::iterator w = waves.begin(); w != waves.end(); ++w) {
+			if (_region->muted()) {
+				(*w)->property_wave_color() = UINT_RGBA_CHANGE_A(ARDOUR_UI::config()->canvasvar_SelectedWaveForm.get(), MUTED_ALPHA);
+			} else {
+				(*w)->property_wave_color() = ARDOUR_UI::config()->canvasvar_SelectedWaveForm.get();
+				(*w)->property_fill_color() = ARDOUR_UI::config()->canvasvar_SelectedWaveFormFill.get();
+			}
+		}
+	} else {
+		UINT_TO_RGBA(ARDOUR_UI::config()->canvasvar_FrameBase.get(), &r, &g, &b, &a);
+		frame->property_fill_color_rgba() = RGBA_TO_UINT(r, g, b, fill_opacity ? fill_opacity : a);
+
+		for (vector<ArdourCanvas::WaveView*>::iterator w = waves.begin(); w != waves.end(); ++w) {
+			if (_region->muted()) {
+				(*w)->property_wave_color() = UINT_RGBA_CHANGE_A(ARDOUR_UI::config()->canvasvar_WaveForm.get(), MUTED_ALPHA);
+			} else {
+				(*w)->property_wave_color() = ARDOUR_UI::config()->canvasvar_WaveForm.get();
+				(*w)->property_fill_color() = ARDOUR_UI::config()->canvasvar_WaveFormFill.get();
+			}
+		}
+	}
+}
