@@ -18,7 +18,6 @@
 */
 
 /* Changing this order will break the menu */
-SNAPTYPE(SnapToFrame)
 SNAPTYPE(SnapToCDFrame)
 SNAPTYPE(SnapToSMPTEFrame)
 SNAPTYPE(SnapToSMPTESeconds)
@@ -33,13 +32,13 @@ SNAPTYPE(SnapToAThirdBeat)
 SNAPTYPE(SnapToBeat)
 SNAPTYPE(SnapToBar)
 SNAPTYPE(SnapToMark)
-SNAPTYPE(SnapToEditCursor)
 SNAPTYPE(SnapToRegionStart)
 SNAPTYPE(SnapToRegionEnd)
 SNAPTYPE(SnapToRegionSync)
 SNAPTYPE(SnapToRegionBoundary)
 
 /* Changing this order will break the menu */
+SNAPMODE(SnapOff)
 SNAPMODE(SnapNormal)
 SNAPMODE(SnapMagnetic)
 
@@ -66,6 +65,7 @@ ZOOMFOCUS(ZoomFocusLeft)
 ZOOMFOCUS(ZoomFocusRight)
 ZOOMFOCUS(ZoomFocusCenter)
 ZOOMFOCUS(ZoomFocusPlayhead)
+ZOOMFOCUS(ZoomFocusMouse)
 ZOOMFOCUS(ZoomFocusEdit)
 
 DISPLAYCONTROL(FollowPlayhead)
@@ -81,7 +81,7 @@ IMPORTMODE(ImportAsTapeTrack=3)
 
 // if this is changed, remember to update the string table in sfdb_ui.cc
 IMPORTPOSITION(ImportAtTimestamp=0)
-IMPORTPOSITION(ImportAtEditCursor=1)
+IMPORTPOSITION(ImportAtEditPoint=1)
 IMPORTPOSITION(ImportAtPlayhead=2)
 IMPORTPOSITION(ImportAtStart=3)
 
@@ -91,3 +91,6 @@ IMPORTDISPOSITION(ImportMergeFiles=1)
 IMPORTDISPOSITION(ImportSerializeFiles=2)
 IMPORTDISPOSITION(ImportDistinctChannels=3)
 
+EDITPOINT(EditAtPlayhead)
+EDITPOINT(EditAtSelectedMarker)
+EDITPOINT(EditAtMouse)
