@@ -46,6 +46,7 @@
 #include <pbd/stateful.h>
 #include <ardour/session.h>
 #include <ardour/tempo.h>
+#include <ardour/stretch.h>
 #include <ardour/location.h>
 #include <ardour/audioregion.h>
 
@@ -104,6 +105,7 @@ class StreamView;
 class AudioStreamView;
 class ControlPoint;
 class SoundFileOmega;
+class RhythmFerret;
 #ifdef FFT_ANALYSIS
 class AnalysisWindow;
 #endif
@@ -133,6 +135,8 @@ class Editor : public PublicEditor
 		return (nframes_t) floor (canvas_width * frames_per_unit);
 	}
 
+	void cycle_snap_mode ();
+	void cycle_snap_choice ();
 	void set_snap_to (Editing::SnapType);
 	void set_snap_mode (Editing::SnapMode);
 	void set_snap_threshold (double pixel_distance) {snap_threshold = pixel_distance;}
@@ -157,6 +161,10 @@ class Editor : public PublicEditor
 	void hide_a_region (boost::shared_ptr<ARDOUR::Region>);
 	void remove_a_region (boost::shared_ptr<ARDOUR::Region>);
 
+#ifdef USE_RUBBERBAND
+	std::vector<std::string> rb_opt_strings;
+#endif
+
 	/* option editor-access */
 
 	void set_show_waveforms (bool yn);
@@ -168,6 +176,7 @@ class Editor : public PublicEditor
 	/* things that need to be public to be used in the main menubar */
 
 	void new_region_from_selection ();
+	void separate_regions_between (const TimeSelection&);
 	void separate_region_from_selection ();
 	void separate_regions_using_location (ARDOUR::Location&);
 	void toggle_playback (bool with_abort);
@@ -175,15 +184,15 @@ class Editor : public PublicEditor
 
 	/* undo related */
 
-	nframes_t unit_to_frame (double unit) {
+	nframes_t unit_to_frame (double unit) const {
 		return (nframes_t) rint (unit * frames_per_unit);
 	}
 	
-	double frame_to_unit (nframes_t frame) {
+	double frame_to_unit (nframes_t frame) const {
 		return rint ((double) frame / (double) frames_per_unit);
 	}
 
-	double frame_to_unit (double frame) {
+	double frame_to_unit (double frame) const {
 		return rint (frame / frames_per_unit);
 	}
 
@@ -194,7 +203,7 @@ class Editor : public PublicEditor
 	   xscroll_adjustment.  
 	*/
 
-	nframes_t pixel_to_frame (double pixel) {
+	nframes64_t pixel_to_frame (double pixel) const {
 		
 		/* pixel can be less than zero when motion events
 		   are processed. since we've already run the world->canvas
@@ -209,10 +218,12 @@ class Editor : public PublicEditor
 		}
 	}
 
-	gulong frame_to_pixel (nframes_t frame) {
+	gulong frame_to_pixel (nframes64_t frame) const {
 		return (gulong) rint ((frame / (frames_per_unit *  GNOME_CANVAS(track_canvas.gobj())->pixels_per_unit)));
 	}
 
+	void flush_canvas ();
+
 	/* selection */
 
 	Selection& get_selection() const { return *selection; }
@@ -225,6 +236,7 @@ class Editor : public PublicEditor
 	void select_all (Selection::Operation op);
 	void invert_selection_in_track ();
 	void invert_selection ();
+	void deselect_all ();
 
 	/* tempo */
 
@@ -250,7 +262,7 @@ class Editor : public PublicEditor
 
 	void      set_zoom_focus (Editing::ZoomFocus);
 	Editing::ZoomFocus get_zoom_focus () const { return zoom_focus; }
-	gdouble   get_current_zoom () { return frames_per_unit; }
+	double   get_current_zoom () const { return frames_per_unit; }
 
 	void temporal_zoom_step (bool coarser);
 
@@ -258,13 +270,11 @@ class Editor : public PublicEditor
 
 	PlaylistSelector& playlist_selector() const;
 	void route_name_changed (TimeAxisView *);
-	gdouble        frames_per_unit;
-	nframes_t leftmost_frame;
 	void clear_playlist (boost::shared_ptr<ARDOUR::Playlist>);
 
-	void new_playlists ();
-	void copy_playlists ();
-	void clear_playlists ();
+	void new_playlists (TimeAxisView*);
+	void copy_playlists (TimeAxisView*);
+	void clear_playlists (TimeAxisView*);
 
 	TrackViewList* get_valid_views (TimeAxisView*, ARDOUR::RouteGroup* grp = 0);
 
@@ -277,8 +287,8 @@ class Editor : public PublicEditor
 
 	/* nudge is initiated by transport controls owned by ARDOUR_UI */
 
-	void nudge_forward (bool next);
-	void nudge_backward (bool next);
+	void nudge_forward (bool next, bool force_playhead);
+	void nudge_backward (bool next, bool force_playhead);
 
 	/* nudge initiated from context menu */
 
@@ -295,6 +305,7 @@ class Editor : public PublicEditor
 	void toggle_waveform_visibility ();
 	void toggle_waveforms_while_recording ();
 	void toggle_measure_visibility ();
+	void toggle_logo_visibility ();
 
 	/* SMPTE timecode & video sync */
 
@@ -318,6 +329,8 @@ class Editor : public PublicEditor
 	/* layers */
 	void set_layer_model (ARDOUR::LayerModel);
 	void update_layering_model ();
+	
+	void toggle_link_region_and_track_selection ();
 
 	/* redirect shared ops menu. caller must free returned menu */
 
@@ -342,10 +355,15 @@ class Editor : public PublicEditor
 	void reset_zoom (double);
 	void reposition_and_zoom (nframes_t, double);
 
-	nframes_t edit_cursor_position(bool);
+	nframes64_t get_preferred_edit_position (bool ignore_playhead = false);
+
 	bool update_mouse_speed ();
 	bool decelerate_mouse_speed ();
 
+	void toggle_meter_updating();
+
+	void show_rhythm_ferret();
+
   protected:
 	void map_transport_state ();
 	void map_position_change (nframes_t);
@@ -362,7 +380,22 @@ class Editor : public PublicEditor
 
 	PlaylistSelector* _playlist_selector;
 
-	void          set_frames_per_unit (double);
+	struct VisualState {
+	    double    frames_per_unit;
+	    nframes_t leftmost_frame;
+	    Editing::ZoomFocus zoom_focus;
+	};
+	
+	VisualState last_visual_state;
+
+	nframes_t   leftmost_frame;
+	double      frames_per_unit;
+	Editing::ZoomFocus zoom_focus;
+
+	void use_visual_state (const VisualState&);
+	void set_frames_per_unit (double);
+	void swap_visual_state ();
+	void post_zoom ();
 
 	Editing::MouseMode mouse_mode;
 
@@ -390,6 +423,7 @@ class Editor : public PublicEditor
 	void location_gone (ARDOUR::Location *);
 	void remove_marker (ArdourCanvas::Item&, GdkEvent*);
 	gint really_remove_marker (ARDOUR::Location* loc);
+	void goto_nth_marker (int nth);
 
 	uint32_t location_marker_color;
 	uint32_t location_range_color;
@@ -400,7 +434,10 @@ class Editor : public PublicEditor
 	struct LocationMarkers {
 	    Marker* start;
 	    Marker* end;
+	    bool    valid;
 
+	    LocationMarkers () : start(0), end(0), valid (true) {}
+	    
 	    ~LocationMarkers ();
 
 	    void hide();
@@ -410,29 +447,32 @@ class Editor : public PublicEditor
 	    void set_color_rgba (uint32_t);
 	};
 
-	LocationMarkers  *find_location_markers (ARDOUR::Location *);
-	ARDOUR::Location* find_location_from_marker (Marker *, bool& is_start);
+	LocationMarkers  *find_location_markers (ARDOUR::Location *) const;
+	ARDOUR::Location* find_location_from_marker (Marker *, bool& is_start) const;
+	Marker* entered_marker;
 
 	typedef std::map<ARDOUR::Location*,LocationMarkers *> LocationMarkerMap;
 	LocationMarkerMap location_markers;
 
 	void hide_marker (ArdourCanvas::Item*, GdkEvent*);
 	void clear_marker_display ();
-	void mouse_add_new_marker (nframes_t where);
+	void mouse_add_new_marker (nframes_t where, bool is_cd=false);
+	void update_cd_marker_display ();
+	void ensure_cd_marker_updated (LocationMarkers * lam, ARDOUR::Location * location);
 
 	TimeAxisView*      clicked_trackview;
 	AudioTimeAxisView* clicked_audio_trackview;
 	RegionView*        clicked_regionview;
-	RegionView*        latest_regionview;
+	RegionSelection    latest_regionviews;
 	uint32_t           clicked_selection;
 	CrossfadeView*     clicked_crossfadeview;
 	ControlPoint*      clicked_control_point;
 
-	void sort_track_selection ();
+	void sort_track_selection (TrackSelection* sel = 0);
 
 	void get_relevant_audio_tracks (std::set<AudioTimeAxisView*>& relevant_tracks);
 	void get_equivalent_regions (RegionView* rv, std::vector<RegionView*>&);
-	void mapover_audio_tracks (sigc::slot<void,AudioTimeAxisView&,uint32_t> sl);
+	void mapover_audio_tracks (sigc::slot<void,AudioTimeAxisView&,uint32_t> sl, TimeAxisView*);
 
 	/* functions to be passed to mapover_audio_tracks(), possibly with sigc::bind()-supplied arguments */
 
@@ -448,11 +488,12 @@ class Editor : public PublicEditor
 
 	void catch_vanishing_regionview (RegionView *);
 
-	bool set_selected_track (TimeAxisView&, Selection::Operation op = Selection::Set, bool no_remove=false);
+	void set_selected_track (TimeAxisView&, Selection::Operation op = Selection::Set, bool no_remove=false);
 	void select_all_tracks ();
 
 	bool set_selected_control_point_from_click (Selection::Operation op = Selection::Set, bool no_remove=false);
-	bool set_selected_track_from_click (bool press, Selection::Operation op = Selection::Set, bool no_remove=false);
+	void set_selected_track_from_click (bool press, Selection::Operation op = Selection::Set, bool no_remove=false);
+	void set_selected_track_as_side_effect (bool force = false);
 	bool set_selected_regionview_from_click (bool press, Selection::Operation op = Selection::Set, bool no_track_remove=false);
 
 	void set_selected_regionview_from_region_list (boost::shared_ptr<ARDOUR::Region> region, Selection::Operation op = Selection::Set);
@@ -525,6 +566,7 @@ class Editor : public PublicEditor
 	ArdourCanvas::Group*      marker_group;
 	ArdourCanvas::Group*      range_marker_group;
 	ArdourCanvas::Group*      transport_marker_group;
+	ArdourCanvas::Group*      cd_marker_group;
 	
 	enum {
 		ruler_metric_smpte = 0,
@@ -537,15 +579,17 @@ class Editor : public PublicEditor
 		ruler_time_marker = 6,
 		ruler_time_range_marker = 7,
 		ruler_time_transport_marker = 8,
+		ruler_time_cd_marker = 9,
 	};
 
 	static GtkCustomMetric ruler_metrics[4];
-	bool                   ruler_shown[9];
+	bool                   ruler_shown[10];
 	bool                   no_ruler_shown_update;
 	
 	gint ruler_button_press (GdkEventButton*);
 	gint ruler_button_release (GdkEventButton*);
 	gint ruler_mouse_motion (GdkEventMotion*);
+	bool ruler_scroll (GdkEventScroll* event);
 
 	gint ruler_pressed_button;
 	Gtk::Widget * ruler_grabbed_widget;
@@ -591,6 +635,7 @@ class Editor : public PublicEditor
 	ArdourCanvas::SimpleRect* marker_bar;
 	ArdourCanvas::SimpleRect* range_marker_bar;
 	ArdourCanvas::SimpleRect* transport_marker_bar;
+	ArdourCanvas::SimpleRect* cd_marker_bar;
 
 	
 	ArdourCanvas::SimpleLine* tempo_line;
@@ -598,6 +643,7 @@ class Editor : public PublicEditor
 	ArdourCanvas::SimpleLine* marker_line;
 	ArdourCanvas::SimpleLine* range_marker_line;
 	ArdourCanvas::SimpleLine* transport_marker_line;
+	ArdourCanvas::SimpleLine* cd_marker_line;
 
 	Gtk::Label  minsec_label;
 	Gtk::Label  bbt_label;
@@ -608,6 +654,7 @@ class Editor : public PublicEditor
 	Gtk::Label  mark_label;
 	Gtk::Label  range_mark_label;
 	Gtk::Label  transport_mark_label;
+	Gtk::Label  cd_mark_label;
 	
 
 	Gtk::VBox          time_button_vbox;
@@ -633,18 +680,33 @@ class Editor : public PublicEditor
 			      */
 
 	Cursor* playhead_cursor;
-	Cursor* edit_cursor;
 	ArdourCanvas::Group* cursor_group;
 
+	void    cursor_to_region_boundary (Cursor*, int32_t dir);
+	void    cursor_to_next_region_boundary (Cursor*);
+	void    cursor_to_previous_region_boundary (Cursor*);
 	void    cursor_to_next_region_point (Cursor*, ARDOUR::RegionPoint);
 	void    cursor_to_previous_region_point (Cursor*, ARDOUR::RegionPoint);
 	void    cursor_to_region_point (Cursor*, ARDOUR::RegionPoint, int32_t dir);
 	void    cursor_to_selection_start (Cursor *);
 	void    cursor_to_selection_end   (Cursor *);
+
+	void    selected_marker_to_region_boundary (int32_t dir);
+	void    selected_marker_to_next_region_boundary ();
+	void    selected_marker_to_previous_region_boundary ();
+	void    selected_marker_to_next_region_point (ARDOUR::RegionPoint);
+	void    selected_marker_to_previous_region_point (ARDOUR::RegionPoint);
+	void    selected_marker_to_region_point (ARDOUR::RegionPoint, int32_t dir);
+	void    selected_marker_to_selection_start ();
+	void    selected_marker_to_selection_end   ();
+
 	void    select_all_selectables_using_cursor (Cursor *, bool);
-	void    select_all_selectables_between_cursors (Cursor *, Cursor *);
+	void    select_all_selectables_using_edit (bool);
+	void    select_all_selectables_between (bool within);
+	void    select_range_between ();
 
 	boost::shared_ptr<ARDOUR::Region> find_next_region (nframes_t, ARDOUR::RegionPoint, int32_t dir, TrackViewList&, TimeAxisView ** = 0);
+	nframes64_t find_next_region_boundary (nframes64_t, int32_t dir, const TrackViewList&);
 
 	vector<nframes_t> region_boundary_cache;
 	void build_region_boundary_cache ();
@@ -696,6 +758,7 @@ class Editor : public PublicEditor
 
 	void tie_vertical_scrolling ();
 	void canvas_horizontally_scrolled ();
+	void canvas_scroll_to (nframes64_t);
 
 	struct VisualChange {
 	    enum Type { 
@@ -736,13 +799,15 @@ class Editor : public PublicEditor
 	    
 	RegionListDisplayModelColumns          region_list_columns;
 	Gtkmm2ext::DnDTreeView<boost::shared_ptr<ARDOUR::Region> > region_list_display;
-
+	
 	Glib::RefPtr<Gtk::TreeStore>           region_list_model;
 	Glib::RefPtr<Gtk::ToggleAction>        toggle_full_region_list_action;
 	Glib::RefPtr<Gtk::ToggleAction>        toggle_show_auto_regions_action;
 
 	void region_list_selection_changed ();
 	bool region_list_selection_filter (const Glib::RefPtr<Gtk::TreeModel>& model, const Gtk::TreeModel::Path& path, bool yn);
+	void region_name_edit (const Glib::ustring&, const Glib::ustring&);
+	void get_regions_corresponding_to (boost::shared_ptr<ARDOUR::Region> region, std::vector<RegionView*>& regions);
 
 	Gtk::Menu          *region_list_menu;
 	Gtk::ScrolledWindow region_list_scroller;
@@ -835,6 +900,7 @@ class Editor : public PublicEditor
 	static Gdk::Cursor* speaker_cursor;
 	static Gdk::Cursor* wait_cursor;
 	static Gdk::Cursor* timebar_cursor;
+	static Gdk::Cursor* transparent_cursor;
 
 	static void build_cursors ();
 
@@ -886,6 +952,7 @@ class Editor : public PublicEditor
 	void toggle_region_mute ();
 	void toggle_region_lock ();
 	void toggle_region_opaque ();
+	void set_region_lock_style (ARDOUR::Region::PositionLockStyle);
 	void raise_region ();
 	void raise_region_to_top ();
 	void lower_region ();
@@ -893,17 +960,21 @@ class Editor : public PublicEditor
 	void split_region ();
 	void split_region_at (nframes_t);
 	void split_regions_at (nframes_t, RegionSelection&);
+	void split_region_at_transients ();
+	void split_region_at_points (boost::shared_ptr<ARDOUR::Region>, ARDOUR::AnalysisFeatureList&);
 	void crop_region_to_selection ();
-	void set_a_regions_sync_position (boost::shared_ptr<ARDOUR::Region>, nframes_t);
-	void set_region_sync_from_edit_cursor ();
+	void crop_region_to (nframes_t start, nframes_t end);
+	void set_sync_point (nframes64_t, const RegionSelection&);
+	void set_region_sync_from_edit_point ();
 	void remove_region_sync();
-	void align_selection (ARDOUR::RegionPoint, nframes_t position);
-	void align_selection_relative (ARDOUR::RegionPoint point, nframes_t position);
+	void align_selection (ARDOUR::RegionPoint, nframes_t position, const RegionSelection&);
+	void align_selection_relative (ARDOUR::RegionPoint point, nframes_t position, const RegionSelection&);
 	void align_region (boost::shared_ptr<ARDOUR::Region>, ARDOUR::RegionPoint point, nframes_t position);
 	void align_region_internal (boost::shared_ptr<ARDOUR::Region>, ARDOUR::RegionPoint point, nframes_t position);
 	void remove_clicked_region ();
 	void destroy_clicked_region ();
 	void edit_region ();
+	void rename_region ();
 	void duplicate_some_regions (RegionSelection&, float times);
 	void duplicate_selection (float times);
 	void region_fill_selection ();
@@ -915,6 +986,14 @@ class Editor : public PublicEditor
 	void reverse_region ();
 	void normalize_region ();
 	void denormalize_region ();
+	void adjust_region_scale_amplitude (bool up);
+
+	void tab_to_transient (bool forward);
+
+	void use_region_as_bar ();
+	void use_range_as_bar ();
+
+	void define_one_bar (nframes64_t start, nframes64_t end);
 
 	void audition_region_from_region_list ();
 	void hide_region_from_region_list ();
@@ -926,6 +1005,8 @@ class Editor : public PublicEditor
 
 	void reset_focus ();
 
+	void split ();
+
 	void cut ();
 	void copy ();
 	void paste (float times);
@@ -933,28 +1014,24 @@ class Editor : public PublicEditor
 	int  get_prefix (float&, bool&);
 
 	void keyboard_paste ();
-	void keyboard_duplicate_region ();
-	void keyboard_duplicate_selection ();
 	void keyboard_insert_region_list_selection ();
 
 	void region_from_selection ();
 	void create_region_from_selection (std::vector<boost::shared_ptr<ARDOUR::AudioRegion> >&);
 
-	bool region_renamed;
-	void rename_region ();
-	void rename_region_finished (bool);
-
 	void play_from_start ();
-	void play_from_edit_cursor ();
+	void play_from_edit_point ();
+	void play_from_edit_point_and_return ();
 	void play_selected_region ();
-	void audition_selected_region ();
+	void play_edit_range ();
 	void loop_selected_region ();
 	void play_location (ARDOUR::Location&);
 	void loop_location (ARDOUR::Location&);
 
-	Editing::ZoomFocus zoom_focus;
-
 	void temporal_zoom_selection ();
+	void temporal_zoom_region ();
+	void toggle_zoom_region ();
+	bool zoomed_to_region;
 	void temporal_zoom_session ();
 	void temporal_zoom (gdouble scale);
 	void temporal_zoom_by_frame (nframes_t start, nframes_t end, const string & op);
@@ -1024,7 +1101,7 @@ class Editor : public PublicEditor
 	void move_to_end ();
 	void goto_frame ();
 	void center_playhead ();
-	void center_edit_cursor ();
+	void center_edit_point ();
 	void edit_cursor_backward ();
 	void edit_cursor_forward ();
 	void playhead_backward ();
@@ -1058,15 +1135,22 @@ class Editor : public PublicEditor
 	void set_selection_from_loop ();
 	void set_selection_from_audio_region ();
 
+	void add_location_mark (nframes64_t where);
 	void add_location_from_audio_region ();
 	void add_location_from_selection ();
 	void set_loop_from_selection (bool play);
 	void set_punch_from_selection ();
+	void set_punch_from_region ();
+
+	void set_loop_from_edit_range (bool play);
+	void set_loop_from_region (bool play);
+	void set_punch_from_edit_range ();
 
 	void set_loop_range (nframes_t start, nframes_t end, std::string cmd);
 	void set_punch_range (nframes_t start, nframes_t end, std::string cmd);
 
 	void add_location_from_playhead_cursor ();
+	bool select_new_marker;
 
 	void reverse_selection ();
 	void edit_envelope ();
@@ -1075,16 +1159,10 @@ class Editor : public PublicEditor
 	void stop_scrolling ();
 
 	bool _scrubbing;
-	bool have_full_mouse_speed;
-	nframes64_t last_scrub_frame;
-	double last_scrub_time;
-	int mouse_speed_update;
-	double mouse_direction;
-	double compute_mouse_speed ();
-	void add_mouse_speed (double, double);
-	double* mouse_speed;
-	size_t mouse_speed_entries;
-	size_t mouse_speed_size;
+	double last_scrub_x;
+	int scrubbing_direction;
+	int scrub_reversals;
+	int scrub_reverse_distance;
 
 	void keyboard_selection_begin ();
 	void keyboard_selection_finish (bool add);
@@ -1122,15 +1200,23 @@ class Editor : public PublicEditor
 
 	void set_fade_in_shape (ARDOUR::AudioRegion::FadeShape);
 	void set_fade_out_shape (ARDOUR::AudioRegion::FadeShape);
-
+	
+	void set_fade_length (bool in);
+	void toggle_fade_active (bool in);
 	void set_fade_in_active (bool);
 	void set_fade_out_active (bool);
 	
 	std::set<boost::shared_ptr<ARDOUR::Playlist> > motion_frozen_playlists;
+	RegionSelection pre_drag_region_selection;
 	void region_drag_motion_callback (ArdourCanvas::Item*, GdkEvent*);
 	void region_drag_finished_callback (ArdourCanvas::Item*, GdkEvent*);
+	bool check_region_drag_possible (AudioTimeAxisView**);
+	void possibly_copy_regions_during_grab (GdkEvent*);
+	void region_drag_splice_motion_callback (ArdourCanvas::Item*, GdkEvent*);
+	void region_drag_splice_finished_callback (ArdourCanvas::Item*, GdkEvent*);
 
 	bool _dragging_playhead;
+	bool _dragging_edit_point;
 
 	void cursor_drag_motion_callback (ArdourCanvas::Item*, GdkEvent*);
 	void cursor_drag_finished_callback (ArdourCanvas::Item*, GdkEvent*);
@@ -1173,6 +1259,8 @@ class Editor : public PublicEditor
 
 	void show_verbose_time_cursor (nframes_t frame, double offset = 0, double xpos=-1, double ypos=-1);
 	void show_verbose_duration_cursor (nframes_t start, nframes_t end, double offset = 0, double xpos=-1, double ypos=-1);
+	double clamp_verbose_cursor_x (double);
+	double clamp_verbose_cursor_y (double);
 
 	/* Canvas event handlers */
 
@@ -1186,6 +1274,14 @@ class Editor : public PublicEditor
 	bool canvas_fade_in_handle_event (GdkEvent* event,ArdourCanvas::Item*, AudioRegionView*);
 	bool canvas_fade_out_event (GdkEvent* event,ArdourCanvas::Item*, AudioRegionView*);
 	bool canvas_fade_out_handle_event (GdkEvent* event,ArdourCanvas::Item*, AudioRegionView*);
+	
+
+	// These variables are used to detect a feedback loop and break it to avoid a gui hang
+private:
+	ArdourCanvas::Item *last_item_entered;
+	int last_item_entered_n;
+public:
+
 	bool canvas_region_view_event (GdkEvent* event,ArdourCanvas::Item*, RegionView*);
 	bool canvas_region_view_name_highlight_event (GdkEvent* event,ArdourCanvas::Item*, RegionView*);
 	bool canvas_region_view_name_event (GdkEvent* event,ArdourCanvas::Item*, RegionView*);
@@ -1201,6 +1297,7 @@ class Editor : public PublicEditor
 	bool canvas_marker_bar_event (GdkEvent* event, ArdourCanvas::Item*);
 	bool canvas_range_marker_bar_event (GdkEvent* event, ArdourCanvas::Item*);
 	bool canvas_transport_marker_bar_event (GdkEvent* event, ArdourCanvas::Item*);
+	bool canvas_cd_marker_bar_event (GdkEvent* event, ArdourCanvas::Item*);
 
 	bool canvas_imageframe_item_view_event(GdkEvent* event, ArdourCanvas::Item*,ImageFrameView*);
 	bool canvas_imageframe_view_event(GdkEvent* event, ArdourCanvas::Item*,ImageFrameTimeAxis*);
@@ -1214,37 +1311,27 @@ class Editor : public PublicEditor
 	/* non-public event handlers */
 
 	bool canvas_playhead_cursor_event (GdkEvent* event, ArdourCanvas::Item*);
-	bool canvas_edit_cursor_event (GdkEvent* event, ArdourCanvas::Item*);
 	bool track_canvas_scroll (GdkEventScroll* event);
+	bool time_canvas_scroll (GdkEventScroll* event);
 
 	bool track_canvas_scroll_event (GdkEventScroll* event);
 	bool track_canvas_button_press_event (GdkEventButton* event);
 	bool track_canvas_button_release_event (GdkEventButton* event);
 	bool track_canvas_motion_notify_event (GdkEventMotion* event);
 
+	bool time_canvas_scroll_event (GdkEventScroll* event);
+
 	Gtk::Allocation canvas_allocation;
 	bool canvas_idle_queued;
 	void track_canvas_allocate (Gtk::Allocation alloc);
 	bool track_canvas_size_allocated ();
 
-	void set_edit_cursor (GdkEvent* event);
-	void set_playhead_cursor (GdkEvent* event);
+	void set_playhead_cursor ();
 
 	void kbd_driver (sigc::slot<void,GdkEvent*>, bool use_track_canvas = true, bool use_time_canvas = true, bool can_select = true);
-	void kbd_set_playhead_cursor ();
-	void kbd_set_edit_cursor ();
 	void kbd_mute_unmute_region ();
-	void kbd_split ();
-	void kbd_set_sync_position ();
-	void kbd_align (ARDOUR::RegionPoint);
-	void kbd_align_relative (ARDOUR::RegionPoint);
 	void kbd_brush ();
-	void kbd_audition ();
 
-	void kbd_do_split (GdkEvent*);
-	void kbd_do_set_sync_position (GdkEvent* ev);
-	void kbd_do_align (GdkEvent*, ARDOUR::RegionPoint);
-	void kbd_do_align_relative (GdkEvent*, ARDOUR::RegionPoint);
 	void kbd_do_brush (GdkEvent*);
 	void kbd_do_audition (GdkEvent*);
 
@@ -1289,6 +1376,7 @@ class Editor : public PublicEditor
 	void marker_menu_edit ();
 	void marker_menu_remove ();
 	void marker_menu_rename ();
+	void marker_menu_lock (bool yn);
 	void marker_menu_hide ();
 	void marker_menu_loop_range ();
 	void marker_menu_select_all_selectables_using_range ();
@@ -1319,6 +1407,7 @@ class Editor : public PublicEditor
 	Gtk::Menu* range_marker_menu;
 	Gtk::Menu* transport_marker_menu;
 	Gtk::Menu* new_transport_marker_menu;
+	Gtk::Menu* cd_marker_menu;
 	ArdourCanvas::Item* marker_menu_item;
 
 	typedef list<Marker*> Marks;
@@ -1346,7 +1435,7 @@ class Editor : public PublicEditor
 
 	void editor_mixer_button_toggled ();
 
-	AudioClock               edit_cursor_clock;
+	AudioClock               edit_point_clock;
 	AudioClock               zoom_range_clock;
 	Gtk::Button              zoom_in_button;
 	Gtk::Button              zoom_out_button;
@@ -1379,6 +1468,8 @@ class Editor : public PublicEditor
 	Gtk::ComboBoxText edit_mode_selector;
 	Gtk::VBox         edit_mode_box;
 
+	void set_edit_mode (ARDOUR::EditMode);
+	void cycle_edit_mode ();
 	void edit_mode_selection_done ();
 
 	Gtk::ComboBoxText snap_type_selector;
@@ -1410,7 +1501,7 @@ class Editor : public PublicEditor
 
 	void                zoom_adjustment_changed();
 
-	void                edit_cursor_clock_changed();
+	void                edit_point_clock_changed();
 	
 	void setup_toolbar ();
 
@@ -1428,6 +1519,7 @@ class Editor : public PublicEditor
 	void track_selection_changed ();
 	void region_selection_changed ();
 	void point_selection_changed ();
+	void marker_selection_changed ();
 
 	enum SelectionOp {
 		CreateSelection,
@@ -1450,13 +1542,15 @@ class Editor : public PublicEditor
 	/* transport range select process */
 	enum RangeMarkerOp {
 		CreateRangeMarker,
-		CreateTransportMarker
+		CreateTransportMarker,
+		CreateCDMarker
 	} range_marker_op;
 
 	void start_range_markerbar_op (ArdourCanvas::Item* item, GdkEvent* event, RangeMarkerOp);
 	void drag_range_markerbar_op (ArdourCanvas::Item* item, GdkEvent* event);
 	void end_range_markerbar_op (ArdourCanvas::Item* item, GdkEvent* event);
 
+	ArdourCanvas::SimpleRect*  cd_marker_bar_drag_rect;
 	ArdourCanvas::SimpleRect*  range_bar_drag_rect;
 	ArdourCanvas::SimpleRect*  transport_bar_drag_rect;
 	ArdourCanvas::Line*        marker_drag_line;
@@ -1526,6 +1620,13 @@ class Editor : public PublicEditor
 	bool ignore_route_order_sync;
 
 	bool route_list_display_button_press (GdkEventButton*);
+	void route_list_display_drag_data_received  (const Glib::RefPtr<Gdk::DragContext>& context,
+						     gint                x,
+						     gint                y,
+						     const Gtk::SelectionData& data,
+						     guint               info,
+						     guint               time);
+
 	bool route_list_selection_filter (const Glib::RefPtr<Gtk::TreeModel>& model, const Gtk::TreeModel::Path& path, bool yn);
 
 	void route_list_change (const Gtk::TreeModel::Path&,const Gtk::TreeModel::iterator&);
@@ -1620,9 +1721,16 @@ class Editor : public PublicEditor
 
 	void trim_finished_callback (ArdourCanvas::Item*, GdkEvent*);
 	void thaw_region_after_trim (RegionView& rv);
-	
-	void trim_region_to_edit_cursor ();
-	void trim_region_from_edit_cursor ();
+
+	void trim_region_front();
+	void trim_region_back();
+	void trim_region (bool front);
+
+	void trim_region_to_edit_point ();
+	void trim_region_from_edit_point ();
+	void trim_region_to_loop ();
+	void trim_region_to_punch ();
+	void trim_region_to_location (const ARDOUR::Location&, const char* cmd);
 
 	bool show_gain_after_trim;
 
@@ -1734,45 +1842,69 @@ class Editor : public PublicEditor
 
 	/* duplication */
 
-	void duplicate_dialog (bool for_region);
+	void duplicate_dialog (bool with_dialog);
 	
-	nframes_t event_frame (GdkEvent*, double* px = 0, double* py = 0);
+	nframes64_t event_frame (GdkEvent*, double* px = 0, double* py = 0) const;
+
+	/* returns false if mouse pointer is not in track or marker canvas
+	 */
+	bool mouse_frame (nframes64_t&, bool& in_track_canvas) const;
 
 	void time_fx_motion (ArdourCanvas::Item*, GdkEvent*);
 	void start_time_fx (ArdourCanvas::Item*, GdkEvent*);
 	void end_time_fx (ArdourCanvas::Item*, GdkEvent*);
 
-	struct TimeStretchDialog : public ArdourDialog {
-	    ARDOUR::Session::TimeStretchRequest request;
+	struct TimeFXDialog : public ArdourDialog {
+	    ARDOUR::TimeFXRequest request;
 	    Editor&               editor;
+	    bool                  pitching;
+	    Gtk::Adjustment       pitch_octave_adjustment;
+	    Gtk::Adjustment       pitch_semitone_adjustment;
+	    Gtk::Adjustment       pitch_cent_adjustment;
+	    Gtk::SpinButton       pitch_octave_spinner;
+	    Gtk::SpinButton       pitch_semitone_spinner;
+	    Gtk::SpinButton       pitch_cent_spinner;
 	    RegionSelection       regions;
 	    Gtk::ProgressBar      progress_bar;
+
+	    /* SoundTouch */
 	    Gtk::ToggleButton     quick_button;
 	    Gtk::ToggleButton     antialias_button;
+	    Gtk::HBox             upper_button_box;
+
+	    /* RubberBand */
+	    Gtk::ComboBoxText     stretch_opts_selector;
+	    Gtk::Label            stretch_opts_label;
+	    Gtk::ToggleButton     precise_button;
+	    Gtk::HBox             opts_box;
+
 	    Gtk::Button*          cancel_button;
 	    Gtk::Button*          action_button;
-	    Gtk::HBox             upper_button_box;
 	    Gtk::VBox             packer;
 	    int                   status;
 
-	    TimeStretchDialog (Editor& e);
+	    TimeFXDialog (Editor& e, bool for_pitch);
 
 	    gint update_progress ();
 	    sigc::connection first_cancel;
 	    sigc::connection first_delete;
-	    void cancel_timestretch_in_progress ();
-	    gint delete_timestretch_in_progress (GdkEventAny*);
+	    void cancel_in_progress ();
+	    gint delete_in_progress (GdkEventAny*);
 	};
 
 	/* "whats mine is yours" */
 
-	friend class TimeStretchDialog;
+	friend class TimeFXDialog;
 
-	TimeStretchDialog* current_timestretch;
+	TimeFXDialog* current_timefx;
 
-	static void* timestretch_thread (void *arg);
-	int run_timestretch (RegionSelection&, float fraction);
-	void do_timestretch (TimeStretchDialog&);
+	static void* timefx_thread (void *arg);
+	void do_timefx (TimeFXDialog&);
+
+	int time_stretch (RegionSelection&, float fraction);
+	int pitch_shift (RegionSelection&, float cents);
+	void pitch_shift_regions ();
+	int time_fx (RegionSelection&, float val, bool pitching);
 
 	/* editor-mixer strip */
 
@@ -1788,7 +1920,7 @@ class Editor : public PublicEditor
 
 	/* nudging tracks */
 
-	void nudge_track (bool use_edit_cursor, bool forwards);
+	void nudge_track (bool use_edit_point, bool forwards);
 
 	/* xfades */
 
@@ -1853,6 +1985,9 @@ class Editor : public PublicEditor
 	AudioClock       nudge_clock;
 
 	nframes_t get_nudge_distance (nframes_t pos, nframes_t& next);
+
+	bool nudge_forward_release (GdkEventButton*);
+	bool nudge_backward_release (GdkEventButton*);
 	
 	/* audio filters */
 
@@ -1873,6 +2008,9 @@ class Editor : public PublicEditor
 
 	TimeAxisView* entered_track;
 	RegionView*   entered_regionview;
+
+
+	void ensure_entered_track_selected (bool op_acts_on_objects = false);
 	bool clear_entered_track;
 	gint left_track_canvas (GdkEventCrossing*);
 	void set_entered_track (TimeAxisView*);
@@ -1885,13 +2023,8 @@ class Editor : public PublicEditor
 	void toggle_gain_envelope_active ();
 	void reset_region_gain_envelopes ();
 
-	Gtk::CheckMenuItem* region_envelope_visible_item;
-	Gtk::CheckMenuItem* region_envelope_active_item;
-	Gtk::CheckMenuItem* region_mute_item;
-	Gtk::CheckMenuItem* region_lock_item;
-	Gtk::CheckMenuItem* region_opaque_item;
-	
 	bool on_key_press_event (GdkEventKey*);
+	bool on_key_release_event (GdkEventKey*);
 
 	void session_state_saved (string);
 
@@ -1902,6 +2035,40 @@ class Editor : public PublicEditor
 	void color_handler ();
 	
 	Gtk::HBox      status_bar_hpacker;
+
+	Editing::EditPoint _edit_point;
+
+	Gtk::ComboBoxText edit_point_selector;
+
+	void set_edit_point_preference (Editing::EditPoint ep);
+	void cycle_edit_point (bool with_marker);
+	void set_edit_point ();
+	void edit_point_selection_done ();
+	void edit_point_chosen (Editing::EditPoint);
+	Glib::RefPtr<Gtk::RadioAction> edit_point_action (Editing::EditPoint);
+	std::vector<std::string> edit_point_strings;
+
+	void selected_marker_moved (ARDOUR::Location*);
+	sigc::connection edit_point_clock_connection_a;
+	sigc::connection edit_point_clock_connection_b;
+
+	bool get_edit_op_range (nframes64_t& start, nframes64_t& end) const;
+
+	void get_regions_at (RegionSelection&, nframes64_t where, const TrackSelection& ts) const;
+	void get_regions_after (RegionSelection&, nframes64_t where, const TrackSelection& ts) const;
+	
+	void get_regions_for_action (RegionSelection&, bool allowed_entered_regionview = true);
+
+	sigc::connection fast_screen_update_connection;
+	gint start_updating ();
+	gint stop_updating ();
+	void fast_update_strips ();
+	bool meters_running;
+
+	void select_next_route ();
+	void select_prev_route ();
+
+	RhythmFerret* rhythm_ferret;
 };
 
 #endif /* __ardour_editor_h__ */
