@@ -102,6 +102,9 @@ Editor::initialize_canvas ()
 	track_canvas.signal_button_press_event().connect (mem_fun (*this, &Editor::track_canvas_button_press_event));
 	track_canvas.signal_button_release_event().connect (mem_fun (*this, &Editor::track_canvas_button_release_event));
 
+	/* just scroll stuff for the timecanvas */
+	time_canvas.signal_scroll_event().connect (mem_fun (*this, &Editor::time_canvas_scroll_event));
+
 	track_canvas.set_name ("EditorMainCanvas");
 	track_canvas.add_events (Gdk::POINTER_MOTION_HINT_MASK|Gdk::SCROLL_MASK);
 	track_canvas.signal_leave_notify_event().connect (mem_fun(*this, &Editor::left_track_canvas));
@@ -138,10 +141,10 @@ Editor::initialize_canvas ()
 	if (Profile->get_sae()) {
 		Image img (::get_icon (X_("saelogo")));
 		logo_item = new ArdourCanvas::Pixbuf (*track_canvas.root(), 0.0, 0.0, img.get_pixbuf());
-		logo_item->property_height_in_pixels() = true;
-		logo_item->property_width_in_pixels() = true;
-		logo_item->property_height_set() = true;
-		logo_item->property_width_set() = true;
+		// logo_item->property_height_in_pixels() = true;
+		// logo_item->property_width_in_pixels() = true;
+		// logo_item->property_height_set() = true;
+		// logo_item->property_width_set() = true;
 		logo_item->show ();
 	}
 	
@@ -158,9 +161,10 @@ Editor::initialize_canvas ()
 	
 	meter_group = new ArdourCanvas::Group (*time_canvas.root(), 0.0, 0.0);
 	tempo_group = new ArdourCanvas::Group (*time_canvas.root(), 0.0, timebar_height);
-	marker_group = new ArdourCanvas::Group (*time_canvas.root(), 0.0, timebar_height * 2.0);
-	range_marker_group = new ArdourCanvas::Group (*time_canvas.root(), 0.0, timebar_height * 3.0);
-	transport_marker_group = new ArdourCanvas::Group (*time_canvas.root(), 0.0, timebar_height * 4.0);
+	range_marker_group = new ArdourCanvas::Group (*time_canvas.root(), 0.0, timebar_height * 2.0);
+	transport_marker_group = new ArdourCanvas::Group (*time_canvas.root(), 0.0, timebar_height * 3.0);
+	marker_group = new ArdourCanvas::Group (*time_canvas.root(), 0.0, timebar_height * 4.0);
+	cd_marker_group = new ArdourCanvas::Group (*time_canvas.root(), 0.0, timebar_height * 5.0);
 	
 	tempo_bar = new ArdourCanvas::SimpleRect (*tempo_group, 0.0, 0.0, max_canvas_coordinate, timebar_height-1.0);
 	tempo_bar->property_outline_pixels() = 0;
@@ -170,6 +174,9 @@ Editor::initialize_canvas ()
 	
 	marker_bar = new ArdourCanvas::SimpleRect (*marker_group, 0.0, 0.0, max_canvas_coordinate, timebar_height-1.0);
 	marker_bar->property_outline_pixels() = 0;
+
+	cd_marker_bar = new ArdourCanvas::SimpleRect (*cd_marker_group, 0.0, 0.0, max_canvas_coordinate, timebar_height-1.0);
+	cd_marker_bar->property_outline_pixels() = 0;
 	
 	range_marker_bar = new ArdourCanvas::SimpleRect (*range_marker_group, 0.0, 0.0, max_canvas_coordinate, timebar_height-1.0);
 	range_marker_bar->property_outline_pixels() = 0;
@@ -177,6 +184,10 @@ Editor::initialize_canvas ()
 	transport_marker_bar = new ArdourCanvas::SimpleRect (*transport_marker_group, 0.0, 0.0, max_canvas_coordinate, timebar_height-1.0);
 	transport_marker_bar->property_outline_pixels() = 0;
 	
+	cd_marker_bar_drag_rect = new ArdourCanvas::SimpleRect (*cd_marker_group, 0.0, 0.0, max_canvas_coordinate, timebar_height-1.0);
+	cd_marker_bar_drag_rect->property_outline_pixels() = 0;
+	cd_marker_bar_drag_rect->hide ();
+
 	range_bar_drag_rect = new ArdourCanvas::SimpleRect (*range_marker_group, 0.0, 0.0, max_canvas_coordinate, timebar_height-1.0);
 	range_bar_drag_rect->property_outline_pixels() = 0;
 	range_bar_drag_rect->hide ();
@@ -235,6 +246,7 @@ Editor::initialize_canvas ()
 	tempo_bar->signal_event().connect (bind (mem_fun (*this, &Editor::canvas_tempo_bar_event), tempo_bar));
 	meter_bar->signal_event().connect (bind (mem_fun (*this, &Editor::canvas_meter_bar_event), meter_bar));
 	marker_bar->signal_event().connect (bind (mem_fun (*this, &Editor::canvas_marker_bar_event), marker_bar));
+	cd_marker_bar->signal_event().connect (bind (mem_fun (*this, &Editor::canvas_cd_marker_bar_event), cd_marker_bar));
 	range_marker_bar->signal_event().connect (bind (mem_fun (*this, &Editor::canvas_range_marker_bar_event), range_marker_bar));
 	transport_marker_bar->signal_event().connect (bind (mem_fun (*this, &Editor::canvas_transport_marker_bar_event), transport_marker_bar));
 	
@@ -245,7 +257,6 @@ Editor::initialize_canvas ()
 	double time_width = FLT_MAX/frames_per_unit;
 	time_canvas.set_scroll_region(0.0, 0.0, time_width, time_height);
 
-	edit_cursor = new Cursor (*this, &Editor::canvas_edit_cursor_event);
 	playhead_cursor = new Cursor (*this, &Editor::canvas_playhead_cursor_event);
 
 	initial_ruler_update_required = true;
@@ -275,7 +286,6 @@ Editor::track_canvas_allocate (Gtk::Allocation alloc)
 	} 
 
 	initial_ruler_update_required = false;
-	
 	track_canvas_size_allocated ();
 }
 
@@ -305,53 +315,33 @@ Editor::track_canvas_size_allocated ()
 	}
 
 	zoom_range_clock.set ((nframes_t) floor ((canvas_width * frames_per_unit)));
-	edit_cursor->set_position (edit_cursor->current_frame);
 	playhead_cursor->set_position (playhead_cursor->current_frame);
 
 	reset_hscrollbar_stepping ();
 	reset_scrolling_region ();
 
-	if (edit_cursor) edit_cursor->set_length (canvas_height);
 	if (playhead_cursor) playhead_cursor->set_length (canvas_height);
 
-	if (marker_drag_line) {
-		marker_drag_line_points.back().set_y(canvas_height);
-		marker_drag_line->property_points() = marker_drag_line_points;
+	double y1 = vertical_adjustment.get_value ();
+ 	
+	for (MarkerSelection::iterator x = selection->markers.begin(); x != selection->markers.end(); ++x) {
+		(*x)->set_line_vpos (y1, canvas_height);
 	}
 
-	if (range_marker_drag_rect) {
-		range_marker_drag_rect->property_y1() = 0.0;
-		range_marker_drag_rect->property_y2() = canvas_height;
-	}
-
-	if (transport_loop_range_rect) {
-		transport_loop_range_rect->property_y1() = 0.0;
-		transport_loop_range_rect->property_y2() = canvas_height;
-	}
-
-	if (transport_punch_range_rect) {
-		transport_punch_range_rect->property_y1() = 0.0;
-		transport_punch_range_rect->property_y2() = canvas_height;
-	}
-
-	if (transport_punchin_line) {
-		transport_punchin_line->property_y1() = 0.0;
-		transport_punchin_line->property_y2() = canvas_height;
-	}
-
-	if (transport_punchout_line) {
-		transport_punchout_line->property_y1() = 0.0;
-		transport_punchout_line->property_y2() = canvas_height;
-	}
-		
+ 	range_marker_drag_rect->property_y1() = y1;
+ 	range_marker_drag_rect->property_y2() = full_canvas_height;
+ 	transport_loop_range_rect->property_y1() = y1;
+ 	transport_loop_range_rect->property_y2() = full_canvas_height;
+ 	transport_punch_range_rect->property_y1() = y1;
+ 	transport_punch_range_rect->property_y2() = full_canvas_height;
+ 	transport_punchin_line->property_y1() = y1;
+ 	transport_punchin_line->property_y2() = full_canvas_height;
+ 	transport_punchout_line->property_y1() = y1;
+ 	transport_punchout_line->property_y2() = full_canvas_height;
+	
 	update_fixed_rulers();
 	redisplay_tempo (true);
 
-	if (logo_item) {
-		logo_item->property_height() = canvas_height;
-		logo_item->property_width() = canvas_width;
-	}
-
 	Resized (); /* EMIT_SIGNAL */
 
 	return false;
@@ -371,12 +361,20 @@ Editor::reset_scrolling_region (Gtk::Allocation* alloc)
 		}
 	}
 
-	double last_canvas_unit =  last_canvas_frame / frames_per_unit;
+	double last_canvas_unit =  max ((last_canvas_frame / frames_per_unit), canvas_width);
 
-	track_canvas.set_scroll_region (0.0, 0.0, max (last_canvas_unit, canvas_width), pos);
+	track_canvas.set_scroll_region (0.0, 0.0, last_canvas_unit, pos);
 
 	// XXX what is the correct height value for the time canvas ? this overstates it
-	time_canvas.set_scroll_region ( 0.0, 0.0, max (last_canvas_unit, canvas_width), canvas_height);
+	time_canvas.set_scroll_region ( 0.0, 0.0, last_canvas_unit, canvas_height);
+
+	range_marker_drag_rect->property_y2() = canvas_height;
+	transport_loop_range_rect->property_y2() = canvas_height;
+	transport_punch_range_rect->property_y2() = canvas_height;
+	transport_punchin_line->property_y2() = canvas_height;
+	transport_punchout_line->property_y2() = canvas_height;
+
+	update_punch_range_view (true);
 
 	controls_layout.queue_resize();
 }
@@ -388,8 +386,8 @@ Editor::controls_layout_size_request (Requisition* req)
 	TreeModel::Children::iterator i;
 	double pos;
 
-        for (pos = 0, i = rows.begin(); i != rows.end(); ++i) {
-	        TimeAxisView *tv = (*i)[route_display_columns.tv];
+	for (pos = 0, i = rows.begin(); i != rows.end(); ++i) {
+		TimeAxisView *tv = (*i)[route_display_columns.tv];
 		if (tv != 0) {
 			pos += tv->effective_height;
 		}
@@ -401,12 +399,9 @@ Editor::controls_layout_size_request (Requisition* req)
 		screen = Gdk::Screen::get_default();
 	}
 
-	/* never let the width of the controls area shrink horizontally */
-
 	edit_controls_vbox.check_resize();
-
 	req->width = max (edit_controls_vbox.get_width(),  controls_layout.get_width());
-	
+
 	/* don't get too big. the fudge factors here are just guesses */
 	
 	req->width = min (req->width, screen->get_width() - 300);
@@ -415,8 +410,13 @@ Editor::controls_layout_size_request (Requisition* req)
 	/* this one is important: it determines how big the layout thinks it really is, as 
 	   opposed to what it displays on the screen
 	*/
+	
+	controls_layout.set_size (edit_controls_vbox.get_width(), (gint) pos);
+	controls_layout.set_size_request(edit_controls_vbox.get_width(), -1);
+	zoom_box.set_size_request(edit_controls_vbox.get_width(), -1);
+	time_button_frame.set_size_request(edit_controls_vbox.get_width() + edit_vscrollbar.get_width(), -1);
 
-	controls_layout.set_size (req->width, (gint) pos);
+	//cerr << "sizes = " << req->width << " " << edit_controls_vbox.get_width() << " " << controls_layout.get_width() << " " << zoom_box.get_width() << " " << time_button_frame.get_width() << endl;//DEBUG	
 }
 
 bool
@@ -439,6 +439,8 @@ Editor::track_canvas_drag_data_received (const RefPtr<Gdk::DragContext>& context
 					 const SelectionData& data,
 					 guint info, guint time)
 {
+	cerr << "drop on canvas, target = " << data.get_target() << endl;
+
 	if (data.get_target() == "regions") {
 		drop_regions (context, x, y, data, info, time);
 	} else {
@@ -688,19 +690,55 @@ Editor::left_track_canvas (GdkEventCrossing *ev)
 	return FALSE;
 }
 
+void
+Editor::tie_vertical_scrolling ()
+{
+	double y1 = vertical_adjustment.get_value();
+
+	playhead_cursor->set_y_axis (y1);
+
+ 	range_marker_drag_rect->property_y1() = y1;
+ 	range_marker_drag_rect->property_y2() = full_canvas_height;
+ 	transport_loop_range_rect->property_y1() = y1;
+ 	transport_loop_range_rect->property_y2() = full_canvas_height;
+ 	transport_punch_range_rect->property_y1() = y1;
+ 	transport_punch_range_rect->property_y2() = full_canvas_height;
+ 	transport_punchin_line->property_y1() = y1;
+ 	transport_punchin_line->property_y2() = full_canvas_height;
+ 	transport_punchout_line->property_y1() = y1;
+ 	transport_punchout_line->property_y2() = full_canvas_height;
+
+	if (!selection->markers.empty()) {
+		for (MarkerSelection::iterator x = selection->markers.begin(); x != selection->markers.end(); ++x) {		
+			(*x)->set_line_vpos (y1, canvas_height);
+		}
+	}
+
+	if (logo_item) {
+		logo_item->property_y() = y1;
+	}
+
+	/* this will do an immediate redraw */
+
+	controls_layout.get_vadjustment()->set_value (y1);
+}
 
 void 
 Editor::canvas_horizontally_scrolled ()
 {
-	/* this is the core function that controls horizontal scrolling of the canvas. it is called
-	   whenever the horizontal_adjustment emits its "value_changed" signal. it typically executes in an
-	   idle handler, which is important because tempo_map_changed() should issue redraws immediately
-	   and not defer them to an idle handler.
-	*/
+	nframes64_t time_origin = (nframes_t) floor (horizontal_adjustment.get_value() * frames_per_unit);
 
-  	leftmost_frame = (nframes_t) floor (horizontal_adjustment.get_value() * frames_per_unit);
+	if (time_origin != leftmost_frame) {
+		canvas_scroll_to (time_origin);
+	}
+}
+
+void
+Editor::canvas_scroll_to (nframes64_t time_origin)
+{
+  	leftmost_frame = time_origin;
 	nframes_t rightmost_frame = leftmost_frame + current_page_frames ();
-	
+
 	if (rightmost_frame > last_canvas_frame) {
 		last_canvas_frame = rightmost_frame;
 		reset_scrolling_region ();
@@ -718,7 +756,6 @@ Editor::canvas_horizontally_scrolled ()
 void
 Editor::color_handler()
 {
-	edit_cursor->canvas_item.property_fill_color_rgba() = ARDOUR_UI::config()->canvasvar_EditCursor.get();
 	playhead_cursor->canvas_item.property_fill_color_rgba() = ARDOUR_UI::config()->canvasvar_PlayHead.get();
 	verbose_canvas_cursor->property_fill_color_rgba() = ARDOUR_UI::config()->canvasvar_VerboseCanvasCursor.get();
 
@@ -731,12 +768,18 @@ Editor::color_handler()
 	marker_bar->property_fill_color_rgba() = ARDOUR_UI::config()->canvasvar_MarkerBar.get();
 	marker_bar->property_outline_color_rgba() = ARDOUR_UI::config()->canvasvar_MarkerBarSeparator.get();
 
+	cd_marker_bar->property_fill_color_rgba() = ARDOUR_UI::config()->canvasvar_CDMarkerBar.get();
+	cd_marker_bar->property_outline_color_rgba() = ARDOUR_UI::config()->canvasvar_MarkerBarSeparator.get();
+
 	range_marker_bar->property_fill_color_rgba() = ARDOUR_UI::config()->canvasvar_RangeMarkerBar.get();
 	range_marker_bar->property_outline_color_rgba() = ARDOUR_UI::config()->canvasvar_MarkerBarSeparator.get();
 
 	transport_marker_bar->property_fill_color_rgba() = ARDOUR_UI::config()->canvasvar_TransportMarkerBar.get();
 	transport_marker_bar->property_outline_color_rgba() = ARDOUR_UI::config()->canvasvar_MarkerBarSeparator.get();
 
+	cd_marker_bar_drag_rect->property_fill_color_rgba() = ARDOUR_UI::config()->canvasvar_RangeDragBarRect.get();
+	cd_marker_bar_drag_rect->property_outline_color_rgba() = ARDOUR_UI::config()->canvasvar_RangeDragBarRect.get();
+
 	range_bar_drag_rect->property_fill_color_rgba() = ARDOUR_UI::config()->canvasvar_RangeDragBarRect.get();
 	range_bar_drag_rect->property_outline_color_rgba() = ARDOUR_UI::config()->canvasvar_RangeDragBarRect.get();
 
@@ -776,4 +819,12 @@ Editor::color_handler()
 		session->tempo_map().apply_with_metrics (*this, &Editor::draw_metric_marks); // redraw metric markers
 }
 
+void
+Editor::flush_canvas ()
+{
+	if (is_mapped()) {
+		track_canvas.update_now ();
+		// gdk_window_process_updates (GTK_LAYOUT(track_canvas.gobj())->bin_window, true);
+	}
+}
 
