@@ -38,6 +38,7 @@ using namespace sigc;
 using namespace ARDOUR;
 using namespace PBD;
 using namespace Gtk;
+using namespace Glib;
 
 void
 Editor::handle_new_route (Session::RouteList& routes)
@@ -61,7 +62,7 @@ Editor::handle_new_route (Session::RouteList& routes)
 		}
 		
 		tv = new AudioTimeAxisView (*this, *session, route, track_canvas);
-		
+		//cerr << "Editor::handle_new_route() called on " << route->name() << endl;//DEBUG
 		row = *(route_display_model->append ());
 		
 		row[route_display_columns.route] = route;
@@ -115,6 +116,14 @@ Editor::remove_route (TimeAxisView *tv)
 	TreeModel::Children rows = route_display_model->children();
 	TreeModel::Children::iterator ri;
 
+	/* Decrement old order keys for tracks `above' the one that is being removed */
+	for (ri = rows.begin(); ri != rows.end(); ++ri) {
+		TimeAxisView* v = (*ri)[route_display_columns.tv];
+		if (v->old_order_key() > tv->old_order_key()) {
+			v->set_old_order_key (v->old_order_key() - 1);
+		}
+	}
+
 	for (ri = rows.begin(); ri != rows.end(); ++ri) {
 		if ((*ri)[route_display_columns.tv] == tv) {
 			route_display_model->erase (ri);
@@ -538,3 +547,21 @@ Editor::route_list_delete (const Gtk::TreeModel::Path& path)
 	session->set_remote_control_ids();
 	redisplay_route_list ();
 }
+
+
+void  
+Editor::route_list_display_drag_data_received (const RefPtr<Gdk::DragContext>& context,
+						int x, int y, 
+						const SelectionData& data,
+						guint info, guint time)
+{
+	cerr << "RouteLD::dddr target = " << data.get_target() << endl;
+	
+	if (data.get_target() == "GTK_TREE_MODEL_ROW") {
+		cerr << "Delete drag data drop to treeview\n";
+		route_list_display.on_drag_data_received (context, x, y, data, info, time);
+		return;
+	}
+	cerr << "some other kind of drag\n";
+	context->drag_finish (true, false, time);
+}
