@@ -24,7 +24,6 @@
 #include <fstream>
 
 #include <samplerate.h>
-
 #include <pbd/convert.h>
 #include <pbd/xml++.h>
 
@@ -38,6 +37,7 @@
 #include <ardour/audioengine.h>
 #include <ardour/gdither.h>
 #include <ardour/utils.h>
+#include <ardour/profile.h>
 
 #include "export_dialog.h"
 #include "ardour_ui.h"
@@ -318,30 +318,52 @@ ExportDialog::ExportDialog(PublicEditor& e)
 	format_table.set_col_spacings (5);
 	format_table.set_row_spacings (5);
 
-	format_table.attach (channel_count_label, 0, 1, 0, 1);
-	format_table.attach (channel_count_combo, 1, 2, 0, 1);
+	int row = 0;
+
+	format_table.attach (channel_count_label, 0, 1, row, row+1);
+	format_table.attach (channel_count_combo, 1, 2, row, row+1);
+
+	row++;
 	
-	format_table.attach (header_format_label, 0, 1, 1, 2);
-	format_table.attach (header_format_combo, 1, 2, 1, 2);
+	format_table.attach (header_format_label, 0, 1, row, row+1);
+	format_table.attach (header_format_combo, 1, 2, row, row+1);
 
-	format_table.attach (bitdepth_format_label, 0, 1, 2, 3);
-	format_table.attach (bitdepth_format_combo, 1, 2, 2, 3);
+	row++;
 
-	format_table.attach (endian_format_label, 0, 1, 3, 4);
-	format_table.attach (endian_format_combo, 1, 2, 3, 4);
+	format_table.attach (bitdepth_format_label, 0, 1, row, row+1);
+	format_table.attach (bitdepth_format_combo, 1, 2, row, row+1);
 
-	format_table.attach (sample_rate_label, 0, 1, 4, 5);
-	format_table.attach (sample_rate_combo, 1, 2, 4, 5);
+	row++;
 
-	format_table.attach (src_quality_label, 0, 1, 5, 6);
-	format_table.attach (src_quality_combo, 1, 2, 5, 6);
+	if (!Profile->get_sae()) {
+		format_table.attach (endian_format_label, 0, 1, row, row+1);
+		format_table.attach (endian_format_combo, 1, 2, row, row+1);
+		row++;
+	}
 
-	format_table.attach (dither_type_label, 0, 1, 6, 7);
-	format_table.attach (dither_type_combo, 1, 2, 6, 7);
+	format_table.attach (sample_rate_label, 0, 1, row, row+1);
+	format_table.attach (sample_rate_combo, 1, 2, row, row+1);
 
-	format_table.attach (cue_file_label, 0, 1, 7, 8);
-	format_table.attach (cue_file_combo, 1, 2, 7, 8);
-	format_table.attach (cuefile_only_checkbox, 0, 2, 8, 9);
+	row++;
+
+	if (!Profile->get_sae()) {
+		format_table.attach (src_quality_label, 0, 1, row, row+1);
+		format_table.attach (src_quality_combo, 1, 2, row, row+1);
+		row++;
+	}
+
+	format_table.attach (dither_type_label, 0, 1, row, row+1);
+	format_table.attach (dither_type_combo, 1, 2, row, row+1);
+
+	row++;
+
+	if (!Profile->get_sae()) {
+		format_table.attach (cue_file_label, 0, 1, row, row+1);
+		format_table.attach (cue_file_combo, 1, 2, row, row+1);
+		row++;
+	
+		format_table.attach (cuefile_only_checkbox, 0, 2, row, row+1);
+	}
 
 	file_entry.set_name ("ExportFileDisplay");
 
@@ -625,10 +647,9 @@ ExportDialog::export_toc_file (Locations::LocationList& locations, const string&
 		return;
 	}
 	
-    string filepath = path + ".toc";
+	string filepath = path + ".toc";
 	ofstream out (filepath.c_str());
 	long unsigned int last_end_time = spec.start_frame, last_start_time = spec.start_frame;
-	int numtracks = 0;
 	gchar buf[18];
 
 	if (!out) {
@@ -643,102 +664,110 @@ ExportDialog::export_toc_file (Locations::LocationList& locations, const string&
 	Locations::LocationList temp;
 
 	for (i = locations.begin(); i != locations.end(); ++i) {
-	  if ((*i)->start() >= spec.start_frame && (*i)->end() <= spec.end_frame && (*i)->is_cd_marker() && !(*i)->is_end()) {
-	    temp.push_back (*i);
-	    if (!(*i)->is_mark()) {
-	      numtracks ++;
-	    }
-	  }
+		if ((*i)->start() >= spec.start_frame && (*i)->end() <= spec.end_frame && (*i)->is_cd_marker() && !(*i)->is_end()) {
+			temp.push_back (*i);
+		}
 	}
 
-	if (numtracks == 0 ) {
-		    /* the user supplied no track markers.
-		       we now treat the session as one track.*/
-
-		    out << endl << "TRACK AUDIO" << endl;
-		   
-		    out << "COPY" << endl;
-
-		    out << "NO PRE_EMPHASIS" << endl;
-   
-		    /* XXX add session properties for catalog etc.
-		       (so far only the session name is used) */
-		    
-		    out << "CD_TEXT {" << endl << "  LANGUAGE 0 {" << endl << "     TITLE \"" << session->name() << "\"" << endl;
-		    out << "  }" << endl << "}" << endl;
-
-		    out << "FILE \"" << path << "\" ";
-		    out << "00:00:00 " ;
-		    frames_to_cd_frames_string (buf, spec.end_frame - spec.start_frame, session->frame_rate());
-		    out << buf << endl;
-		    out << "START 00:00:00" << endl;
-
-		    last_start_time = spec.start_frame;
-		    last_end_time = spec.end_frame;
-	} 
-
-	if (temp.size()) {
+	if (temp.size() > 0) {
 		LocationSortByStart cmp;
 		temp.sort (cmp);
+		Location * curr_range = 0;
+		Locations::LocationList::iterator nexti;
 
 		for (i = temp.begin(); i != temp.end(); ++i) {
-	
-		      if (!(*i)->is_mark()) {
-			/*this is a track */
-		        out << endl << "TRACK AUDIO" << endl;
 
-			if ((*i)->cd_info.find("scms") != (*i)->cd_info.end())  {
-			  out << "NO ";
+			if ((*i)->start() >= last_end_time)
+			{
+				/* this is a track, defined by a cd range marker or a cd location marker outside of a cd range */
+				out << endl << "TRACK AUDIO" << endl;
+				
+				if ((*i)->cd_info.find("scms") != (*i)->cd_info.end())  {
+					out << "NO ";
+				}
+				out << "COPY" << endl;
+				
+				if ((*i)->cd_info.find("preemph") != (*i)->cd_info.end())  {
+					out << "PRE_EMPHASIS" << endl;
+				} else {
+					out << "NO PRE_EMPHASIS" << endl;
+				}
+				
+				if ((*i)->cd_info.find("isrc") != (*i)->cd_info.end())  {
+					out << "ISRC \"" << (*i)->cd_info["isrc"] << "\"" << endl;
+				}
+				
+				out << "CD_TEXT {" << endl << "  LANGUAGE 0 {" << endl << "     TITLE \"" << (*i)->name() << "\"" << endl;
+				if ((*i)->cd_info.find("performer") != (*i)->cd_info.end()) {
+					out << "     PERFORMER \"" << (*i)->cd_info["performer"]  << "\"" << endl;
+				}
+				if ((*i)->cd_info.find("string_composer") != (*i)->cd_info.end()) {
+					out  << "     COMPOSER \"" << (*i)->cd_info["string_composer"] << "\"" << endl;
+				}
+				
+				if ((*i)->cd_info.find("isrc") != (*i)->cd_info.end()) {			  
+					out  << "     ISRC \"";
+					out << (*i)->cd_info["isrc"].substr(0,2) << "-";
+					out << (*i)->cd_info["isrc"].substr(2,3) << "-";
+					out << (*i)->cd_info["isrc"].substr(5,2) << "-";
+					out << (*i)->cd_info["isrc"].substr(7,5) << "\"" << endl;
+				}
+				
+				out << "  }" << endl << "}" << endl;
+				
+				frames_to_cd_frames_string (buf, last_end_time - spec.start_frame, session->frame_rate());
+				out << "FILE \"" << path << "\" " << buf;
+				
+				if ((*i)->is_mark()) {
+					// a mark track location needs to look ahead to the next marker's start to determine length
+					nexti = i;
+					++nexti;
+					if (nexti != temp.end()) {
+						frames_to_cd_frames_string (buf, (*nexti)->start() - last_end_time, session->frame_rate());
+						out << buf << endl;
+						
+						frames_to_cd_frames_string (buf, (*i)->start() - last_end_time, session->frame_rate());
+						out << "START" << buf << endl;
+						
+						last_start_time = (*i)->start();
+						last_end_time = (*nexti)->start();
+					}
+					else {
+						// this was the last marker, use session end
+						frames_to_cd_frames_string (buf, spec.end_frame - last_end_time, session->frame_rate());
+						out << buf << endl;
+						
+						frames_to_cd_frames_string (buf, (*i)->start() - last_end_time, session->frame_rate());
+						out << "START" << buf << endl;
+						
+						last_start_time = (*i)->start();
+						last_end_time = spec.end_frame;
+					}
+
+					curr_range = 0;
+				}
+				else {
+					// range
+					frames_to_cd_frames_string (buf, (*i)->end() - last_end_time, session->frame_rate());
+					out << buf << endl;
+					
+					frames_to_cd_frames_string (buf, (*i)->start() - last_end_time, session->frame_rate());
+					out << "START" << buf << endl;
+					
+					last_start_time = (*i)->start();
+					last_end_time = (*i)->end();
+
+					curr_range = (*i);
+				}
+				
 			}
-			out << "COPY" << endl;
-
-			if ((*i)->cd_info.find("preemph") != (*i)->cd_info.end())  {
-			  out << "PRE_EMPHASIS" << endl;
-			} else {
-			  out << "NO PRE_EMPHASIS" << endl;
+			else if ((*i)->is_mark()) 
+			{
+				/* this is an index within a track */
+				
+				frames_to_cd_frames_string (buf, (*i)->start() - last_start_time, session->frame_rate());
+				out << "INDEX" << buf << endl;
 			}
-
-			if ((*i)->cd_info.find("isrc") != (*i)->cd_info.end())  {
-			  out << "ISRC \"" << (*i)->cd_info["isrc"] << "\"" << endl;
-			}
-
-			out << "CD_TEXT {" << endl << "  LANGUAGE 0 {" << endl << "     TITLE \"" << (*i)->name() << "\"" << endl;
-			if ((*i)->cd_info.find("performer") != (*i)->cd_info.end()) {
-			  out << "     PERFORMER \"" << (*i)->cd_info["performer"]  << "\"" << endl;
-			}
-			if ((*i)->cd_info.find("string_composer") != (*i)->cd_info.end()) {
-			  out  << "     COMPOSER \"" << (*i)->cd_info["string_composer"] << "\"" << endl;
-			}
-
-			if ((*i)->cd_info.find("isrc") != (*i)->cd_info.end()) {			  
-			  out  << "     ISRC \"";
-			  out << (*i)->cd_info["isrc"].substr(0,2) << "-";
-			  out << (*i)->cd_info["isrc"].substr(2,3) << "-";
-			  out << (*i)->cd_info["isrc"].substr(5,2) << "-";
-			  out << (*i)->cd_info["isrc"].substr(7,5) << "\"" << endl;
-			}
-
-			out << "  }" << endl << "}" << endl;
-
-			frames_to_cd_frames_string (buf, last_end_time - spec.start_frame, session->frame_rate());
-			out << "FILE \"" << path << "\" " << buf;
-
-			frames_to_cd_frames_string (buf, (*i)->end() - last_end_time, session->frame_rate());
-			out << buf << endl;
-
-			frames_to_cd_frames_string (buf, (*i)->start() - last_end_time, session->frame_rate());
-			out << "START" << buf << endl;
-			
-			last_start_time = (*i)->start();
-			last_end_time = (*i)->end();
-		 
-
-		      } else  if ((*i)->start() < last_end_time) {
-			/* this is an index within a track */
-			
-			frames_to_cd_frames_string (buf, (*i)->start() - last_start_time, session->frame_rate());
-			out << "INDEX" << buf << endl;
-		      }
 		}
 	}
 	
@@ -783,7 +812,7 @@ ExportDialog::export_cue_file (Locations::LocationList& locations, const string&
 		  out << "FILE " << path  << ' ' << (header_format_combo.get_active_text()) << endl;
 	}
 
-	if (numtracks == 0) {
+	if (false && numtracks == 0) {
 		    /* the user has supplied no track markers.
 		       the entire export is treated as one track. 
 		    */
@@ -814,58 +843,78 @@ ExportDialog::export_cue_file (Locations::LocationList& locations, const string&
 	if (temp.size()) {
 		LocationSortByStart cmp;
 		temp.sort (cmp);
+		Location * curr_range = 0;
+		Locations::LocationList::iterator nexti;
 
 		for ( i = temp.begin(); i != temp.end(); ++i) {
 
-		    if (!(*i)->is_mark() && ((*i)->start() >= last_track_end)) {
-		      /* this is a track and it doesn't start inside another one*/
-		      
-		      tracknum++;
-		      indexnum = 0;
-		      out << endl << "TRACK " << tracknum << " AUDIO" << endl;
-		      out << "FLAGS " ;
-		      
-		      if ((*i)->cd_info.find("scms") != (*i)->cd_info.end())  {
-			out << "SCMS ";
-		      } else {
-			out << "DCP ";
-		      }
-		      
-		      if ((*i)->cd_info.find("preemph") != (*i)->cd_info.end())  {
-			out << "PRE";
-		      }
-		      out << endl;
-		      
-		      if ((*i)->cd_info.find("isrc") != (*i)->cd_info.end())  {
-			out << "ISRC " << (*i)->cd_info["isrc"] << endl;
-			
-		      }
-		      if ((*i)->name() != "") {
-			out << "TITLE \"" << (*i)->name() << "\"" << endl;
-		      }	      
-		      
-		      if ((*i)->cd_info.find("performer") != (*i)->cd_info.end()) {
-			out << "PERFORMER \"" <<  (*i)->cd_info["performer"] << "\"" << endl;
-		      }
-		      
-		      if ((*i)->cd_info.find("string_composer") != (*i)->cd_info.end()) {
-			out << "SONGWRITER \"" << (*i)->cd_info["string_composer"]  << "\"" << endl;
-		      }
-			snprintf (buf, sizeof(buf), "INDEX %02d", indexnum);
-			out << buf;
-			frames_to_cd_frames_string (buf, last_track_end - spec.start_frame, session->frame_rate());
-			out << buf << endl;
-			indexnum++;
-			last_track_end = (*i)->end();
-		    } 
-		    if ((tracknum > 0) && ((*i)->start() < last_track_end)) {
-		      /*this is an index and it lies within a track*/
-		      snprintf (buf, sizeof(buf), "INDEX %02d", indexnum);
-		      out << buf;
-		      frames_to_cd_frames_string (buf,(*i)->start() - spec.start_frame, session->frame_rate());
-		      out << buf << endl;
-		      indexnum++;
-		    }
+			if ((*i)->start() >= last_track_end)
+			{
+				/* this is a track and it doesn't start inside another one*/
+				
+				tracknum++;
+				indexnum = 0;
+				out << endl << "TRACK " << tracknum << " AUDIO" << endl;
+				out << "FLAGS " ;
+				
+				if ((*i)->cd_info.find("scms") != (*i)->cd_info.end())  {
+					out << "SCMS ";
+				} else {
+					out << "DCP ";
+				}
+				
+				if ((*i)->cd_info.find("preemph") != (*i)->cd_info.end())  {
+					out << "PRE";
+				}
+				out << endl;
+				
+				if ((*i)->cd_info.find("isrc") != (*i)->cd_info.end())  {
+					out << "ISRC " << (*i)->cd_info["isrc"] << endl;
+					
+				}
+				if ((*i)->name() != "") {
+					out << "TITLE \"" << (*i)->name() << "\"" << endl;
+				}	      
+				
+				if ((*i)->cd_info.find("performer") != (*i)->cd_info.end()) {
+					out << "PERFORMER \"" <<  (*i)->cd_info["performer"] << "\"" << endl;
+				}
+				
+				if ((*i)->cd_info.find("string_composer") != (*i)->cd_info.end()) {
+					out << "SONGWRITER \"" << (*i)->cd_info["string_composer"]  << "\"" << endl;
+				}
+				snprintf (buf, sizeof(buf), "INDEX %02d", indexnum);
+				out << buf;
+				frames_to_cd_frames_string (buf, last_track_end - spec.start_frame, session->frame_rate());
+				out << buf << endl;
+				indexnum++;
+
+				if ((*i)->is_mark()) {
+					// need to find the next start to define the end
+					nexti = i;
+					++nexti;
+					if (nexti != temp.end()) {
+						last_track_end = (*nexti)->start();
+					}
+					else {
+						last_track_end = spec.end_frame;
+					}
+					curr_range = 0;
+				}
+				else {
+					last_track_end = (*i)->end();
+					curr_range = (*i);
+				}
+			} 
+	
+			if ((tracknum > 0) && ((*i)->start() < last_track_end)) {
+				/*this is an index and it lies within a track*/
+				snprintf (buf, sizeof(buf), "INDEX %02d", indexnum);
+				out << buf;
+				frames_to_cd_frames_string (buf,(*i)->start() - spec.start_frame, session->frame_rate());
+				out << buf << endl;
+				indexnum++;
+			}
 		}
 	}
 	
@@ -886,12 +935,24 @@ void
 ExportDialog::do_export ()
 {
 	string filepath = file_entry.get_text();
- 	
+
+	if (!ARDOUR_UI::instance()->the_engine().connected()) {
+		MessageDialog msg (*this, 
+				   _("Not connected to audioengine"),
+				   true,
+				   MESSAGE_ERROR,
+				   BUTTONS_OK);
+		msg.set_secondary_text (_("Ardour cannot export audio when disconnected"));
+		msg.present ();
+		msg.run ();
+		return;
+	}
+		
 	if(!is_filepath_valid(filepath)){
 		return;
 	}
 
-	if (export_cd_markers_allowed) {
+	if (!Profile->get_sae() && export_cd_markers_allowed) {
 		if (cue_file_combo.get_active_text () != _("None")) {
 			do_export_cd_markers (file_entry.get_text(), cue_file_combo.get_active_text ());
 		}
@@ -1185,10 +1246,12 @@ ExportDialog::initSpec(string &filepath)
 	spec.format = 0;
 
 	spec.format |= sndfile_header_format_from_string (header_format_combo.get_active_text ());
-	
-	if ((spec.format & SF_FORMAT_WAV) == 0) {
-		/* RIFF/WAV specifies endianess */
-		spec.format |= sndfile_endian_format_from_string (endian_format_combo.get_active_text ());
+
+	if (!Profile->get_sae()) {
+		if ((spec.format & SF_FORMAT_WAV) == 0) {
+			/* RIFF/WAV specifies endianess */
+			spec.format |= sndfile_endian_format_from_string (endian_format_combo.get_active_text ());
+		}
 	}
 
 	spec.format |= sndfile_bitdepth_format_from_string (bitdepth_format_combo.get_active_text ());
@@ -1210,17 +1273,21 @@ ExportDialog::initSpec(string &filepath)
 		spec.sample_rate = session->frame_rate();
 	}
 	
-	string src_str = src_quality_combo.get_active_text();
-	if (src_str == _("fastest")) {
-		spec.src_quality = SRC_ZERO_ORDER_HOLD;
-	} else if (src_str == _("linear")) {
-		spec.src_quality = SRC_LINEAR;
-	} else if (src_str == _("better")) {
-		spec.src_quality = SRC_SINC_FASTEST;
-	} else if (src_str == _("intermediate")) {
-		spec.src_quality = SRC_SINC_MEDIUM_QUALITY;
-	} else {
+	if (Profile->get_sae()) {
 		spec.src_quality = SRC_SINC_BEST_QUALITY;
+	} else {
+		string src_str = src_quality_combo.get_active_text();
+		if (src_str == _("fastest")) {
+			spec.src_quality = SRC_ZERO_ORDER_HOLD;
+		} else if (src_str == _("linear")) {
+			spec.src_quality = SRC_LINEAR;
+		} else if (src_str == _("better")) {
+			spec.src_quality = SRC_SINC_FASTEST;
+		} else if (src_str == _("intermediate")) {
+			spec.src_quality = SRC_SINC_MEDIUM_QUALITY;
+		} else {
+			spec.src_quality = SRC_SINC_BEST_QUALITY;
+		}
 	}
 
 	string dither_str = dither_type_combo.get_active_text();
