@@ -73,7 +73,8 @@ GainMeter::GainMeter (boost::shared_ptr<IO> io, Session& s)
 	  // 0.781787 is the value needed for gain to be set to 0.
 	  gain_adjustment (0.781787, 0.0, 1.0, 0.01, 0.1),
 	  gain_automation_style_button (""),
-	  gain_automation_state_button ("")
+	  gain_automation_state_button (""),
+	  regular_meter_width(5)
 	
 {
 	if (slider == 0) {
@@ -140,18 +141,16 @@ GainMeter::GainMeter (boost::shared_ptr<IO> io, Session& s)
 	hbox.set_spacing (2);
 	hbox.pack_start (*fader_vbox, true, true);
 
-	set_width(Narrow);
-
 	Route* r;
 
 	if ((r = dynamic_cast<Route*> (_io.get())) != 0) {
 
-	        /* 
+		/* 
 		   if we have a non-hidden route (ie. we're not the click or the auditioner), 
 		   pack some route-dependent stuff.
 		*/
 
-	        gain_display_box.pack_end (peak_display, true, true);
+		gain_display_box.pack_end (peak_display, true, true);
 		hbox.pack_end (meter_packer, true, true);
 
 		if (!r->hidden()) {
@@ -207,22 +206,15 @@ GainMeter::GainMeter (boost::shared_ptr<IO> io, Session& s)
 	ResetGroupPeakDisplays.connect (mem_fun(*this, &GainMeter::reset_group_peak_display));
 
 	UI::instance()->theme_changed.connect (mem_fun(*this, &GainMeter::on_theme_changed));
+	ColorsChanged.connect (mem_fun (*this, &GainMeter::color_handler));
+	//hide_all();
 }
 
 void
-GainMeter::set_width (Width w)
+GainMeter::set_width (Width w, int len)
 {
-	switch (w) {
-	case Wide:
-		peak_display.show();
-		break;
-	case Narrow:
-		peak_display.hide();
-		break;
-	}
-
 	_width = w;
-	setup_meters ();
+	setup_meters (len);
 }
 
 Glib::RefPtr<Gdk::Pixmap>
@@ -406,7 +398,7 @@ GainMeter::hide_all_meters ()
 }
 
 void
-GainMeter::setup_meters ()
+GainMeter::setup_meters (int len)
 {
 	uint32_t nmeters = _io->n_outputs();
 	guint16 width;
@@ -449,17 +441,23 @@ GainMeter::setup_meters ()
 
 	/* pack them backwards */
 
-	if (_width == Wide) {
-	        meter_packer.pack_end (meter_metric_area, false, false);
-		meter_metric_area.show_all ();
-	}
+	meter_packer.pack_end (meter_metric_area, false, false);
+	meter_metric_area.show_all ();
+
+	int b = ARDOUR_UI::config()->canvasvar_MeterColorBase.get();
+	int m = ARDOUR_UI::config()->canvasvar_MeterColorMid.get();
+	int t = ARDOUR_UI::config()->canvasvar_MeterColorTop.get();
+	int c = ARDOUR_UI::config()->canvasvar_MeterColorClip.get();
+
+	//cerr << "GainMeter::setup_meters() called color_changed = " << color_changed << " colors: " << hex << b << " " << m << " " << t << " " << c << endl;//DEBUG
 
 	for (int32_t n = nmeters-1; nmeters && n >= 0 ; --n) {
-		if (meters[n].width != width) {
+		if (meters[n].width != width || meters[n].length != len || color_changed) {
 			delete meters[n].meter;
-			meters[n].meter = new FastMeter ((uint32_t) floor (Config->get_meter_hold()), width, FastMeter::Vertical);
+			meters[n].meter = new FastMeter ((uint32_t) floor (Config->get_meter_hold()), width, FastMeter::Vertical, len, b, m, t, c);
+			//cerr << "GainMeter::setup_meters() w:l = " << width << ":" << len << endl;//DEBUG
 			meters[n].width = width;
-
+			meters[n].length = len;
 			meters[n].meter->add_events (Gdk::BUTTON_RELEASE_MASK);
 			meters[n].meter->signal_button_release_event().connect (bind (mem_fun(*this, &GainMeter::meter_button_release), n));
 		}
@@ -468,6 +466,7 @@ GainMeter::setup_meters ()
 		meters[n].meter->show_all ();
 		meters[n].packed = true;
 	}
+	color_changed = false;
 }	
 
 int
@@ -494,9 +493,9 @@ GainMeter::peak_button_release (GdkEventButton* ev)
 {
 	/* reset peak label */
 
-	if (ev->button == 1 && Keyboard::modifier_state_equals (ev->state, Keyboard::Control|Keyboard::Shift)) {
+	if (ev->button == 1 && Keyboard::modifier_state_equals (ev->state, Keyboard::PrimaryModifier|Keyboard::TertiaryModifier)) {
 		ResetAllPeakDisplays ();
-	} else if (ev->button == 1 && Keyboard::modifier_state_equals (ev->state, Keyboard::Control)) {
+	} else if (ev->button == 1 && Keyboard::modifier_state_equals (ev->state, Keyboard::PrimaryModifier)) {
 		Route* r;
 		if ((r = dynamic_cast<Route*> (_io.get())) != 0) {
 			ResetGroupPeakDisplays (r->mix_group());
@@ -706,10 +705,10 @@ GainMeter::meter_press(GdkEventButton* ev)
 
 			if (ev->button == 2) {
 
-				// ctrl-button2 click is the midi binding click
+				// Primary-button2 click is the midi binding click
 				// button2-click is "momentary"
 				
-				if (!Keyboard::modifier_state_equals (ev->state, Keyboard::ModifierMask (Keyboard::Control))) {
+				if (!Keyboard::modifier_state_equals (ev->state, Keyboard::ModifierMask (Keyboard::PrimaryModifier))) {
 					wait_for_release = true;
 					old_meter_point = _route->meter_point ();
 				}
@@ -717,9 +716,9 @@ GainMeter::meter_press(GdkEventButton* ev)
 
 			if (ev->button == 1 || ev->button == 2) {
 
-				if (Keyboard::modifier_state_equals (ev->state, Keyboard::ModifierMask (Keyboard::Control|Keyboard::Shift))) {
+				if (Keyboard::modifier_state_equals (ev->state, Keyboard::ModifierMask (Keyboard::PrimaryModifier|Keyboard::TertiaryModifier))) {
 
-					/* ctrl-shift-click applies change to all routes */
+					/* Primary+Tertiary-click applies change to all routes */
 
 					_session.begin_reversible_command (_("meter point change"));
                                         Session::GlobalMeteringStateCommand *cmd = new Session::GlobalMeteringStateCommand (_session, this);
@@ -729,10 +728,10 @@ GainMeter::meter_press(GdkEventButton* ev)
 					_session.commit_reversible_command ();
 					
 					
-				} else if (Keyboard::modifier_state_equals (ev->state, Keyboard::Control)) {
+				} else if (Keyboard::modifier_state_equals (ev->state, Keyboard::PrimaryModifier)) {
 
-					/* ctrl-click: solo mix group.
-					   ctrl-button2 is MIDI learn.
+					/* Primary-click: solo mix group.
+					   NOTE: Primary-button2 is MIDI learn.
 					*/
 					
 					if (ev->button == 1) {
@@ -958,3 +957,16 @@ GainMeter::gain_automation_state_changed ()
 		gain_watching = ARDOUR_UI::RapidScreenUpdate.connect (mem_fun (*this, &GainMeter::effective_gain_display));
 	}
 }
+
+void GainMeter::clear_meters ()
+{
+	for (vector<MeterInfo>::iterator i = meters.begin(); i < meters.end(); i++) {
+		(*i).meter->clear();
+	}
+}
+
+void GainMeter::color_handler()
+{
+	color_changed = true;
+	setup_meters();
+}
