@@ -42,6 +42,7 @@
 #include <ardour/panner.h>
 #include <ardour/send.h>
 #include <ardour/insert.h>
+#include <ardour/profile.h>
 #include <ardour/ladspa_plugin.h>
 #include <ardour/connection.h>
 #include <ardour/session_connection.h>
@@ -402,10 +403,6 @@ MixerStrip::set_width (Width w, void* owner)
 
 	_width_owner = owner;
 
-	if (_width == w) {
-		return;
-	}
-
 	ensure_xml_node ();
 	
 	_width = w;
@@ -760,12 +757,14 @@ void
 MixerStrip::input_changed (IOChange change, void *src)
 {
 	Gtkmm2ext::UI::instance()->call_slot (mem_fun(*this, &MixerStrip::update_input_display));
+	set_width(_width, this);
 }
 
 void
 MixerStrip::output_changed (IOChange change, void *src)
 {
 	Gtkmm2ext::UI::instance()->call_slot (mem_fun(*this, &MixerStrip::update_output_display));
+	set_width(_width, this);
 }
 
 
@@ -992,7 +991,9 @@ MixerStrip::build_route_ops_menu ()
 	build_remote_control_menu ();
 	
 	items.push_back (SeparatorElem());
-	items.push_back (MenuElem (_("Remote Control ID"), *remote_control_menu));
+	if (!Profile->get_sae()) {
+              items.push_back (MenuElem (_("Remote Control ID"), *remote_control_menu));
+        }
 
 	items.push_back (SeparatorElem());
 	items.push_back (MenuElem (_("Remove"), mem_fun(*this, &RouteUI::remove_this_route)));
@@ -1222,5 +1223,6 @@ MixerStrip::meter_changed (void *src)
 		}
 
 		gpm.setup_meters ();
+		set_width(_width, this);
 }
 
