@@ -64,7 +64,7 @@ Mixer_UI::Mixer_UI ()
 	: Window (Gtk::WINDOW_TOPLEVEL)
 {
 	session = 0;
-	_strip_width = Wide;
+	Config->get_default_narrow_ms() ? _strip_width = Narrow : _strip_width = Wide;
 	track_menu = 0;
 	mix_group_context_menu = 0;
 	no_track_list_redisplay = false;
@@ -202,10 +202,6 @@ Mixer_UI::Mixer_UI ()
 	list_hpane.signal_size_allocate().connect (bind (mem_fun(*this, &Mixer_UI::pane_allocation_handler), 
 							 static_cast<Gtk::Paned*> (&list_hpane)));
 	
-
-	rhs_pane1.set_data ("collapse-direction", (gpointer) 0);
-	list_hpane.set_data ("collapse-direction", (gpointer) 1);
-
 	global_vpacker.pack_start (list_hpane, true, true);
 
 	add (global_vpacker);
@@ -286,6 +282,8 @@ Mixer_UI::add_strip (Session::RouteList& routes)
 		strip = new MixerStrip (*this, *session, route);
 		strips.push_back (strip);
 
+		Config->get_default_narrow_ms() ? _strip_width = Narrow : _strip_width = Wide;
+
 		if (strip->width_owner() != strip) {
 			strip->set_width (_strip_width, this);
 		}
@@ -327,6 +325,14 @@ Mixer_UI::remove_strip (MixerStrip* strip)
 		strips.erase (i);
 	}
 
+	/* Decrement old order keys for strips `above' the one that is being removed */
+	for (ri = rows.begin(); ri != rows.end(); ++ri) {
+		MixerStrip* s = (*ri)[track_columns.strip];
+		if (s->old_order_key() > strip->old_order_key()) {
+			s->set_old_order_key (s->old_order_key() - 1);
+		}
+	}
+
 	for (ri = rows.begin(); ri != rows.end(); ++ri) {
 		if ((*ri)[track_columns.strip] == strip) {
 			track_model->erase (ri);
@@ -394,7 +400,7 @@ Mixer_UI::strip_button_release_event (GdkEventButton *ev, MixerStrip *strip)
 		if (_selection.selected (strip->route())) {
 			_selection.remove (strip->route());
 		} else {
-			if (Keyboard::modifier_state_equals (ev->state, Keyboard::Shift)) {
+			if (Keyboard::modifier_state_equals (ev->state, Keyboard::TertiaryModifier)) {
 				_selection.add (strip->route());
 			} else {
 				_selection.set (strip->route());
