@@ -78,7 +78,7 @@ class PublicEditor : public Gtk::Window, public PBD::StatefulThingWithGoingAway
 	PublicEditor();
 	virtual ~PublicEditor();
 
-	typedef list<TimeAxisView *> TrackViewList;
+	typedef TrackSelection TrackViewList;
 
 	static PublicEditor& instance() { return *_instance; }
 
@@ -106,11 +106,11 @@ class PublicEditor : public Gtk::Window, public PBD::StatefulThingWithGoingAway
 	virtual void separate_region_from_selection () = 0;
 	virtual void toggle_playback (bool with_abort) = 0;
 	virtual void transition_to_rolling (bool fwd) = 0;
-	virtual nframes_t unit_to_frame (double unit) = 0;
-	virtual double frame_to_unit (nframes_t frame) = 0;
-	virtual double frame_to_unit (double frame) = 0;
-	virtual nframes_t pixel_to_frame (double pixel) = 0;
-	virtual gulong frame_to_pixel (nframes_t frame) = 0;
+	virtual nframes_t unit_to_frame (double unit) const = 0;
+	virtual double frame_to_unit (nframes_t frame) const = 0;
+	virtual double frame_to_unit (double frame) const = 0;
+	virtual nframes64_t pixel_to_frame (double pixel) const = 0;
+	virtual gulong frame_to_pixel (nframes64_t frame) const = 0;
 	virtual Selection& get_selection() const = 0;
 	virtual Selection& get_cut_buffer() const = 0;
 	virtual bool extend_selection_to_track (TimeAxisView&) = 0;
@@ -125,15 +125,15 @@ class PublicEditor : public Gtk::Window, public PBD::StatefulThingWithGoingAway
 	virtual Gtk::HBox& get_status_bar_packer()  = 0;
 	virtual void      set_zoom_focus (Editing::ZoomFocus) = 0;
 	virtual Editing::ZoomFocus get_zoom_focus () const = 0;
-	virtual gdouble   get_current_zoom () = 0;
+	virtual gdouble   get_current_zoom () const = 0;
 	virtual PlaylistSelector& playlist_selector() const = 0;
 	virtual void route_name_changed (TimeAxisView *) = 0;
 	virtual void clear_playlist (boost::shared_ptr<ARDOUR::Playlist>) = 0;
-	virtual void new_playlists () = 0;
-	virtual void copy_playlists () = 0;
-	virtual void clear_playlists () = 0;
+	virtual void new_playlists (TimeAxisView*) = 0;
+	virtual void copy_playlists (TimeAxisView*) = 0;
+	virtual void clear_playlists (TimeAxisView*) = 0;
 	virtual void select_all_tracks () = 0;
-	virtual bool set_selected_track (TimeAxisView&, Selection::Operation op = Selection::Set, bool no_remove = false) = 0;
+	virtual void set_selected_track (TimeAxisView&, Selection::Operation op = Selection::Set, bool no_remove = false) = 0;
 	virtual void set_selected_mixer_strip (TimeAxisView&) = 0;
 	virtual void hide_track_in_display (TimeAxisView& tv) = 0;
 	virtual void show_track_in_display (TimeAxisView& tv) = 0;
@@ -155,7 +155,9 @@ class PublicEditor : public Gtk::Window, public PBD::StatefulThingWithGoingAway
 	virtual void remove_last_capture () = 0;
 	virtual void maximise_editing_space() = 0;
 	virtual void restore_editing_space() = 0;
-	virtual nframes_t edit_cursor_position(bool sync) = 0;
+	virtual nframes64_t get_preferred_edit_position (bool ignore_playhead = false) = 0;
+	virtual void toggle_meter_updating() = 0;
+	virtual void split_region_at_points (boost::shared_ptr<ARDOUR::Region>, ARDOUR::AnalysisFeatureList&) = 0;
 
 	sigc::signal<void> ZoomFocusChanged;
 	sigc::signal<void> ZoomChanged;
