@@ -38,6 +38,8 @@
 #include "enums.h"
 #include "time_axis_view.h"
 #include "canvas.h"
+#include "level_meter.h"
+
 
 namespace ARDOUR {
 	class Session;
@@ -80,6 +82,7 @@ public:
 	void get_inverted_selectables (Selection&, list<Selectable*>&);
 		
 	boost::shared_ptr<ARDOUR::Region> find_next_region (nframes_t pos, ARDOUR::RegionPoint, int32_t dir);
+	nframes64_t find_next_region_boundary (nframes64_t pos, int32_t dir);
 
 	/* Editing operations */
 	bool cut_copy_clear (Selection&, Editing::CutCopyOp);
@@ -98,6 +101,13 @@ public:
 	StreamView*         view() const { return _view; }
 	ARDOUR::RouteGroup* edit_group() const;
 	boost::shared_ptr<ARDOUR::Playlist> playlist() const;
+	void fast_update ();
+	void hide_meter ();
+	void show_meter ();
+	void reset_meter ();
+	void clear_meter ();
+	void io_changed (ARDOUR::IOChange, void *);
+	void meter_changed (void *);
 
 protected:
 	friend class StreamView;
@@ -243,6 +253,8 @@ protected:
 	sigc::connection modified_connection;
 
 	void post_construct ();
+	
+	LevelMeter	lm;
 };
 
 #endif /* __ardour_route_time_axis_h__ */
