@@ -43,6 +43,7 @@
 #include "public_editor.h"
 #include "time_axis_view.h"
 #include "simplerect.h"
+#include "simpleline.h"
 #include "selection.h"
 #include "keyboard.h"
 #include "rgb_macros.h"
@@ -71,7 +72,7 @@ bool TimeAxisView::need_size_info = true;
 TimeAxisView::TimeAxisView (ARDOUR::Session& sess, PublicEditor& ed, TimeAxisView* rent, Canvas& canvas) 
 	: AxisView (sess), 
 	  editor (ed),
-	  controls_table (2, 8)
+	  controls_table (2, 7)
 {
 	if (need_size_info) {
 		compute_controls_size_info ();
@@ -125,7 +126,7 @@ TimeAxisView::TimeAxisView (ARDOUR::Session& sess, PublicEditor& ed, TimeAxisVie
 	controls_table.set_col_spacings (0);
 	controls_table.set_homogeneous (true);
 
-	controls_table.attach (name_hbox, 0, 4, 0, 1, Gtk::FILL|Gtk::EXPAND, Gtk::FILL|Gtk::EXPAND);
+	controls_table.attach (name_hbox, 0, 4, 0, 1,  Gtk::FILL|Gtk::EXPAND,  Gtk::FILL|Gtk::EXPAND, 3, 0);
 	controls_table.show_all ();
 	controls_table.set_no_show_all ();
 
@@ -140,8 +141,6 @@ TimeAxisView::TimeAxisView (ARDOUR::Session& sess, PublicEditor& ed, TimeAxisVie
 	controls_ebox.signal_button_release_event().connect (mem_fun (*this, &TimeAxisView::controls_ebox_button_release));
 	controls_ebox.signal_scroll_event().connect (mem_fun (*this, &TimeAxisView::controls_ebox_scroll), true);
 
-	controls_lhs_pad.set_name ("TimeAxisViewControlsPadding");
-	controls_hbox.pack_start (controls_lhs_pad,false,false);
 	controls_hbox.pack_start (controls_ebox,true,true);
 	controls_hbox.show ();
 
@@ -248,14 +247,14 @@ TimeAxisView::controls_ebox_scroll (GdkEventScroll* ev)
 {
 	switch (ev->direction) {
 	case GDK_SCROLL_UP:
-		if (Keyboard::modifier_state_equals (ev->state, Keyboard::Shift)) {
+		if (Keyboard::modifier_state_equals (ev->state, Keyboard::TertiaryModifier)) {
 			step_height (true);
 			return true;
 		}
 		break;
 		
 	case GDK_SCROLL_DOWN:
-		if (Keyboard::modifier_state_equals (ev->state, Keyboard::Shift)) {
+		if (Keyboard::modifier_state_equals (ev->state, Keyboard::TertiaryModifier)) {
 			step_height (false);
 			return true;
 		}
@@ -353,6 +352,16 @@ TimeAxisView::step_height (bool bigger)
        }
 }
 
+void
+TimeAxisView::set_heights (TrackHeight h)
+{
+	TrackSelection& ts (editor.get_selection().tracks);
+
+	for (TrackSelection::iterator i = ts.begin(); i != ts.end(); ++i) {
+		(*i)->set_height (h);
+	}
+}
+
 void
 TimeAxisView::set_height (TrackHeight h)
 {
@@ -365,11 +374,12 @@ TimeAxisView::set_height_pixels (uint32_t h)
 {
 	height = h;
 	controls_frame.set_size_request (-1, height + ((order == 0) ? 1 : 0));
-
+	//cerr << "TimeAxisView::set_height_pixels() called h = " << h << endl;//DEBUG
  	if (canvas_item_visible (selection_group)) {
 		/* resize the selection rect */
 		show_selection (editor.get_selection().time);
 	}
+	
 }
 
 bool
@@ -519,10 +529,7 @@ TimeAxisView::popup_display_menu (guint32 when)
 		build_display_menu ();
 	}
 
-	if (!get_selected()) {
-		editor.set_selected_track (*this, Selection::Set);
-	}
-
+	editor.set_selected_track (*this, Selection::Add);
 	display_menu->popup (1, when);	
 }
 
@@ -546,12 +553,16 @@ TimeAxisView::popup_size_menu (guint32 when)
 void
 TimeAxisView::set_selected (bool yn)
 {
-	AxisView::set_selected (yn);
+	if (yn == _selected) {
+		return;
+	}
+	
+	Selectable::set_selected (yn);
 
 	if (_selected) {
 		controls_ebox.set_name (controls_base_selected_name);
 		controls_frame.set_name (controls_base_selected_name);
-
+		
 		/* propagate any existing selection, if the mode is right */
 
 		if (editor.current_mouse_mode() == Editing::MouseRange && !editor.get_selection().time.empty()) {
@@ -572,8 +583,6 @@ TimeAxisView::set_selected (bool yn)
 		for (vector<TimeAxisView*>::iterator i = children.begin(); i != children.end(); ++i) {
 			(*i)->set_selected (false);
 		}
-
-		
 	}
 }
 
@@ -586,12 +595,12 @@ TimeAxisView::build_size_menu ()
 	size_menu->set_name ("ArdourContextMenu");
 	MenuList& items = size_menu->items();
 	
-	items.push_back (MenuElem (_("Largest"), bind (mem_fun (*this, &TimeAxisView::set_height), Largest)));
-	items.push_back (MenuElem (_("Large"), bind (mem_fun (*this, &TimeAxisView::set_height), Large)));
-	items.push_back (MenuElem (_("Larger"), bind (mem_fun (*this, &TimeAxisView::set_height), Larger)));
-	items.push_back (MenuElem (_("Normal"), bind (mem_fun (*this, &TimeAxisView::set_height), Normal)));
-	items.push_back (MenuElem (_("Smaller"), bind (mem_fun (*this, &TimeAxisView::set_height),Smaller)));
-	items.push_back (MenuElem (_("Small"), bind (mem_fun (*this, &TimeAxisView::set_height), Small)));
+	items.push_back (MenuElem (_("Largest"), bind (mem_fun (*this, &TimeAxisView::set_heights), Largest)));
+	items.push_back (MenuElem (_("Large"), bind (mem_fun (*this, &TimeAxisView::set_heights), Large)));
+	items.push_back (MenuElem (_("Larger"), bind (mem_fun (*this, &TimeAxisView::set_heights), Larger)));
+	items.push_back (MenuElem (_("Normal"), bind (mem_fun (*this, &TimeAxisView::set_heights), Normal)));
+	items.push_back (MenuElem (_("Smaller"), bind (mem_fun (*this, &TimeAxisView::set_heights),Smaller)));
+	items.push_back (MenuElem (_("Small"), bind (mem_fun (*this, &TimeAxisView::set_heights), Small)));
 }
 
 void
@@ -1101,3 +1110,37 @@ TimeAxisView::covers_y_position (double y)
 
 	return 0;
 }
+
+void
+TimeAxisView::show_temporary_lines (const AnalysisFeatureList& pos)
+{
+	while (temp_lines.size()< pos.size()) {
+		ArdourCanvas::SimpleLine* l = new ArdourCanvas::SimpleLine (*canvas_display);
+		l->property_color_rgba() = (guint) ARDOUR_UI::config()->canvasvar_ZeroLine.get();
+		l->property_y1() = 0;
+		l->property_y2() = height;
+		temp_lines.push_back (l);
+	}
+
+	while (temp_lines.size() > pos.size()) {
+		ArdourCanvas::SimpleLine *line = temp_lines.back();
+		temp_lines.pop_back ();
+		delete line;
+	}
+
+	AnalysisFeatureList::const_iterator i;
+	list<ArdourCanvas::SimpleLine*>::iterator l;
+
+	for (i = pos.begin(), l = temp_lines.begin(); i != pos.end() && l != temp_lines.end(); ++i, ++l) {
+		(*l)->property_x1() = editor.frame_to_pixel (*i);
+		(*l)->property_x2() = editor.frame_to_pixel (*i);
+	}
+}
+
+void
+TimeAxisView::hide_temporary_lines ()
+{
+	for (list<ArdourCanvas::SimpleLine*>::iterator l = temp_lines.begin(); l != temp_lines.end(); ++l) {
+		(*l)->hide ();
+	}
+}
