@@ -27,6 +27,7 @@
 #include <pbd/statefuldestructible.h> 
 
 #include <ardour/ardour.h>
+#include <ardour/readable.h>
 
 class XMLNode;
 
@@ -40,7 +41,7 @@ enum RegionEditState {
 	EditChangesID      = 2
 };
 
-class Region : public PBD::StatefulDestructible, public boost::enable_shared_from_this<Region>
+class Region : public PBD::StatefulDestructible, public Readable, public boost::enable_shared_from_this<Region>
 {
   public:
 	enum Flag {
@@ -66,6 +67,11 @@ class Region : public PBD::StatefulDestructible, public boost::enable_shared_fro
 		range_guarantoor = USHRT_MAX
 	};
 
+	enum PositionLockStyle {
+		AudioTime,
+		MusicTime
+	};
+
  	static const Flag DefaultFlags = Flag (Opaque|DefaultFadeIn|DefaultFadeOut|FadeIn|FadeOut);
 
 	static Change FadeChanged;
@@ -88,8 +94,20 @@ class Region : public PBD::StatefulDestructible, public boost::enable_shared_fro
 	nframes_t position () const { return _position; }
 	nframes_t start ()    const { return _start; }
 	nframes_t length()    const { return _length; }
-	layer_t        layer ()    const { return _layer; }
-	
+	layer_t   layer ()    const { return _layer; }
+
+	/* these two are valid ONLY during a StateChanged signal handler */
+
+	nframes_t last_position() const { return _last_position; }
+	nframes_t last_length() const { return _last_length; }
+
+	nframes64_t ancestral_start () const { return _ancestral_start; }
+	nframes64_t ancestral_length () const { return _ancestral_length; }
+	float stretch() const { return _stretch; }
+	float shift() const { return _shift; }
+
+	void set_ancestral_data (nframes64_t start, nframes64_t length, float stretch, float shift);
+
 	nframes_t sync_offset(int& dir) const;
 	nframes_t sync_position() const;
 
@@ -108,13 +126,16 @@ class Region : public PBD::StatefulDestructible, public boost::enable_shared_fro
 	bool whole_file() const { return _flags & WholeFile ; }
 	Flag flags()      const { return _flags; }
 
+	PositionLockStyle positional_lock_style() const { return _positional_lock_style; }
+	void set_position_lock_style (PositionLockStyle ps);
+
 	virtual bool should_save_state () const { return !(_flags & DoNotSaveState); };
 
 	void freeze ();
 	void thaw (const string& why);
 
 	bool covers (nframes_t frame) const {
-		return first_frame() <= frame && frame < last_frame();
+		return first_frame() <= frame && frame <= last_frame();
 	}
 
 	OverlapType coverage (nframes_t start, nframes_t end) const {
@@ -136,7 +157,8 @@ class Region : public PBD::StatefulDestructible, public boost::enable_shared_fro
 	void set_position (nframes_t, void *src);
 	void set_position_on_top (nframes_t, void *src);
 	void special_set_position (nframes_t);
-	void nudge_position (long, void *src);
+	void update_position_after_tempo_map_change ();
+	void nudge_position (nframes64_t, void *src);
 
 	bool at_natural_position () const;
 	void move_to_natural_position (void *src);
@@ -176,6 +198,13 @@ class Region : public PBD::StatefulDestructible, public boost::enable_shared_fro
 	uint64_t last_layer_op() const { return _last_layer_op; }
 	void set_last_layer_op (uint64_t when);
 
+	virtual int get_transients (AnalysisFeatureList&, bool force_new = false) { 
+		// no transients, but its OK
+		return 0;
+	}
+
+	void invalidate_transients ();	
+
   protected:
 	friend class RegionFactory;
 
@@ -185,40 +214,49 @@ class Region : public PBD::StatefulDestructible, public boost::enable_shared_fro
 	Region (boost::shared_ptr<const Region>);
 	Region (const XMLNode&);
 
-
   protected:
 	XMLNode& get_short_state (); /* used only by Session */
 
 	void send_change (Change);
 
 	void trim_to_internal (nframes_t position, nframes_t length, void *src);
+	void set_position_internal (nframes_t pos, bool allow_bbt_recompute);
 
 	bool copied() const { return _flags & Copied; }
 	void maybe_uncopy ();
 	void first_edit ();
 	
 	virtual bool verify_start (nframes_t) = 0;
-	virtual bool verify_start_and_length (nframes_t, nframes_t) = 0;
+	virtual bool verify_start_and_length (nframes_t, nframes_t&) = 0;
 	virtual bool verify_start_mutable (nframes_t&_start) = 0;
-	virtual bool verify_length (nframes_t) = 0;
+	virtual bool verify_length (nframes_t&) = 0;
 	virtual void recompute_at_start () = 0;
 	virtual void recompute_at_end () = 0;
 	
-	
-	nframes_t          _start;
-	nframes_t          _length;
-	nframes_t          _position;
+	nframes_t                _start;
+	nframes_t                _length;
+	nframes_t                _last_length;
+	nframes_t                _position;
+	nframes_t                _last_position;
 	Flag                     _flags;
-	nframes_t          _sync_position;
-	layer_t                 _layer;
-	string                  _name;        
-	mutable RegionEditState _first_edit;
-	int                     _frozen;
-	Glib::Mutex             lock;
+	PositionLockStyle        _positional_lock_style;
+	nframes_t                _sync_position;
+	layer_t                  _layer;
+	string                   _name;        
+	mutable RegionEditState  _first_edit;
+	int                      _frozen;
+	Glib::Mutex               lock;
 	boost::weak_ptr<ARDOUR::Playlist> _playlist;
-	mutable uint32_t        _read_data_count; // modified in read()
-	Change                   pending_changed;
-	uint64_t                _last_layer_op; // timestamp
+	mutable uint32_t         _read_data_count; // modified in read()
+	Change                    pending_changed;
+	uint64_t                 _last_layer_op; // timestamp
+	nframes64_t              _ancestral_start;
+	nframes64_t              _ancestral_length;
+	float                    _stretch;
+	float                    _shift;
+	BBT_Time                 _bbt_time;
+	AnalysisFeatureList      _transients;
+	bool valid_transients;
 };
 
 } /* namespace ARDOUR */
