@@ -30,7 +30,9 @@ namespace ARDOUR {
 class ResampledImportableSource : public ImportableSource 
 {
   public:
-	ResampledImportableSource (SNDFILE* sf, SF_INFO* info, nframes_t rate, SrcQuality);
+	ResampledImportableSource (const std::string& path,
+			nframes_t rate, SrcQuality);
+
 	~ResampledImportableSource ();
 	
 	nframes_t read (Sample* buffer, nframes_t nframes);
