@@ -125,6 +125,7 @@ class Session : public PBD::StatefulDestructible
 		    SetDiskstreamSpeed,
 		    Locate,
 		    LocateRoll,
+		    LocateRollLocate,
 		    SetLoop,
 		    PunchIn,
 		    PunchOut,
@@ -210,9 +211,9 @@ class Session : public PBD::StatefulDestructible
 	/* creating from an XML file */
 
 	Session (AudioEngine&,
-		 string fullpath,
-		 string snapshot_name,
-		 string* mix_template = 0);
+		 const string& fullpath,
+		 const string& snapshot_name,
+		 string mix_template = "");
 
 	/* creating a new Session */
 
@@ -251,6 +252,9 @@ class Session : public PBD::StatefulDestructible
 	std::string peak_dir () const;
 	std::string dead_sound_dir () const;
 	std::string automation_dir () const;
+	std::string analysis_dir() const;
+	
+	int ensure_subdirs ();
 
 	Glib::ustring peak_path (Glib::ustring) const;
 
@@ -347,7 +351,7 @@ class Session : public PBD::StatefulDestructible
 
 	sigc::signal<void,RouteList&> RouteAdded;
 
-	void request_roll ();
+	void request_roll_at_and_return (nframes_t start, nframes_t return_to);
 	void request_bounded_roll (nframes_t start, nframes_t end);
 	void request_stop (bool abort = false);
 	void request_locate (nframes_t frame, bool with_roll = false);
@@ -377,7 +381,10 @@ class Session : public PBD::StatefulDestructible
 	nframes_t get_maximum_extent () const;
 	nframes_t current_end_frame() const { return end_location->start(); }
 	nframes_t current_start_frame() const { return start_location->start(); }
+	// "actual" sample rate of session, set by current audioengine rate, pullup/down etc.
 	nframes_t frame_rate() const   { return _current_frame_rate; }
+	// "native" sample rate of session, regardless of current audioengine rate, pullup/down etc
+	nframes_t nominal_frame_rate() const   { return _nominal_frame_rate; }
 	nframes_t frames_per_hour() const { return _frames_per_hour; }
 
 	double frames_per_smpte_frame() const { return _frames_per_smpte_frame; }
@@ -493,6 +500,7 @@ class Session : public PBD::StatefulDestructible
 
 	nframes_t transport_frame () const {return _transport_frame; }
 	nframes_t audible_frame () const;
+	nframes64_t requested_return_frame() const { return _requested_return_frame; }
 
 	enum PullupFormat {
 		pullup_Plus4Plus1,
@@ -537,6 +545,9 @@ class Session : public PBD::StatefulDestructible
 	bool        transport_stopped() const { return _transport_speed == 0.0f; }
 	bool        transport_rolling() const { return _transport_speed != 0.0f; }
 
+	void set_silent (bool yn);
+	bool silent () { return _silent; }
+
 	int jack_slave_sync (nframes_t);
 
 	TempoMap& tempo_map() { return *_tempo_map; }
@@ -573,7 +584,7 @@ class Session : public PBD::StatefulDestructible
 	    
 	};
 
-	int import_audiofile (import_status&);
+	void import_audiofiles (import_status&);
 	bool sample_rate_convert (import_status&, string infile, string& outfile);
 	string build_tmp_convert_name (string file);
 
@@ -607,6 +618,12 @@ class Session : public PBD::StatefulDestructible
 	
 	sigc::signal<int,boost::shared_ptr<ARDOUR::Playlist> > AskAboutPlaylistDeletion;
 
+	/* handlers should return 0 for "ignore the rate mismatch"
+	   and !0 for "do not use this session"
+	*/
+
+	static sigc::signal<int,nframes_t, nframes_t> AskAboutSampleRateMismatch;
+
 	/* handlers should return !0 for use pending state, 0 for
 	   ignore it.
 	*/
@@ -690,6 +707,13 @@ class Session : public PBD::StatefulDestructible
 	uint32_t n_plugin_inserts() const { return _plugin_inserts.size(); }
 	uint32_t n_sends() const { return _sends.size(); }
 
+	static void set_disable_all_loaded_plugins (bool yn) { 
+		_disable_all_loaded_plugins = yn;
+	}
+	static bool get_disable_all_loaded_plugins() { 
+		return _disable_all_loaded_plugins;
+	}
+
 	uint32_t next_send_id();
 	uint32_t next_insert_id();
 	void mark_send_id (uint32_t);
@@ -845,21 +869,6 @@ class Session : public PBD::StatefulDestructible
 
 	boost::shared_ptr<IO>  click_io() { return _click_io; }
 		
-	/* tempo FX */
-
-	struct TimeStretchRequest {
-	    boost::shared_ptr<ARDOUR::AudioRegion> region;
-	    float                fraction; /* session: read ; GUI: write */
-	    float                progress; /* session: write ; GUI: read */
-	    bool                 running;  /* read/write */
-	    bool                 quick_seek; /* GUI: write */
-	    bool                 antialias;  /* GUI: write */
-
-	    TimeStretchRequest () {} 
-	};
-
-	boost::shared_ptr<AudioRegion> tempoize_region (TimeStretchRequest&);
-
 	/* disk, buffer loads */
 
 	uint32_t playback_load ();
@@ -952,7 +961,7 @@ class Session : public PBD::StatefulDestructible
 	void update_latency_compensation (bool, bool);
 	
   private:
-	int  create (bool& new_session, string* mix_template, nframes_t initial_length);
+	int  create (bool& new_session, const string& mix_template, nframes_t initial_length);
 	void destroy ();
 
 	nframes_t compute_initial_length ();
@@ -976,33 +985,36 @@ class Session : public PBD::StatefulDestructible
 
 	typedef void (Session::*process_function_type)(nframes_t);
 
-	AudioEngine            &_engine;
-	mutable gint            processing_prohibited;
+	AudioEngine&            _engine;
+	mutable gint             processing_prohibited;
 	process_function_type    process_function;
 	process_function_type    last_process_function;
 	bool                     waiting_for_sync_offset;
-	nframes_t          _base_frame_rate;
-	nframes_t          _current_frame_rate;  //this includes video pullup offset
+	nframes_t               _base_frame_rate;
+	nframes_t               _current_frame_rate;  //this includes video pullup offset
+	nframes_t               _nominal_frame_rate;  //ignores audioengine setting, "native" SR
 	int                      transport_sub_state;
-	mutable gint           _record_status;
-	nframes_t          _transport_frame;
+	mutable gint            _record_status;
+	volatile nframes_t      _transport_frame;
 	Location*                end_location;
 	Location*                start_location;
-	Slave                  *_slave;
+	Slave*                  _slave;
+	bool                    _silent;
 	volatile float          _transport_speed;
 	volatile float          _desired_transport_speed;
 	float                   _last_transport_speed;
 	bool                     auto_play_legal;
-	nframes_t          _last_slave_transport_frame;
-	nframes_t           maximum_output_latency;
-	nframes_t           last_stop_frame;
+	nframes_t               _last_slave_transport_frame;
+	nframes_t                maximum_output_latency;
+	nframes_t                last_stop_frame;
+	volatile nframes64_t    _requested_return_frame;
 	vector<Sample *>        _passthru_buffers;
 	vector<Sample *>        _silent_buffers;
 	vector<Sample *>        _send_buffers;
-	nframes_t           current_block_size;
-	nframes_t          _worst_output_latency;
-	nframes_t          _worst_input_latency;
-	nframes_t          _worst_track_latency;
+	nframes_t                current_block_size;
+	nframes_t               _worst_output_latency;
+	nframes_t               _worst_input_latency;
+	nframes_t               _worst_track_latency;
 	bool                    _have_captured;
 	float                   _meter_hold;
 	float                   _meter_falloff;
@@ -1442,7 +1454,7 @@ class Session : public PBD::StatefulDestructible
 
 	SerializedRCUManager<RouteList>  routes;
 
-	void   add_routes (RouteList&, bool save = true);
+	void   add_routes (RouteList&, bool save);
 	uint32_t destructive_index;
 
 	int load_routes (const XMLNode&);
@@ -1719,6 +1731,8 @@ class Session : public PBD::StatefulDestructible
 	
 	void set_history_depth (uint32_t depth);
 	void sync_order_keys ();
+
+	static bool _disable_all_loaded_plugins;
 };
 
 } // namespace ARDOUR
