@@ -20,6 +20,7 @@
 #include <cmath>
 #include <climits>
 #include <cfloat>
+#include <algorithm>
 
 #include <set>
 
@@ -32,6 +33,7 @@
 #include <pbd/xml++.h>
 #include <pbd/stacktrace.h>
 #include <pbd/enumwriter.h>
+#include <pbd/convert.h>
 
 #include <ardour/audioregion.h>
 #include <ardour/session.h>
@@ -41,12 +43,14 @@
 #include <ardour/audiofilter.h>
 #include <ardour/audiofilesource.h>
 #include <ardour/region_factory.h>
+#include <ardour/transient_detector.h>
 
 #include "i18n.h"
 #include <locale.h>
 
 using namespace std;
 using namespace ARDOUR;
+using namespace PBD;
 
 /* a Session will reset these to its chosen defaults by calling AudioRegion::set_default_fade() */
 
@@ -81,6 +85,7 @@ AudioRegion::AudioRegion (boost::shared_ptr<AudioSource> src, nframes_t start, n
 	set_default_envelope ();
 
 	listen_to_my_curves ();
+	listen_to_my_sources ();
 }
 
 AudioRegion::AudioRegion (boost::shared_ptr<AudioSource> src, nframes_t start, nframes_t length, const string& name, layer_t layer, Flag flags)
@@ -92,7 +97,7 @@ AudioRegion::AudioRegion (boost::shared_ptr<AudioSource> src, nframes_t start, n
 	/* basic AudioRegion constructor */
 
 	sources.push_back (src);
-	master_sources.push_back (src);
+	master_sources.push_back (src);	
 	src->GoingAway.connect (mem_fun (*this, &AudioRegion::source_deleted));
 
 	boost::shared_ptr<AudioFileSource> afs = boost::dynamic_pointer_cast<AudioFileSource> (src);
@@ -106,17 +111,18 @@ AudioRegion::AudioRegion (boost::shared_ptr<AudioSource> src, nframes_t start, n
 	set_default_envelope ();
 
 	listen_to_my_curves ();
+	listen_to_my_sources ();
 }
 
-AudioRegion::AudioRegion (SourceList& srcs, nframes_t start, nframes_t length, const string& name, layer_t layer, Flag flags)
+AudioRegion::AudioRegion (const SourceList& srcs, nframes_t start, nframes_t length, const string& name, layer_t layer, Flag flags)
 	: Region (start, length, name, layer, flags),
 	  _fade_in (0.0, 2.0, 1.0, false),
 	  _fade_out (0.0, 2.0, 1.0, false),
 	  _envelope (0.0, 2.0, 1.0, false)
 {
 	/* basic AudioRegion constructor */
-
-	for (SourceList::iterator i=srcs.begin(); i != srcs.end(); ++i) {
+	
+	for (SourceList::const_iterator i=srcs.begin(); i != srcs.end(); ++i) {
 		sources.push_back (*i);
 		master_sources.push_back (*i);
 		(*i)->GoingAway.connect (mem_fun (*this, &AudioRegion::source_deleted));
@@ -133,6 +139,7 @@ AudioRegion::AudioRegion (SourceList& srcs, nframes_t start, nframes_t length, c
 	set_default_envelope ();
 
 	listen_to_my_curves ();
+	listen_to_my_sources ();
 }
 
 
@@ -167,6 +174,7 @@ AudioRegion::AudioRegion (boost::shared_ptr<const AudioRegion> other, nframes_t
 			(*i)->GoingAway.connect (mem_fun (*this, &AudioRegion::source_deleted));
 		}
 		master_sources.push_back (*i);
+
 	}
 
 	/* return to default fades if the existing ones are too long */
@@ -197,6 +205,7 @@ AudioRegion::AudioRegion (boost::shared_ptr<const AudioRegion> other, nframes_t
 	_scale_amplitude = other->_scale_amplitude;
 
 	listen_to_my_curves ();
+	listen_to_my_sources ();
 }
 
 AudioRegion::AudioRegion (boost::shared_ptr<const AudioRegion> other)
@@ -238,6 +247,7 @@ AudioRegion::AudioRegion (boost::shared_ptr<const AudioRegion> other)
 	_fade_out_disabled = 0;
 	
 	listen_to_my_curves ();
+	listen_to_my_sources ();
 }
 
 AudioRegion::AudioRegion (boost::shared_ptr<AudioSource> src, const XMLNode& node)
@@ -262,6 +272,7 @@ AudioRegion::AudioRegion (boost::shared_ptr<AudioSource> src, const XMLNode& nod
 	}
 
 	listen_to_my_curves ();
+	listen_to_my_sources ();
 }
 
 AudioRegion::AudioRegion (SourceList& srcs, const XMLNode& node)
@@ -302,6 +313,7 @@ AudioRegion::AudioRegion (SourceList& srcs, const XMLNode& node)
 	}
 
 	listen_to_my_curves ();
+	listen_to_my_sources ();
 }
 
 AudioRegion::~AudioRegion ()
@@ -318,6 +330,14 @@ AudioRegion::~AudioRegion ()
 	GoingAway (); /* EMIT SIGNAL */
 }
 
+void
+AudioRegion::listen_to_my_sources ()
+{
+	for (SourceList::const_iterator i = sources.begin(); i != sources.end(); ++i) {
+		(*i)->AnalysisChanged.connect (mem_fun (*this, &AudioRegion::invalidate_transients));
+	}
+}
+
 void
 AudioRegion::listen_to_my_curves ()
 {
@@ -327,7 +347,7 @@ AudioRegion::listen_to_my_curves ()
 }
 
 bool
-AudioRegion::verify_length (nframes_t len)
+AudioRegion::verify_length (nframes_t& len)
 {
 	boost::shared_ptr<AudioFileSource> afs = boost::dynamic_pointer_cast<AudioFileSource>(source());
 
@@ -335,16 +355,19 @@ AudioRegion::verify_length (nframes_t len)
 		return true;
 	}
 
+	nframes_t maxlen = 0;
+
 	for (uint32_t n=0; n < sources.size(); ++n) {
-		if (_start > sources[n]->length() - len) {
-			return false;
-		}
+		maxlen = max (maxlen, sources[n]->length() - _start);
 	}
+	
+	len = min (len, maxlen);
+	
 	return true;
 }
 
 bool
-AudioRegion::verify_start_and_length (nframes_t new_start, nframes_t new_length)
+AudioRegion::verify_start_and_length (nframes_t new_start, nframes_t& new_length)
 {
 	boost::shared_ptr<AudioFileSource> afs = boost::dynamic_pointer_cast<AudioFileSource>(source());
 
@@ -352,11 +375,14 @@ AudioRegion::verify_start_and_length (nframes_t new_start, nframes_t new_length)
 		return true;
 	}
 
+	nframes_t maxlen = 0;
+
 	for (uint32_t n=0; n < sources.size(); ++n) {
-		if (new_length > sources[n]->length() - new_start) {
-			return false;
-		}
+		maxlen = max (maxlen, sources[n]->length() - new_start);
 	}
+
+	new_length = min (new_length, maxlen);
+
 	return true;
 }
 bool
@@ -428,12 +454,20 @@ AudioRegion::read_peaks (PeakData *buf, nframes_t npeaks, nframes_t offset, nfra
 	}
 }
 
+nframes64_t
+AudioRegion::read (Sample* buf, nframes64_t position, nframes64_t cnt, int channel) const
+{
+	/* raw read, no fades, no gain, nada */
+	return _read_at (sources, buf, 0, 0, _position + position, cnt, channel, 0, 0, true);
+}
+
 nframes_t
 AudioRegion::read_at (Sample *buf, Sample *mixdown_buffer, float *gain_buffer, nframes_t position, 
 		      nframes_t cnt, 
 		      uint32_t chan_n, nframes_t read_frames, nframes_t skip_frames) const
 {
-	return _read_at (sources, buf, mixdown_buffer, gain_buffer, position, cnt, chan_n, read_frames, skip_frames);
+	/* regular diskstream/butler read complete with fades etc */
+	return _read_at (sources, buf, mixdown_buffer, gain_buffer, position, cnt, chan_n, read_frames, skip_frames, false);
 }
 
 nframes_t
@@ -446,13 +480,16 @@ AudioRegion::master_read_at (Sample *buf, Sample *mixdown_buffer, float *gain_bu
 nframes_t
 AudioRegion::_read_at (const SourceList& srcs, Sample *buf, Sample *mixdown_buffer, float *gain_buffer,
 		       nframes_t position, nframes_t cnt, 
-		       uint32_t chan_n, nframes_t read_frames, nframes_t skip_frames) const
+		       uint32_t chan_n, 
+		       nframes_t read_frames, 
+		       nframes_t skip_frames,
+		       bool raw) const
 {
 	nframes_t internal_offset;
 	nframes_t buf_offset;
 	nframes_t to_read;
 
-	if (muted()) {
+	if (muted() && !raw) {
 		return 0; /* read nothing */
 	}
 
@@ -475,14 +512,16 @@ AudioRegion::_read_at (const SourceList& srcs, Sample *buf, Sample *mixdown_buff
 		return 0; /* read nothing */
 	}
 
-	if (opaque()) {
+	if (opaque() || raw) {
 		/* overwrite whatever is there */
 		mixdown_buffer = buf + buf_offset;
 	} else {
 		mixdown_buffer += buf_offset;
 	}
 
-	_read_data_count = 0;
+	if (!raw) {
+		_read_data_count = 0;
+	}
 
 	if (chan_n < n_channels()) {
 
@@ -491,7 +530,9 @@ AudioRegion::_read_at (const SourceList& srcs, Sample *buf, Sample *mixdown_buff
 			return 0; /* "read nothing" */
 		}
 		
-		_read_data_count += srcs[chan_n]->read_data_count();
+		if (!raw) {
+			_read_data_count += srcs[chan_n]->read_data_count();
+		}
 
 	} else {
 		
@@ -503,37 +544,41 @@ AudioRegion::_read_at (const SourceList& srcs, Sample *buf, Sample *mixdown_buff
 
 		/* no fades required */
 
-		goto merge;
+		if (!raw) {
+			goto merge;
+		}
 	}
 
 	/* fade in */
 
-	if (_flags & FadeIn) {
-
-		nframes_t fade_in_length = (nframes_t) _fade_in.back()->when;
-		
-		/* see if this read is within the fade in */
-
-		if (internal_offset < fade_in_length) {
-		
-			nframes_t limit;
-
-			limit = min (to_read, fade_in_length - internal_offset);
-
-			_fade_in.get_vector (internal_offset, internal_offset+limit, gain_buffer, limit);
-
-			for (nframes_t n = 0; n < limit; ++n) {
-				mixdown_buffer[n] *= gain_buffer[n];
+	if (!raw) {
+	
+		if (_flags & FadeIn) {
+			
+			nframes_t fade_in_length = (nframes_t) _fade_in.back()->when;
+			
+			/* see if this read is within the fade in */
+			
+			if (internal_offset < fade_in_length) {
+				
+				nframes_t limit;
+				
+				limit = min (to_read, fade_in_length - internal_offset);
+				
+				_fade_in.get_vector (internal_offset, internal_offset+limit, gain_buffer, limit);
+				
+				for (nframes_t n = 0; n < limit; ++n) {
+					mixdown_buffer[n] *= gain_buffer[n];
+				}
 			}
 		}
-	}
-	
-	/* fade out */
-
-	if (_flags & FadeOut) {
-	
-		/* see if some part of this read is within the fade out */
-
+		
+		/* fade out */
+		
+		if (_flags & FadeOut) {
+			
+			/* see if some part of this read is within the fade out */
+			
 		/* .................        >|            REGION
 		                            _length
  					    
@@ -544,65 +589,66 @@ AudioRegion::_read_at (const SourceList& srcs, Sample *buf, Sample *mixdown_buff
                         |--------------|
                         ^internal_offset
                                        ^internal_offset + to_read
-
-                  we need the intersection of [internal_offset,internal_offset+to_read] with
-                  [_length - fade_out_length, _length]
-
+				       
+				       we need the intersection of [internal_offset,internal_offset+to_read] with
+				       [_length - fade_out_length, _length]
+				       
 		*/
 
 	
-		nframes_t fade_out_length = (nframes_t) _fade_out.back()->when;
-		nframes_t fade_interval_start = max(internal_offset, _length-fade_out_length);
-		nframes_t fade_interval_end   = min(internal_offset + to_read, _length);
-
-		if (fade_interval_end > fade_interval_start) {
-			/* (part of the) the fade out is  in this buffer */
+			nframes_t fade_out_length = (nframes_t) _fade_out.back()->when;
+			nframes_t fade_interval_start = max(internal_offset, _length-fade_out_length);
+			nframes_t fade_interval_end   = min(internal_offset + to_read, _length);
 			
-			nframes_t limit = fade_interval_end - fade_interval_start;
-			nframes_t curve_offset = fade_interval_start - (_length-fade_out_length);
-			nframes_t fade_offset = fade_interval_start - internal_offset;
-								       
-			_fade_out.get_vector (curve_offset,curve_offset+limit, gain_buffer, limit);
-
-			for (nframes_t n = 0, m = fade_offset; n < limit; ++n, ++m) {
-				mixdown_buffer[m] *= gain_buffer[n];
+			if (fade_interval_end > fade_interval_start) {
+				/* (part of the) the fade out is  in this buffer */
+				
+				nframes_t limit = fade_interval_end - fade_interval_start;
+				nframes_t curve_offset = fade_interval_start - (_length-fade_out_length);
+				nframes_t fade_offset = fade_interval_start - internal_offset;
+				
+				_fade_out.get_vector (curve_offset,curve_offset+limit, gain_buffer, limit);
+				
+				for (nframes_t n = 0, m = fade_offset; n < limit; ++n, ++m) {
+					mixdown_buffer[m] *= gain_buffer[n];
+				}
+			} 
+			
+		}
+		
+		/* Regular gain curves */
+		
+		if (envelope_active())  {
+			_envelope.get_vector (internal_offset, internal_offset + to_read, gain_buffer, to_read);
+			
+			if (_scale_amplitude != 1.0f) {
+				for (nframes_t n = 0; n < to_read; ++n) {
+					mixdown_buffer[n] *= gain_buffer[n] * _scale_amplitude;
+				}
+			} else {
+				for (nframes_t n = 0; n < to_read; ++n) {
+					mixdown_buffer[n] *= gain_buffer[n];
+				}
+			}
+		} else if (_scale_amplitude != 1.0f) {
+			Session::apply_gain_to_buffer (mixdown_buffer, to_read, _scale_amplitude);
+		}
+	
+	  merge:
+		
+		if (!opaque()) {
+			
+			/* gack. the things we do for users.
+			 */
+			
+			buf += buf_offset;
+			
+			for (nframes_t n = 0; n < to_read; ++n) {
+				buf[n] += mixdown_buffer[n];
 			}
 		} 
-
 	}
 
-	/* Regular gain curves */
-
-	if (envelope_active())  {
-		_envelope.get_vector (internal_offset, internal_offset + to_read, gain_buffer, to_read);
-		
-		if (_scale_amplitude != 1.0f) {
-			for (nframes_t n = 0; n < to_read; ++n) {
-				mixdown_buffer[n] *= gain_buffer[n] * _scale_amplitude;
-			}
-		} else {
-			for (nframes_t n = 0; n < to_read; ++n) {
-				mixdown_buffer[n] *= gain_buffer[n];
-			}
-		}
-	} else if (_scale_amplitude != 1.0f) {
-		Session::apply_gain_to_buffer (mixdown_buffer, to_read, _scale_amplitude);
-	}
-
-  merge:
-
-	if (!opaque()) {
-
-		/* gack. the things we do for users.
-		 */
-
-		buf += buf_offset;
-		
-		for (nframes_t n = 0; n < to_read; ++n) {
-			buf[n] += mixdown_buffer[n];
-		}
-	} 
-	
 	return to_read;
 }
 	
@@ -678,6 +724,12 @@ AudioRegion::state (bool full)
 		child->add_property ("default", "yes");
 	}
 
+	for (uint32_t n=0; n < master_sources.size(); ++n) {
+		snprintf (buf2, sizeof(buf2), "master-source-%d", n);
+		master_sources[n]->id().print (buf, sizeof (buf));
+		node.add_property (buf2, buf);
+	}
+
 	if (full && _extra_xml) {
 		node.add_child_copy (*_extra_xml);
 	}
@@ -695,7 +747,7 @@ AudioRegion::set_live_state (const XMLNode& node, Change& what_changed, bool sen
 	Region::set_live_state (node, what_changed, false);
 
 	uint32_t old_flags = _flags;
-		
+
 	if ((prop = node.property ("flags")) != 0) {
 		_flags = Flag (string_2_enum (prop->value(), _flags));
 
@@ -746,17 +798,44 @@ AudioRegion::set_live_state (const XMLNode& node, Change& what_changed, bool sen
 			
 			_fade_in.clear ();
 			
-			if ((prop = child->property ("default")) != 0 || (prop = child->property ("steepness")) != 0 || _fade_in.set_state (*child)) {
+			if ((prop = child->property ("default")) != 0 || (prop = child->property ("steepness")) != 0) {
 				set_default_fade_in ();
-			} 
+			} else {
+				XMLNode* grandchild = child->child ("AutomationList");
+				if (grandchild) {
+					_fade_in.set_state (*grandchild);
+				}
+			}
+
+			if ((prop = child->property ("active")) != 0) {
+				if (prop->value() == "yes") {
+					set_fade_in_active (true);
+				} else {
+					set_fade_in_active (true);
+				}
+			}
 
 		} else if (child->name() == "FadeOut") {
 			
 			_fade_out.clear ();
 
-			if ((prop = child->property ("default")) != 0 || (prop = child->property ("steepness")) != 0 || _fade_out.set_state (*child)) {
+			if ((prop = child->property ("default")) != 0 || (prop = child->property ("steepness")) != 0) {
 				set_default_fade_out ();
-			} 
+			} else {
+				XMLNode* grandchild = child->child ("AutomationList");
+				if (grandchild) {
+					_fade_out.set_state (*grandchild);
+				} 
+			}
+
+			if ((prop = child->property ("active")) != 0) {
+				if (prop->value() == "yes") {
+					set_fade_out_active (true);
+				} else {
+					set_fade_out_active (false);
+				}
+			}
+
 		} 
 	}
 
@@ -911,6 +990,10 @@ AudioRegion::set_fade_out (FadeShape shape, nframes_t len)
 void
 AudioRegion::set_fade_in_length (nframes_t len)
 {
+	if (len > _length) {
+		len = _length - 1;
+	}
+
 	bool changed = _fade_in.extend_to (len);
 
 	if (changed) {
@@ -922,13 +1005,16 @@ AudioRegion::set_fade_in_length (nframes_t len)
 void
 AudioRegion::set_fade_out_length (nframes_t len)
 {
+	if (len > _length) {
+		len = _length - 1;
+	}
+
 	bool changed =	_fade_out.extend_to (len);
 
 	if (changed) {
 		_flags = Flag (_flags & ~DefaultFadeOut);
+		send_change (FadeOutChanged);
 	}
-
-	send_change (FadeOutChanged);
 }
 
 void
@@ -1113,6 +1199,12 @@ AudioRegion::master_source_names ()
 	return names;
 }
 
+void
+AudioRegion::set_master_sources (SourceList& srcs)
+{
+	master_sources = srcs;
+}
+
 bool
 AudioRegion::source_equivalent (boost::shared_ptr<const Region> o) const
 {
@@ -1426,6 +1518,112 @@ AudioRegion::set_playlist (boost::weak_ptr<Playlist> wpl)
 	}
 }
 
+int
+AudioRegion::get_transients (AnalysisFeatureList& results, bool force_new)
+{
+	boost::shared_ptr<Playlist> pl = playlist();
+
+	if (!pl) {
+		return -1;
+	}
+
+	if (valid_transients && !force_new) {
+		results = _transients;
+		return 0;
+	}
+
+	SourceList::iterator s;
+	
+	for (s = sources.begin() ; s != sources.end(); ++s) {
+		if (!(*s)->has_been_analysed()) {
+			cerr << "For " << name() << " source " << (*s)->name() << " has not been analyzed\n";
+			break;
+		}
+	}
+	
+	if (s == sources.end()) {
+		/* all sources are analyzed, merge data from each one */
+
+		for (s = sources.begin() ; s != sources.end(); ++s) {
+
+			/* find the set of transients within the bounds of this region */
+
+			AnalysisFeatureList::iterator low = lower_bound ((*s)->transients.begin(),
+									 (*s)->transients.end(),
+									 _start);
+
+			AnalysisFeatureList::iterator high = upper_bound ((*s)->transients.begin(),
+									  (*s)->transients.end(),
+									  _start + _length);
+									 
+			/* and add them */
+
+			results.insert (results.end(), low, high);
+		}
+
+		TransientDetector::cleanup_transients (results, pl->session().frame_rate(), 3.0);
+		
+		/* translate all transients to current position */
+
+		for (AnalysisFeatureList::iterator x = results.begin(); x != results.end(); ++x) {
+			(*x) -= _start;
+			(*x) += _position;
+		}
+
+		_transients = results;
+		valid_transients = true;
+
+		return 0;
+	}
+
+	TransientDetector t (pl->session().frame_rate());
+	bool existing_results = !results.empty();
+
+	_transients.clear ();
+	valid_transients = false;
+
+	for (uint32_t i = 0; i < n_channels(); ++i) {
+
+		AnalysisFeatureList these_results;
+
+		t.reset ();
+
+		if (t.run ("", this, i, these_results)) {
+			return -1;
+		}
+
+		/* translate all transients to give absolute position */
+		
+		for (AnalysisFeatureList::iterator i = these_results.begin(); i != these_results.end(); ++i) {
+			(*i) += _position;
+		}
+
+		/* merge */
+		
+		_transients.insert (_transients.end(), these_results.begin(), these_results.end());
+	}
+	
+	if (!results.empty()) {
+		if (existing_results) {
+			
+			/* merge our transients into the existing ones, then clean up
+			   those.
+			*/
+
+			results.insert (results.end(), _transients.begin(), _transients.end());
+			TransientDetector::cleanup_transients (results, pl->session().frame_rate(), 3.0);
+		}
+
+		/* make sure ours are clean too */
+
+		TransientDetector::cleanup_transients (_transients, pl->session().frame_rate(), 3.0);
+	}
+
+	valid_transients = true;
+
+	return 0;
+}
+
 extern "C" {
 
 	int region_read_peaks_from_c (void *arg, uint32_t npeaks, uint32_t start, uint32_t cnt, intptr_t data, uint32_t n_chan, double samples_per_unit) 
