@@ -46,6 +46,7 @@
 #include <midi++/mmc.h>
 
 #include <ardour/ardour.h>
+#include <ardour/analyser.h>
 #include <ardour/audio_library.h>
 #include <ardour/configuration.h>
 #include <ardour/profile.h>
@@ -285,10 +286,22 @@ ARDOUR::init (bool use_vst, bool try_optimization)
 		return -1;
 	}
 #endif
+	
+	/* Make VAMP look in our library ahead of anything else */
+
+	char *p = getenv ("VAMP_PATH");
+	string vamppath = VAMP_DIR;
+	if (p) {
+		vamppath += ':';
+		vamppath += p;
+	} 
+	setenv ("VAMP_PATH", vamppath.c_str(), 1);
+
 
 	setup_hardware_optimization (try_optimization);
 
 	SourceFactory::init ();
+	Analyser::init ();
 
 	/* singleton - first object is "it" */
 	new PluginManager ();
@@ -351,7 +364,7 @@ ARDOUR::new_change ()
 string
 ARDOUR::get_ardour_revision ()
 {
-	return "$Rev: 2489 $";
+	return "$Rev: 2983 $";
 }
 
 string
@@ -467,7 +480,7 @@ find_file (string name, string dir, string subdir = "")
 string
 ARDOUR::find_config_file (string name)
 {
-	char* envvar;
+	const char* envvar;
 
 	if ((envvar = getenv("ARDOUR_CONFIG_PATH")) == 0) {
 		envvar = CONFIG_DIR;
@@ -479,7 +492,7 @@ ARDOUR::find_config_file (string name)
 string
 ARDOUR::find_data_file (string name, string subdir)
 {
-	char* envvar;
+	const char* envvar;
 	if ((envvar = getenv("ARDOUR_DATA_PATH")) == 0) {
 		envvar = DATA_DIR;
 	}
