@@ -15,7 +15,6 @@
     along with this program; if not, write to the Free Software
     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 
-    $Id: panner.cc,v 1.48 2006/04/21 21:16:02 essej Exp $
 */
 
 #include <cmath>
@@ -28,10 +27,12 @@
 #include <unistd.h>
 #include <float.h>
 
+#include <glibmm.h>
+
 #include <pbd/error.h>
 #include <pbd/failed_constructor.h>
-#include <pbd/basename.h>
 #include <pbd/xml++.h>
+#include <pbd/enumwriter.h>
 
 #include <ardour/session.h>
 #include <ardour/panner.h>
@@ -43,33 +44,35 @@
 
 #include <pbd/mathfix.h>
 
-
 using namespace std;
 using namespace ARDOUR;
+using namespace PBD;
 
 float Panner::current_automation_version_number = 1.0;
 
 string EqualPowerStereoPanner::name = "Equal Power Stereo";
 string Multi2dPanner::name = "Multiple (2D)";
 
-/* this is a default mapper of MIDI control values to a pan position
-   others can be imagined. see Panner::set_midi_to_pan_function().
+/* this is a default mapper of  control values to a pan position
+   others can be imagined. 
 */
 
-static pan_t direct_midi_to_pan (double fract) { 
+static pan_t direct_control_to_pan (double fract) { 
 	return fract;
 }
 
-static double direct_pan_to_midi (pan_t val) { 
+static double direct_pan_to_control (pan_t val) { 
 	return val;
 }
 
 StreamPanner::StreamPanner (Panner& p)
 	: parent (p),
-	  _midi_control (*this, (MIDI::Port*) 0)
+	  _control (X_("panner"), *this)
 {
 	_muted = false;
 
+	parent.session().add_controllable (&_control);
+
 	x = 0.5;
 	y = 0.5;
 	z = 0.5;
@@ -79,84 +82,30 @@ StreamPanner::~StreamPanner ()
 {
 }
 
-StreamPanner::MIDIControl::MIDIControl (StreamPanner& s, MIDI::Port* port)
-	: MIDI::Controllable (port, 0), sp (s), setting(false)
-{
-	midi_to_pan = direct_midi_to_pan;
-	pan_to_midi = direct_pan_to_midi;
-	last_written = 0; /* XXX need a good out-of-bound-value */
-}
-
 void
-StreamPanner::MIDIControl::set_value (float val)
+StreamPanner::PanControllable::set_value (float val)
 {
-	setting = true;
-	sp.set_position (midi_to_pan (val));
-	setting = false;
+	panner.set_position (direct_control_to_pan (val));
 }
 
-void
-StreamPanner::MIDIControl::send_feedback (pan_t value)
+float
+StreamPanner::PanControllable::get_value (void) const
 {
-
-	if (!setting && get_midi_feedback() && pan_to_midi) {
-		MIDI::byte val = (MIDI::byte) (pan_to_midi (value) * 127.0f);
-		MIDI::channel_t ch = 0;
-		MIDI::eventType ev = MIDI::none;
-		MIDI::byte additional = 0;
-		MIDI::EventTwoBytes data;
-	    
-		if (get_control_info (ch, ev, additional)) {
-			data.controller_number = additional;
-			data.value = val;
-			last_written = val;
-			
-			sp.get_parent().session().send_midi_message (get_port(), ev, ch, data);
-		}
-
-		// send_midi_feedback (pan_to_midi (val));
-	}
-	
+	float xpos;
+	panner.get_effective_position (xpos);
+	return direct_pan_to_control (xpos);
 }
 
-MIDI::byte*
-StreamPanner::MIDIControl::write_feedback (MIDI::byte* buf, int32_t& bufsize, pan_t val, bool force)
+bool
+StreamPanner::PanControllable::can_send_feedback () const
 {
-	if (get_midi_feedback() && pan_to_midi && bufsize > 2) {
-		MIDI::channel_t ch = 0;
-		MIDI::eventType ev = MIDI::none;
-		MIDI::byte additional = 0;
-		MIDI::byte pm;
-		if (get_control_info (ch, ev, additional)) {
+	AutoState astate = panner.get_parent().automation_state ();
 
-			pm = (MIDI::byte) (pan_to_midi (val) * 127.0);
-
-			if (pm != last_written || force) {
-				*buf++ = (0xF0 & ev) | (0xF & ch);
-				*buf++ = additional; /* controller number */
-				*buf++ = pm;
-				last_written = pm;
-				bufsize -= 3;
-			}
-		}
+	if ((astate == Play) || (astate == Touch && !panner.get_parent().touching())) {
+		return true;
 	}
 
-	return buf;
-}
-
-
-void
-StreamPanner::reset_midi_control (MIDI::Port* port, bool on)
-{
-	MIDI::channel_t chn;
-	MIDI::eventType ev;
-	MIDI::byte extra;
-
-	_midi_control.get_control_info (chn, ev, extra);
-	if (!on) {
-		chn = -1;
-	}
-	_midi_control.midi_rebind (port, chn);
+	return false;
 }
 
 void
@@ -179,10 +128,7 @@ StreamPanner::set_position (float xpos, bool link_call)
 		x = xpos;
 		update ();
 		Changed ();
-
-		if (parent.session().get_midi_feedback()) {
-			_midi_control.send_feedback (x);
-		}
+		_control.Changed ();
 	}
 }
 
@@ -223,42 +169,11 @@ StreamPanner::set_state (const XMLNode& node)
 {
 	const XMLProperty* prop;
 	XMLNodeConstIterator iter;
-	XMLNodeList midi_kids;
 
 	if ((prop = node.property (X_("muted")))) {
 		set_muted (prop->value() == "yes");
 	}
 
-	midi_kids = node.children ("MIDI");
-	
-	for (iter = midi_kids.begin(); iter != midi_kids.end(); ++iter) {
-	
-		XMLNodeList kids;
-		XMLNodeConstIterator miter;
-		XMLNode*    child;
-
-		kids = (*iter)->children ();
-
-		for (miter = kids.begin(); miter != kids.end(); ++miter) {
-
-			child =* miter;
-
-			if (child->name() == "pan") {
-			
-				MIDI::eventType ev = MIDI::on; /* initialize to keep gcc happy */
-				MIDI::byte additional = 0;  /* ditto */
-				MIDI::channel_t chn = 0;    /* ditto */
-
-				if (get_midi_node_info (child, ev, chn, additional)) {
-					_midi_control.set_control_type (chn, ev, additional);
-				} else {
-					error << _("MIDI pan control specification is incomplete, so it has been ignored") << endmsg;
-				}
-			}
-		}
-	}
-
-	
 	return 0;
 }
 
@@ -266,68 +181,6 @@ void
 StreamPanner::add_state (XMLNode& node)
 {
 	node.add_property (X_("muted"), (muted() ? "yes" : "no"));
-
-	/* MIDI control */
-
-	MIDI::channel_t chn;
-	MIDI::eventType ev;
-	MIDI::byte      additional;
-	XMLNode*        midi_node = 0;
-	XMLNode*        child;
-
-	if (_midi_control.get_control_info (chn, ev, additional)) {
-
-		midi_node = node.add_child ("MIDI");
-
-		child = midi_node->add_child ("pan");
-		set_midi_node_info (child, ev, chn, additional);
-	}
-
-}
-
-
-bool
-StreamPanner::get_midi_node_info (XMLNode * node, MIDI::eventType & ev, MIDI::channel_t & chan, MIDI::byte & additional)
-{
-	bool ok = true;
-	const XMLProperty* prop;
-	int xx;
-
-	if ((prop = node->property ("event")) != 0) {
-		sscanf (prop->value().c_str(), "0x%x", &xx);
-		ev = (MIDI::eventType) xx;
-	} else {
-		ok = false;
-	}
-
-	if (ok && ((prop = node->property ("channel")) != 0)) {
-		sscanf (prop->value().c_str(), "%d", &xx);
-		chan = (MIDI::channel_t) xx;
-	} else {
-		ok = false;
-	}
-
-	if (ok && ((prop = node->property ("additional")) != 0)) {
-		sscanf (prop->value().c_str(), "0x%x", &xx);
-		additional = (MIDI::byte) xx;
-	}
-
-	return ok;
-}
-
-bool
-StreamPanner::set_midi_node_info (XMLNode * node, MIDI::eventType ev, MIDI::channel_t chan, MIDI::byte additional)
-{
-	char buf[32];
-
-	snprintf (buf, sizeof(buf), "0x%x", ev);
-	node->add_property ("event", buf);
-	snprintf (buf, sizeof(buf), "%d", chan);
-	node->add_property ("channel", buf);
-	snprintf (buf, sizeof(buf), "0x%x", additional);
-	node->add_property ("additional", buf);
-
-	return true;
 }
 
 /*---------------------------------------------------------------------- */
@@ -342,7 +195,7 @@ BaseStereoPanner::~BaseStereoPanner ()
 }
 
 void
-BaseStereoPanner::snapshot (jack_nframes_t now)
+BaseStereoPanner::snapshot (nframes_t now)
 {
 	if (_automation.automation_state() == Write || _automation.automation_state() == Touch) {
 		_automation.rt_add (now, x);
@@ -350,18 +203,13 @@ BaseStereoPanner::snapshot (jack_nframes_t now)
 }
 
 void
-BaseStereoPanner::transport_stopped (jack_nframes_t frame)
+BaseStereoPanner::transport_stopped (nframes_t frame)
 {
-	_automation.reposition_for_rt_add (frame);
-
 	if (_automation.automation_state() != Off) {
-		
-		if (_automation.automation_write()) {
-			_automation.save_state (_("automation write pass"));
-		}
-
 		set_position (_automation.eval (frame));
 	}
+
+	_automation.reposition_for_rt_add (frame);
 }
 
 void
@@ -383,29 +231,6 @@ BaseStereoPanner::set_automation_state (AutoState state)
 	}
 }
 
-int
-BaseStereoPanner::save (ostream& out) const
-{
-	LocaleGuard lg (X_("POSIX"));
-
-	/* force a single format for numeric data to ease session interchange
-	   across national boundaries.
-	*/
-
-	out << "begin" << endl;
-
-	for (AutomationList::const_iterator i = _automation.const_begin(); i != _automation.const_end(); ++i) {
-		out << '\t' << (jack_nframes_t) floor ((*i)->when) << ' ' << (*i)->value << endl;
-		if (!out) {
-			error << compose (_("error writing pan automation file (%s)"), strerror (errno)) << endmsg;
-			return -1;
-		}
-	}
-	out << "end" << endl;
-
-	return 0;
-}
-				
 int
 BaseStereoPanner::load (istream& in, string path, uint32_t& linecnt)
 {
@@ -425,23 +250,22 @@ BaseStereoPanner::load (istream& in, string path, uint32_t& linecnt)
 		}
 
 		if (sscanf (line, "%" PRIu32 " %lf", &when, &value) != 2) {
-			warning << compose(_("badly formatted pan automation event record at line %1 of %2 (ignored) [%3]"), linecnt, path, line) << endmsg;
+			warning << string_compose(_("badly formatted pan automation event record at line %1 of %2 (ignored) [%3]"), linecnt, path, line) << endmsg;
 			continue;
 		}
 
-		_automation.add (when, value, true);
+		_automation.fast_simple_add (when, value);
 	}
 
 	/* now that we are done loading */
 
-	_automation.save_state (_("loaded from disk"));
-	_automation.StateChanged (Change (0));
+	_automation.StateChanged ();
 
 	return 0;
 }
 
 void
-BaseStereoPanner::distribute (Sample* src, Sample** obufs, gain_t gain_coeff, jack_nframes_t nframes)
+BaseStereoPanner::distribute (Sample* src, Sample** obufs, gain_t gain_coeff, nframes_t nframes)
 {
 	pan_t delta;
 	Sample* dst;
@@ -459,8 +283,8 @@ BaseStereoPanner::distribute (Sample* src, Sample** obufs, gain_t gain_coeff, ja
 		
 		/* interpolate over 64 frames or nframes, whichever is smaller */
 		
-		jack_nframes_t limit = min ((jack_nframes_t)64, nframes);
-		jack_nframes_t n;
+		nframes_t limit = min ((nframes_t)64, nframes);
+		nframes_t n;
 
 		delta = -(delta / (float) (limit));
 		
@@ -509,8 +333,8 @@ BaseStereoPanner::distribute (Sample* src, Sample** obufs, gain_t gain_coeff, ja
 		
 		/* interpolate over 64 frames or nframes, whichever is smaller */
 		
-		jack_nframes_t limit = min ((jack_nframes_t)64, nframes);
-		jack_nframes_t n;
+		nframes_t limit = min ((nframes_t)64, nframes);
+		nframes_t n;
 
 		delta = -(delta / (float) (limit));
 
@@ -595,7 +419,7 @@ EqualPowerStereoPanner::update ()
 
 void
 EqualPowerStereoPanner::distribute_automated (Sample* src, Sample** obufs, 
-					      jack_nframes_t start, jack_nframes_t end, jack_nframes_t nframes,
+					      nframes_t start, nframes_t end, nframes_t nframes,
 					      pan_t** buffers)
 {
 	Sample* dst;
@@ -613,7 +437,7 @@ EqualPowerStereoPanner::distribute_automated (Sample* src, Sample** obufs,
 
 	/* store effective pan position. do this even if we are muted */
 
-	if (nframes > 0)
+	if (nframes > 0) 
 		effective_x = buffers[0][nframes-1];
 
 	if (_muted) {
@@ -627,7 +451,7 @@ EqualPowerStereoPanner::distribute_automated (Sample* src, Sample** obufs,
 	const float pan_law_attenuation = -3.0f;
 	const float scale = 2.0f - 4.0f * powf (10.0f,pan_law_attenuation/20.0f);
 
-	for (jack_nframes_t n = 0; n < nframes; ++n) {
+	for (nframes_t n = 0; n < nframes; ++n) {
 
 		float panR = buffers[0][n];
 		float panL = 1 - panR;
@@ -641,7 +465,7 @@ EqualPowerStereoPanner::distribute_automated (Sample* src, Sample** obufs,
 	dst = obufs[0];
 	pbuf = buffers[0];
 	
-	for (jack_nframes_t n = 0; n < nframes; ++n) {
+	for (nframes_t n = 0; n < nframes; ++n) {
 		dst[n] += src[n] * pbuf[n];
 	}	
 
@@ -652,7 +476,7 @@ EqualPowerStereoPanner::distribute_automated (Sample* src, Sample** obufs,
 	dst = obufs[1];
 	pbuf = buffers[1];
 
-	for (jack_nframes_t n = 0; n < nframes; ++n) {
+	for (nframes_t n = 0; n < nframes; ++n) {
 		dst[n] += src[n] * pbuf[n];
 	}	
 	
@@ -681,18 +505,15 @@ EqualPowerStereoPanner::state (bool full_state)
 	snprintf (buf, sizeof (buf), "%.12g", x); 
 	root->add_property (X_("x"), buf);
 	root->add_property (X_("type"), EqualPowerStereoPanner::name);
-	if (full_state) {
-		snprintf (buf, sizeof (buf), "0x%x", _automation.automation_state()); 
-	} else {
-		/* never store automation states other than off in a template */
-		snprintf (buf, sizeof (buf), "0x%x", ARDOUR::Off); 
-	}
-	root->add_property (X_("automation-state"), buf);
-	snprintf (buf, sizeof (buf), "0x%x", _automation.automation_style()); 
-	root->add_property (X_("automation-style"), buf);
+
+	XMLNode* autonode = new XMLNode (X_("Automation"));
+	autonode->add_child_nocopy (_automation.state (full_state));
+	root->add_child_nocopy (*autonode);
 
 	StreamPanner::add_state (*root);
 
+	root->add_child_nocopy (_control.get_state ());
+
 	return *root;
 }
 
@@ -700,7 +521,6 @@ int
 EqualPowerStereoPanner::set_state (const XMLNode& node)
 {
 	const XMLProperty* prop;
-	int x;
 	float pos;
 	LocaleGuard lg (X_("POSIX"));
 
@@ -709,22 +529,25 @@ EqualPowerStereoPanner::set_state (const XMLNode& node)
 		set_position (pos, true);
 	} 
 
-	if ((prop = node.property (X_("automation-state")))) {
-		sscanf (prop->value().c_str(), "0x%x", &x);
-		_automation.set_automation_state ((AutoState) x);
+	StreamPanner::set_state (node);
 
-		if (x != Off) {
-			set_position (_automation.eval (parent.session().transport_frame()));
+	for (XMLNodeConstIterator iter = node.children().begin(); iter != node.children().end(); ++iter) {
+
+		if ((*iter)->name() == X_("controllable")) {
+			if ((prop = (*iter)->property("name")) != 0 && prop->value() == "panner") {
+				_control.set_state (**iter);
+			}
+
+		} else if ((*iter)->name() == X_("Automation")) {
+
+			_automation.set_state (*((*iter)->children().front()));
+
+			if (_automation.automation_state() != Off) {
+				set_position (_automation.eval (parent.session().transport_frame()));
+			}
 		}
 	}
 
-	if ((prop = node.property (X_("automation-style")))) {
-		sscanf (prop->value().c_str(), "0x%x", &x);
-		_automation.set_automation_style ((AutoStyle) x);
-	}
-
-	StreamPanner::set_state (node);
-	
 	return 0;
 }
 
@@ -741,13 +564,13 @@ Multi2dPanner::~Multi2dPanner ()
 }
 
 void
-Multi2dPanner::snapshot (jack_nframes_t now)
+Multi2dPanner::snapshot (nframes_t now)
 {
 	// how?
 }
 
 void
-Multi2dPanner::transport_stopped (jack_nframes_t frame)
+Multi2dPanner::transport_stopped (nframes_t frame)
 {
 	//what?
 }
@@ -797,7 +620,7 @@ Multi2dPanner::update ()
 }
 
 void
-Multi2dPanner::distribute (Sample* src, Sample** obufs, gain_t gain_coeff, jack_nframes_t nframes)
+Multi2dPanner::distribute (Sample* src, Sample** obufs, gain_t gain_coeff, nframes_t nframes)
 {
 	Sample* dst;
 	pan_t pan;
@@ -818,8 +641,8 @@ Multi2dPanner::distribute (Sample* src, Sample** obufs, gain_t gain_coeff, jack_
 			
 			/* interpolate over 64 frames or nframes, whichever is smaller */
 			
-			jack_nframes_t limit = min ((jack_nframes_t)64, nframes);
-			jack_nframes_t n;
+			nframes_t limit = min ((nframes_t)64, nframes);
+			nframes_t n;
 			
 			delta = -(delta / (float) (limit));
 		
@@ -830,10 +653,7 @@ Multi2dPanner::distribute (Sample* src, Sample** obufs, gain_t gain_coeff, jack_
 			}
 			
 			pan = left * gain_coeff;
-			
-			for (; n < nframes; ++n) {
-				dst[n] += src[n] * pan;
-			}
+			Session::mix_buffers_with_gain(dst+n,src+n,nframes-n,pan);
 			
 		} else {
 
@@ -843,20 +663,10 @@ Multi2dPanner::distribute (Sample* src, Sample** obufs, gain_t gain_coeff, jack_
 			if ((pan *= gain_coeff) != 1.0f) {
 				
 				if (pan != 0.0f) {
-					
-					for (jack_nframes_t n = 0; n < nframes; ++n) {
-						dst[n] += src[n] * pan;
-					}      
-					
+					Session::mix_buffers_with_gain(dst,src,nframes,pan);
 				} 
-
-				
 			} else {
-				
-				for (jack_nframes_t n = 0; n < nframes; ++n) {
-					dst[n] += src[n];
-				}      
-
+					Session::mix_buffers_no_gain(dst,src,nframes);
 			}
 #endif
 #ifdef CAN_INTERP
@@ -869,7 +679,7 @@ Multi2dPanner::distribute (Sample* src, Sample** obufs, gain_t gain_coeff, jack_
 
 void
 Multi2dPanner::distribute_automated (Sample* src, Sample** obufs, 
-				     jack_nframes_t start, jack_nframes_t end, jack_nframes_t nframes,
+				     nframes_t start, nframes_t end, nframes_t nframes,
 				     pan_t** buffers)
 {
 	if (_muted) {
@@ -893,12 +703,6 @@ Multi2dPanner::load (istream& in, string path, uint32_t& linecnt)
 	return 0;
 }
 
-int
-Multi2dPanner::save (ostream& out) const
-{
-	return 0;
-}
-
 XMLNode&
 Multi2dPanner::get_state (void)
 {
@@ -918,6 +722,8 @@ Multi2dPanner::state (bool full_state)
 	root->add_property (X_("y"), buf);
 	root->add_property (X_("type"), Multi2dPanner::name);
 
+	/* XXX no meaningful automation yet */
+
 	return *root;
 }
 
@@ -955,11 +761,10 @@ Panner::Panner (string name, Session& s)
 	: _session (s)
 {
 	set_name (name);
+
 	_linked = false;
 	_link_direction = SameDirection;
 	_bypassed = false;
-
-	reset_midi_control (_session.mmc_port(), _session.get_mmc_control());
 }
 
 Panner::~Panner ()
@@ -986,17 +791,6 @@ Panner::set_link_direction (LinkDirection ld)
 	}
 }
 
-void
-Panner::set_name (string str)
-{
-	automation_path = _session.automation_dir();
-	automation_path += _session.snap_name();
-	automation_path += "-pan-";
-	automation_path += legalize_for_path (str);
-	automation_path += ".automation";
-}
-
-
 void
 Panner::set_bypassed (bool yn)
 {
@@ -1013,7 +807,6 @@ Panner::reset (uint32_t nouts, uint32_t npans)
 	uint32_t n;
 	bool changed = false;
 
-
 	if (nouts < 2 || (nouts == outputs.size() && npans == size())) {
 		return;
 	} 
@@ -1106,8 +899,6 @@ Panner::reset (uint32_t nouts, uint32_t npans)
 		(*x)->update ();
 	}
 
-	reset_midi_control (_session.mmc_port(), _session.get_mmc_control());
-
 	/* force hard left/right panning in a common case: 2in/2out 
 	*/
 	
@@ -1203,7 +994,7 @@ Panner::automation_style () const
 }
 
 void
-Panner::transport_stopped (jack_nframes_t frame)
+Panner::transport_stopped (nframes_t frame)
 {
 	for (vector<StreamPanner*>::iterator i = begin(); i != end(); ++i) {
 		(*i)->transport_stopped (frame);
@@ -1211,7 +1002,7 @@ Panner::transport_stopped (jack_nframes_t frame)
 }	
 
 void
-Panner::snapshot (jack_nframes_t now)
+Panner::snapshot (nframes_t now)
 {
 	for (vector<StreamPanner*>::iterator i = begin(); i != end(); ++i) {
 		(*i)->snapshot (now);
@@ -1227,102 +1018,6 @@ Panner::clear_automation ()
 	_session.set_dirty ();
 }	
 
-int
-Panner::save () const
-{
-	ofstream out (automation_path.c_str());
-	
-	if (!out) {
-		error << compose (_("cannot open pan automation file \"%1\" for saving (%s)"), automation_path, strerror (errno))
-		      << endmsg;
-		return -1;
-	}
-
-	out << X_("version ") << current_automation_version_number << endl;
-
-	for (vector<StreamPanner*>::const_iterator i = begin(); i != end(); ++i) {
-		if ((*i)->save (out)) {
-			return -1;
-		}
-	}
-
-	return 0;
-}
-
-int
-Panner::load ()
-{
-	char line[128];
-	uint32_t linecnt = 0;
-	float version;
-	iterator sp;
-	LocaleGuard lg (X_("POSIX"));
-
-	if (automation_path.length() == 0) {
-		return 0;
-	}
-	
-	if (access (automation_path.c_str(), F_OK)) {
-		return 0;
-	}
-
-	ifstream in (automation_path.c_str());
-
-	if (!in) {
-		error << compose (_("cannot open pan automation file %1 (%2)"),
-				  automation_path, strerror (errno))
-		      << endmsg;
-		return -1;
-	}
-
-	sp = begin();
-
-	while (in.getline (line, sizeof(line), '\n')) {
-
-		if (++linecnt == 1) {
-			if (memcmp (line, X_("version"), 7) == 0) {
-				if (sscanf (line, "version %f", &version) != 1) {
-					error << compose(_("badly formed version number in pan automation event file \"%1\""), automation_path) << endmsg;
-					return -1;
-				}
-			} else {
-				error << compose(_("no version information in pan automation event file \"%1\" (first line = %2)"), 
-						 automation_path, line) << endmsg;
-				return -1;
-			}
-
-			if (version != current_automation_version_number) {
-				error << compose(_("mismatched pan automation event file version (%1)"), version) << endmsg;
-				return -1;
-			}
-
-			continue;
-		}
-
-		if (strlen (line) == 0 || line[0] == '#') {
-			continue;
-		}
-
-		if (strcmp (line, "begin") == 0) {
-			
-			if (sp == end()) {
-				error << compose (_("too many panner states found in pan automation file %1"),
-						  automation_path)
-				      << endmsg;
-				return -1;
-			}
-
-			if ((*sp)->load (in, automation_path, linecnt)) {
-				return -1;
-			}
-			
-			++sp;
-		}
-	}
-
-	return 0;
-}
-
 struct PanPlugins {
     string name;
     uint32_t nouts;
@@ -1332,7 +1027,7 @@ struct PanPlugins {
 PanPlugins pan_plugins[] = {
 	{ EqualPowerStereoPanner::name, 2, EqualPowerStereoPanner::factory },
 	{ Multi2dPanner::name, 3, Multi2dPanner::factory },
-	{ string (""), 0 }
+	{ string (""), 0, 0 }
 };
 
 XMLNode&
@@ -1347,13 +1042,8 @@ Panner::state (bool full)
 	XMLNode* root = new XMLNode (X_("Panner"));
 	char buf[32];
 
-	for (iterator p = begin(); p != end(); ++p) {
-		root->add_child_nocopy ((*p)->state (full));
-	}
-
 	root->add_property (X_("linked"), (_linked ? "yes" : "no"));
-	snprintf (buf, sizeof (buf), "%d", _link_direction);
-	root->add_property (X_("link_direction"), buf);
+	root->add_property (X_("link_direction"), enum_2_string (_link_direction));
 	root->add_property (X_("bypassed"), (bypassed() ? "yes" : "no"));
 
 	/* add each output */
@@ -1367,10 +1057,8 @@ Panner::state (bool full)
 		root->add_child_nocopy (*onode);
 	}
 
-	if (full) {
-		if (save () == 0) {
-			root->add_property (X_("automation"), PBD::basename (automation_path));
-		}
+	for (vector<StreamPanner*>::const_iterator i = begin(); i != end(); ++i) {
+		root->add_child_nocopy ((*i)->state (full));
 	}
 
 	return *root;
@@ -1399,8 +1087,8 @@ Panner::set_state (const XMLNode& node)
 	}
 
 	if ((prop = node.property (X_("link_direction"))) != 0) {
-		sscanf (prop->value().c_str(), "%d", &i);
-		set_link_direction ((LinkDirection) (i));
+		LinkDirection ld; /* here to provide type information */
+		set_link_direction (LinkDirection (string_2_enum (prop->value(), ld)));
 	}
 
 	nlist = node.children();
@@ -1411,10 +1099,10 @@ Panner::set_state (const XMLNode& node)
 			float x, y;
 			
 			prop = (*niter)->property (X_("x"));
-			sscanf (prop->value().c_str(), "%f", &x);
+			sscanf (prop->value().c_str(), "%g", &x);
 			
 			prop = (*niter)->property (X_("y"));
-			sscanf (prop->value().c_str(), "%f", &y);
+			sscanf (prop->value().c_str(), "%g", &y);
 			
 			outputs.push_back (Output (x, y));
 		}
@@ -1447,7 +1135,7 @@ Panner::set_state (const XMLNode& node)
 				
 				
 				if (!pan_plugins[i].factory) {
-					error << compose (_("Unknown panner plugin \"%1\" found in pan state - ignored"),
+					error << string_compose (_("Unknown panner plugin \"%1\" found in pan state - ignored"),
 							  prop->value())
 					      << endmsg;
 				}
@@ -1461,7 +1149,7 @@ Panner::set_state (const XMLNode& node)
 		} 	
 	}
 
-	/* don't try to load automation if it wasn't marked as existing */
+	/* don't try to do old-school automation loading if it wasn't marked as existing */
 
 	if ((prop = node.property (X_("automation")))) {
 
@@ -1488,14 +1176,6 @@ Panner::touching () const
 	return false;
 }
 
-void
-Panner::reset_midi_control (MIDI::Port* port, bool on)
-{
-	for (vector<StreamPanner*>::const_iterator i = begin(); i != end(); ++i) {
-		(*i)->reset_midi_control (port, on);
-	}
-}      
-
 void
 Panner::set_position (float xpos, StreamPanner& orig)
 {
@@ -1639,41 +1319,83 @@ Panner::set_position (float xpos, float ypos, float zpos, StreamPanner& orig)
 	}
 }
 
+/* old school automation handling */
+
 void
-Panner::send_all_midi_feedback ()
+Panner::set_name (string str)
 {
-	if (_session.get_midi_feedback()) {
-		float xpos;
-		
-		// do feedback for all panners
-		for (vector<StreamPanner*>::iterator i = begin(); i != end(); ++i) {
-			(*i)->get_effective_position (xpos);
-
-			(*i)->midi_control().send_feedback (xpos);
-		}
-		
-	}
+	automation_path = _session.automation_dir();
+	automation_path += _session.snap_name();
+	automation_path += "-pan-";
+	automation_path += legalize_for_path (str);
+	automation_path += ".automation";
 }
 
-MIDI::byte*
-Panner::write_midi_feedback (MIDI::byte* buf, int32_t& bufsize)
+int
+Panner::load ()
 {
-	AutoState astate = automation_state ();
+	char line[128];
+	uint32_t linecnt = 0;
+	float version;
+	iterator sp;
+	LocaleGuard lg (X_("POSIX"));
 
-	if (_session.get_midi_feedback() && 
-	    (astate == Play || (astate == Touch && !touching()))) {
-		
-		float xpos;
-		
-		// do feedback for all panners
-		for (vector<StreamPanner*>::iterator i = begin(); i != end(); ++i) {
-			(*i)->get_effective_position (xpos);
+	if (automation_path.length() == 0) {
+		return 0;
+	}
+	
+	if (access (automation_path.c_str(), F_OK)) {
+		return 0;
+	}
+
+	ifstream in (automation_path.c_str());
+
+	if (!in) {
+		error << string_compose (_("cannot open pan automation file %1 (%2)"),
+				  automation_path, strerror (errno))
+		      << endmsg;
+		return -1;
+	}
+
+	sp = begin();
+
+	while (in.getline (line, sizeof(line), '\n')) {
+
+		if (++linecnt == 1) {
+			if (memcmp (line, X_("version"), 7) == 0) {
+				if (sscanf (line, "version %f", &version) != 1) {
+					error << string_compose(_("badly formed version number in pan automation event file \"%1\""), automation_path) << endmsg;
+					return -1;
+				}
+			} else {
+				error << string_compose(_("no version information in pan automation event file \"%1\" (first line = %2)"), 
+						 automation_path, line) << endmsg;
+				return -1;
+			}
+
+			continue;
+		}
+
+		if (strlen (line) == 0 || line[0] == '#') {
+			continue;
+		}
+
+		if (strcmp (line, "begin") == 0) {
 			
-			buf = (*i)->midi_control().write_feedback (buf, bufsize, xpos);
+			if (sp == end()) {
+				error << string_compose (_("too many panner states found in pan automation file %1"),
+						  automation_path)
+				      << endmsg;
+				return -1;
+			}
+
+			if ((*sp)->load (in, automation_path, linecnt)) {
+				return -1;
+			}
+			
+			++sp;
 		}
-		
 	}
 
-	return buf;
+	return 0;
 }
-
