@@ -36,10 +36,19 @@
 #include <ardour/plugin.h>
 #include <ardour/ladspa_plugin.h>
 
+#ifdef HAVE_SLV2
+#include <slv2/slv2.h>
+#include <ardour/lv2_plugin.h>
+#endif
+
 #ifdef VST_SUPPORT
 #include <ardour/vst_plugin.h>
 #endif
 
+#ifdef HAVE_AUDIOUNITS
+#include <ardour/audio_unit.h>
+#endif
+
 #include <pbd/error.h>
 #include <pbd/stl_delete.h>
 
@@ -81,21 +90,44 @@ PluginManager::PluginManager ()
 		vst_path = s;
 	}
 
-	refresh ();
 	if (_manager == 0) {
 		_manager = this;
 	}
+
+	/* the plugin manager is constructed too early to use Profile */
+
+	if (getenv ("ARDOUR_SAE")) {
+		ladspa_plugin_whitelist.push_back (1203); // single band parametric
+		ladspa_plugin_whitelist.push_back (1772); // caps compressor
+		ladspa_plugin_whitelist.push_back (1913); // fast lookahead limiter
+		ladspa_plugin_whitelist.push_back (1075); // simple RMS expander
+		ladspa_plugin_whitelist.push_back (1061); // feedback delay line (max 5s)
+		ladspa_plugin_whitelist.push_back (1216); // gverb
+		ladspa_plugin_whitelist.push_back (2150); // tap pitch shifter
+	} 
+
+#ifdef HAVE_SLV2
+	_lv2_world = new LV2World();
+#endif
+
+	refresh ();
 }
 
 void
 PluginManager::refresh ()
 {
 	ladspa_refresh ();
+#ifdef HAVE_SLV2
+	lv2_refresh ();
+#endif
 #ifdef VST_SUPPORT
 	if (Config->get_use_vst()) {
 		vst_refresh ();
 	}
 #endif // VST_SUPPORT
+#ifdef HAVE_AUDIOUNITS
+	au_refresh ();
+#endif
 }
 
 void
@@ -110,6 +142,7 @@ PluginManager::ladspa_refresh ()
 	ladspa_discover_from_path (ladspa_path);
 }
 
+
 int
 PluginManager::add_ladspa_directory (string path)
 {
@@ -245,6 +278,12 @@ PluginManager::ladspa_discover (string path)
 			break;
 		}
 
+		if (!ladspa_plugin_whitelist.empty()) {
+			if (find (ladspa_plugin_whitelist.begin(), ladspa_plugin_whitelist.end(), descriptor->UniqueID) == ladspa_plugin_whitelist.end()) {
+				continue;
+			}
+		} 
+
 		PluginInfoPtr info(new LadspaPluginInfo);
 		info->name = descriptor->Name;
 		info->category = get_ladspa_category(descriptor->UniqueID);
@@ -254,7 +293,10 @@ PluginManager::ladspa_discover (string path)
 		info->n_inputs = 0;
 		info->n_outputs = 0;
 		info->type = ARDOUR::LADSPA;
-		info->unique_id = descriptor->UniqueID;
+		
+		char buf[32];
+		snprintf (buf, sizeof (buf), "%lu", descriptor->UniqueID);
+		info->unique_id = buf;
 		
 		for (uint32_t n=0; n < descriptor->PortCount; ++n) {
 			if ( LADSPA_IS_PORT_AUDIO (descriptor->PortDescriptors[n]) ) {
@@ -284,18 +326,18 @@ PluginManager::get_ladspa_category (uint32_t plugin_id)
 
 	snprintf(buf, sizeof(buf), "%s%" PRIu32, LADSPA_BASE, plugin_id);
 	pattern.subject = buf;
-	pattern.predicate = RDF_TYPE;
+	pattern.predicate = (char*)RDF_TYPE;
 	pattern.object = 0;
 	pattern.object_type = lrdf_uri;
 
 	lrdf_statement* matches1 = lrdf_matches (&pattern);
 
 	if (!matches1) {
-		return _("Unknown");
+		return "";
 	}
 
 	pattern.subject = matches1->object;
-	pattern.predicate = LADSPA_BASE "hasLabel";
+	pattern.predicate = (char*)(LADSPA_BASE "hasLabel");
 	pattern.object = 0;
 	pattern.object_type = lrdf_literal;
 
@@ -303,7 +345,7 @@ PluginManager::get_ladspa_category (uint32_t plugin_id)
 	lrdf_free_statements(matches1);
 
 	if (!matches2) {
-		return _("Unknown");
+		return ("");
 	}
 
 	string label = matches2->object;
@@ -312,6 +354,37 @@ PluginManager::get_ladspa_category (uint32_t plugin_id)
 	return label;
 }
 
+#ifdef HAVE_SLV2
+void
+PluginManager::lv2_refresh ()
+{
+	lv2_discover();
+}
+
+int
+PluginManager::lv2_discover ()
+{
+	_lv2_plugin_info = LV2PluginInfo::discover(_lv2_world);
+	return 0;
+}
+#endif
+
+#ifdef HAVE_AUDIOUNITS
+void
+PluginManager::au_refresh ()
+{
+	au_discover();
+}
+
+int
+PluginManager::au_discover ()
+{
+	_au_plugin_info = AUPluginInfo::discover();
+	return 0;
+}
+
+#endif
+
 #ifdef VST_SUPPORT
 
 void
@@ -370,6 +443,7 @@ int
 PluginManager::vst_discover (string path)
 {
 	FSTInfo* finfo;
+	char buf[32];
 
 	if ((finfo = fst_get_info (const_cast<char *> (path.c_str()))) == 0) {
 		warning << "Cannot get VST information from " << path << endmsg;
@@ -392,6 +466,9 @@ PluginManager::vst_discover (string path)
 		info->name = finfo->name;
 	}
 
+	
+	snprintf (buf, sizeof (buf), "%d", finfo->UniqueID);
+	info->unique_id = buf;
 	info->category = "VST";
 	info->path = path;
 	// need to set info->creator but FST doesn't provide it
