@@ -29,6 +29,7 @@
 #include <glibmm/thread.h>
 #include <pbd/pthread_utils.h>
 #include <pbd/memento_command.h>
+#include <pbd/stacktrace.h>
 
 #include <midi++/mmc.h>
 #include <midi++/port.h>
@@ -54,7 +55,7 @@ Session::request_input_change_handling ()
 	if (!(_state_of_the_state & (InitialConnecting|Deletion))) {
 		Event* ev = new Event (Event::InputConfigurationChange, Event::Add, Event::Immediate, 0, 0.0);
 		queue_event (ev);
-	}
+	} 
 }
 
 void
@@ -371,17 +372,35 @@ Session::non_realtime_stop (bool abort, int on_entry, bool& finished)
 		update_latency_compensation (true, abort);
 	}
 
-	if ((Config->get_slave_source() == None && Config->get_auto_return()) || (post_transport_work & PostTransportLocate) || synced_to_jack()) {
+	if ((Config->get_slave_source() == None && Config->get_auto_return()) || 
+	    (post_transport_work & PostTransportLocate) || 
+	    (_requested_return_frame >= 0) ||
+	    synced_to_jack()) {
 		
 		if (pending_locate_flush) {
 			flush_all_redirects ();
 		}
 		
-		if (((Config->get_slave_source() == None && Config->get_auto_return()) || synced_to_jack()) && !(post_transport_work & PostTransportLocate)) {
+		if (((Config->get_slave_source() == None && Config->get_auto_return()) || 
+		     synced_to_jack() ||
+		     _requested_return_frame >= 0) &&
+		    !(post_transport_work & PostTransportLocate)) {
 
-			_transport_frame = last_stop_frame;
+			bool do_locate = false;
+
+			if (_requested_return_frame >= 0) {
+				_transport_frame = _requested_return_frame;
+				_requested_return_frame = -1;
+				do_locate = true;
+			} else {
+				_transport_frame = last_stop_frame;
+			}
 
 			if (synced_to_jack() && !play_loop) {
+				do_locate = true;
+			}
+			
+			if (do_locate) {
 				// cerr << "non-realtimestop: transport locate to " << _transport_frame << endl;
 				_engine.transport_locate (_transport_frame);
 			}
@@ -460,7 +479,9 @@ Session::non_realtime_stop (bool abort, int on_entry, bool& finished)
 		
 	}
 
-	PositionChanged (_transport_frame); /* EMIT SIGNAL */
+        nframes_t tf = _transport_frame;
+
+        PositionChanged (tf); /* EMIT SIGNAL */
 	TransportStateChange (); /* EMIT SIGNAL */
 
 	/* and start it up again if relevant */
@@ -1164,6 +1185,14 @@ Session::setup_auto_play ()
 	merge_event (ev);
 }
 
+void
+Session::request_roll_at_and_return (nframes_t start, nframes_t return_to)
+{
+	request_locate (start, false);
+ 	Event *ev = new Event (Event::LocateRollLocate, Event::Add, Event::Immediate, return_to, 1.0);
+	queue_event (ev);
+}
+
 void
 Session::request_bounded_roll (nframes_t start, nframes_t end)
 {
