@@ -44,7 +44,7 @@ using namespace sigc;
 
 sigc::signal<void,boost::shared_ptr<Source> > SourceFactory::SourceCreated;
 Glib::Cond* SourceFactory::PeaksToBuild;
-Glib::StaticMutex SourceFactory::peak_building_lock;
+Glib::StaticMutex SourceFactory::peak_building_lock = GLIBMM_STATIC_MUTEX_INIT;
 std::list<boost::weak_ptr<AudioSource> > SourceFactory::files_with_peaks;
 
 static void 
@@ -64,7 +64,7 @@ peak_thread_work ()
 		if (SourceFactory::files_with_peaks.empty()) {
 			goto wait;
 		}
-		
+
 		boost::shared_ptr<AudioSource> as (SourceFactory::files_with_peaks.front().lock());
 		SourceFactory::files_with_peaks.pop_front ();
 		SourceFactory::peak_building_lock.unlock ();
@@ -72,7 +72,6 @@ peak_thread_work ()
 		if (!as) {
 			continue;
 		}
-
 		as->setup_peakfile ();
 	}
 }
@@ -116,6 +115,7 @@ boost::shared_ptr<Source>
 SourceFactory::createSilent (Session& s, const XMLNode& node, nframes_t nframes, float sr)
 {
 	boost::shared_ptr<Source> ret (new SilentFileSource (s, node, nframes, sr));
+	// no analysis data - the file is non-existent
 	SourceCreated (ret);
 	return ret;
 }
@@ -126,11 +126,10 @@ SourceFactory::create (Session& s, const XMLNode& node, bool defer_peaks)
 {
 	try {
 		boost::shared_ptr<Source> ret (new CoreAudioSource (s, node));
-		if (!defer_peaks) {
-			if (setup_peakfile (ret, false)) {
-				return boost::shared_ptr<Source>();
-			}
+		if (setup_peakfile (ret, defer_peaks)) {
+			return boost::shared_ptr<Source>();
 		}
+		ret->check_for_analysis_data_on_disk ();
 		SourceCreated (ret);
 		return ret;
 	} 
@@ -141,11 +140,10 @@ SourceFactory::create (Session& s, const XMLNode& node, bool defer_peaks)
 		/* this is allowed to throw */
 
 		boost::shared_ptr<Source> ret (new SndFileSource (s, node));
-		if (!defer_peaks) {
-			if (setup_peakfile (ret, false)) {
-				return boost::shared_ptr<Source>();
-			}
+		if (setup_peakfile (ret, defer_peaks)) {
+			return boost::shared_ptr<Source>();
 		}
+		ret->check_for_analysis_data_on_disk ();
 		SourceCreated (ret);
 		return ret;
 	}
@@ -162,12 +160,10 @@ SourceFactory::create (Session& s, const XMLNode& node, bool defer_peaks)
 
 	boost::shared_ptr<Source> ret (new SndFileSource (s, node));
 	
-	if (!defer_peaks) {
-		if (setup_peakfile (ret, false)) {
-			return boost::shared_ptr<Source>();
-		}
+	if (setup_peakfile (ret, defer_peaks)) {
+		return boost::shared_ptr<Source>();
 	}
-	
+	ret->check_for_analysis_data_on_disk ();
 	SourceCreated (ret);
 	return ret;
 }
@@ -182,11 +178,10 @@ SourceFactory::createReadable (Session& s, string path, int chn, AudioFileSource
 
 		try {
 			boost::shared_ptr<Source> ret (new CoreAudioSource (s, path, chn, flags));
-			if (!defer_peaks) {
-				if (setup_peakfile (ret, false)) {
-					return boost::shared_ptr<Source>();
-				}
+			if (setup_peakfile (ret, defer_peaks)) {
+				return boost::shared_ptr<Source>();
 			}
+			ret->check_for_analysis_data_on_disk ();
 			if (announce) {
 				SourceCreated (ret);
 			}
@@ -198,11 +193,10 @@ SourceFactory::createReadable (Session& s, string path, int chn, AudioFileSource
 			/* this is allowed to throw */
 
 			boost::shared_ptr<Source> ret (new SndFileSource (s, path, chn, flags));
-			if (!defer_peaks) {
-				if (setup_peakfile (ret, false)) {
-					return boost::shared_ptr<Source>();
-				}
+			if (setup_peakfile (ret, defer_peaks)) {
+				return boost::shared_ptr<Source>();
 			}
+			ret->check_for_analysis_data_on_disk ();
 			if (announce) {
 				SourceCreated (ret);
 			}
@@ -212,11 +206,10 @@ SourceFactory::createReadable (Session& s, string path, int chn, AudioFileSource
 	} else {
 
 		boost::shared_ptr<Source> ret (new SndFileSource (s, path, chn, flags));
-		if (!defer_peaks) {
-			if (setup_peakfile (ret, false)) {
-				return boost::shared_ptr<Source>();
-			}
+		if (setup_peakfile (ret, defer_peaks)) {
+			return boost::shared_ptr<Source>();
 		}
+		ret->check_for_analysis_data_on_disk ();
 		if (announce) {
 			SourceCreated (ret);
 		}
@@ -233,12 +226,12 @@ SourceFactory::createReadable (Session& s, string path, int chn, AudioFileSource
 {
 	boost::shared_ptr<Source> ret (new SndFileSource (s, path, chn, flags));
 
-	if (!defer_peaks) {
-		if (setup_peakfile (ret, false)) {
-			return boost::shared_ptr<Source>();
-		}
+	if (setup_peakfile (ret, defer_peaks)) {
+		return boost::shared_ptr<Source>();
 	}
 
+	ret->check_for_analysis_data_on_disk ();
+
 	if (announce) {
 		SourceCreated (ret);
 	}
@@ -261,12 +254,12 @@ SourceFactory::createWritable (Session& s, std::string path, bool destructive, n
 					(destructive ? AudioFileSource::Flag (SndFileSource::default_writable_flags | AudioFileSource::Destructive) :
 					 SndFileSource::default_writable_flags)));	
 
-	if (!defer_peaks) {
-		if (setup_peakfile (ret, false)) {
-			return boost::shared_ptr<Source>();
-		}
+	if (setup_peakfile (ret, defer_peaks)) {
+		return boost::shared_ptr<Source>();
 	}
 
+	// no analysis data - this is a new file
+
 	if (announce) {
 		SourceCreated (ret);
 	}
