@@ -1,5 +1,5 @@
 // -*- c++ -*-
-/* $Id: exception.cc 4 2005-05-13 20:47:18Z taybin $ */
+/* $Id: exception.cc 2836 2008-01-07 20:41:51Z paul $ */
 
 /* exception.cc
  *
@@ -21,6 +21,9 @@
  */
 
 #include <glib/gmessages.h>
+#ifndef g_assert
+#include <glib/gtestutils.h>
+#endif
 #include <glibmm/exception.h>
 
 
