@@ -15,7 +15,7 @@
     along with this program; if not, write to the Free Software
     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 
-    $Id: gtk_ui.cc 2475 2007-09-20 18:04:47Z paul $
+    $Id: gtk_ui.cc 2938 2008-01-18 19:58:34Z paul $
 */
 
 #include <cmath>
@@ -560,19 +560,17 @@ UI::handle_fatal (const char *message)
 void
 UI::popup_error (const char *text)
 {
-	PopUp *pup;
-
 	if (!caller_is_ui_thread()) {
 		error << "non-UI threads can't use UI::popup_error" 
 		      << endmsg;
 		return;
 	}
-	
-	pup = new PopUp (WIN_POS_MOUSE, 0, true);
-	pup->set_text (text);
-	pup->touch ();
-}
 
+	MessageDialog msg (text, true, MESSAGE_ERROR, BUTTONS_OK);
+	msg.set_title (_("Error"));
+	msg.set_position (WIN_POS_MOUSE);
+	msg.run ();
+}
 
 void
 UI::flush_pending ()
@@ -593,7 +591,8 @@ UI::flush_pending ()
 bool
 UI::just_hide_it (GdkEventAny *ev, Window *win)
 {
-	win->hide_all ();
+	cerr << "++++ JUST HIDING " << win->get_window() << endl;
+	win->hide ();
 	return true;
 }
 
