@@ -22,6 +22,9 @@
 
 #include <string>
 #include <vector>
+#include <sstream>
+#include <iostream>
+#include <glibmm/ustring.h>
 
 namespace PBD {
 
@@ -30,6 +33,7 @@ std::string short_version (std::string, std::string::size_type target_length);
 int    atoi (const std::string&);
 double atof (const std::string&);
 void   url_decode (std::string&);
+void   url_decode (Glib::ustring&);
 
 // std::string length2string (const int32_t frames, const float sample_rate);
 std::string length2string (const int64_t frames, const double sample_rate);
@@ -37,6 +41,14 @@ std::string length2string (const int64_t frames, const double sample_rate);
 std::vector<std::string> internationalize (const char *, const char **);
 bool strings_equal_ignore_case (const std::string& a, const std::string& b);
 
+template <class T> std::string 
+to_string (T t, std::ios_base & (*f)(std::ios_base&))
+{
+	std::ostringstream oss;
+	oss << f << t;
+	return oss.str();
+}
+
 } //namespace PBD
 
 #endif /* __pbd_convert_h__ */
