@@ -13,10 +13,10 @@
 ///
 ////////////////////////////////////////////////////////////////////////////////
 //
-// Last changed  : $Date: 2005/05/18 14:13:49 $
-// File revision : $Revision: 1.2 $
+// Last changed  : $Date: 2007-03-16 17:44:10 -0400 (Fri, 16 Mar 2007) $
+// File revision : $Revision: 1605 $
 //
-// $Id: TDStretch.cpp,v 1.2 2005/05/18 14:13:49 taybin Exp $
+// $Id: TDStretch.cpp 1605 2007-03-16 21:44:10Z paul $
 //
 ////////////////////////////////////////////////////////////////////////////////
 //
@@ -114,13 +114,6 @@ TDStretch::~TDStretch()
 
 
     
-// Calculates the x having the closest 2^x value for the given value
-static int _getClosest2Power(double value)
-{
-    return (int)(log(value) / log(2.0) + 0.5);
-}
-
-
 
 // Sets routine control parameters. These control are certain time constants
 // defining how the sound is stretched to the desired duration.
