@@ -14,7 +14,6 @@
 	You should have received a copy of the GNU General Public License
 	along with this program; if not, write to the Free Software
 	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
-
 */
 
 #include <iostream>
@@ -22,6 +21,7 @@
 #include <cmath>
 #include <sstream>
 #include <vector>
+#include <iomanip>
 
 #define __STDC_FORMAT_MACROS
 #include <inttypes.h>
@@ -45,6 +45,8 @@
 #include <ardour/location.h>
 #include <ardour/dB.h>
 #include <ardour/panner.h>
+#include <ardour/tempo.h>
+#include <ardour/types.h>
 
 #include "mackie_control_protocol.h"
 
@@ -69,27 +71,6 @@ using boost::shared_ptr;
 
 MackieMidiBuilder builder;
 
-// Copied from tranzport_control_protocol.cc
-static inline double 
-gain_to_slider_position (ARDOUR::gain_t g)
-{
-	if (g == 0) return 0;
-	return pow((6.0*log(g)/log(2.0)+192.0)/198.0, 8.0);
-}
-
-/*
-	Copied from tranzport_control_protocol.cc
-	TODO this seems to return slightly wrong values, namely
-	with the UI slider at max, we get a 0.99something value.
-*/
-static inline ARDOUR::gain_t 
-slider_position_to_gain (double pos)
-{
-	/* XXX Marcus writes: this doesn't seem right to me. but i don't have a better answer ... */
-	if (pos == 0.0) return 0;
-	return pow (2.0,(sqrt(sqrt(sqrt(pos)))*198.0-192.0)/6.0);
-}
-
 MackieControlProtocol::MackieControlProtocol (Session& session)
 	: ControlProtocol  (session, X_("Mackie"))
 	, _current_initial_bank( 0 )
@@ -100,6 +81,7 @@ MackieControlProtocol::MackieControlProtocol (Session& session)
 	, pfd( 0 )
 	, nfds( 0 )
 	, _jog_wheel( *this )
+	, _timecode_type( ARDOUR::AnyTime::BBT )
 {
 #ifdef DEBUG
 	cout << "MackieControlProtocol::MackieControlProtocol" << endl;
@@ -139,14 +121,28 @@ Mackie::Surface & MackieControlProtocol::surface()
 	return *_surface;
 }
 
-const Mackie::MackiePort & MackieControlProtocol::mcu_port() const
+const Mackie::SurfacePort & MackieControlProtocol::mcu_port() const
 {
-	return dynamic_cast<const MackiePort &>( *_ports[0] );
+	if ( _ports.size() < 1 )
+	{
+		return _dummy_port;
+	}
+	else
+	{
+		return dynamic_cast<const MackiePort &>( *_ports[0] );
+	}
 }
 
-Mackie::MackiePort & MackieControlProtocol::mcu_port()
+Mackie::SurfacePort & MackieControlProtocol::mcu_port()
 {
-	return dynamic_cast<const MackiePort &>( *_ports[0] );
+	if ( _ports.size() < 1 )
+	{
+		return _dummy_port;
+	}
+	else
+	{
+		return dynamic_cast<MackiePort &>( *_ports[0] );
+	}
 }
 
 // go to the previous track.
@@ -309,51 +305,28 @@ void MackieControlProtocol::switch_banks( int initial )
 		{
 			Strip & strip = *surface().strips[i];
 			// send zero for this strip
-			port_for_id(i).write( builder.zero_strip( strip ) );
+			MackiePort & port = port_for_id(i);
+			port.write( builder.zero_strip( port, strip ) );
 		}
 	}
 	
 	// display the current start bank.
-	if ( mcu_port().emulation() == MackiePort::bcf2000 )
-	{
-		if ( _current_initial_bank == 0 )
-		{
-			// send Ar. to 2-char display on the master
-			mcu_port().write( builder.two_char_display( "Ar", ".." ) );
-		}
-		else
-		{
-			// write the current first remote_id to the 2-char display
-			mcu_port().write( builder.two_char_display( _current_initial_bank ) );
-		}
-	}
+	surface().display_bank_start( mcu_port(), builder, _current_initial_bank );
 }
 
 void MackieControlProtocol::zero_all()
 {
 	// TODO turn off SMPTE displays
 	
-	if ( mcu_port().emulation() == MackiePort::bcf2000 )
-	{
-		// clear 2-char display
-		mcu_port().write( builder.two_char_display( "LC" ) );
-	}
-	
 	// zero all strips
 	for ( Surface::Strips::iterator it = surface().strips.begin(); it != surface().strips.end(); ++it )
 	{
-		port_for_id( (*it)->index() ).write( builder.zero_strip( **it ) );
+		MackiePort & port = port_for_id( (*it)->index() );
+		port.write( builder.zero_strip( port, **it ) );
 	}
 	
 	// and the master strip
-	mcu_port().write( builder.zero_strip( master_strip() ) );
-	
-	// and the led ring for the master strip, in bcf mode
-	if ( mcu_port().emulation() == MackiePort::bcf2000 )
-	{
-		Control & control = *surface().controls_by_name["jog"];
-		mcu_port().write( builder.build_led_ring( dynamic_cast<Pot &>( control ), off ) );
-	}
+	mcu_port().write( builder.zero_strip( dynamic_cast<MackiePort&>( mcu_port() ), master_strip() ) );
 	
 	// turn off global buttons and leds
 	// global buttons are only ever on mcu_port, so we don't have
@@ -366,6 +339,9 @@ void MackieControlProtocol::zero_all()
 			mcu_port().write( builder.zero_control( control ) );
 		}
 	}
+
+	// any hardware-specific stuff
+	surface().zero_all( mcu_port(), builder );
 }
 
 int MackieControlProtocol::set_active( bool yn )
@@ -488,7 +464,7 @@ void MackieControlProtocol::update_led( Mackie::Button & button, Mackie::LedStat
 {
 	if ( ls != none )
 	{
-		MackiePort * port = 0;
+		SurfacePort * port = 0;
 		if ( button.group().is_strip() )
 		{
 			if ( button.group().is_master() )
@@ -508,9 +484,28 @@ void MackieControlProtocol::update_led( Mackie::Button & button, Mackie::LedStat
 	}
 }
 
+void MackieControlProtocol::update_smpte_beats_led()
+{
+	switch ( _timecode_type )
+	{
+		case ARDOUR::AnyTime::BBT:
+			update_global_led( "beats", on );
+			update_global_led( "smpte", off );
+			break;
+		case ARDOUR::AnyTime::SMPTE:
+			update_global_led( "smpte", on );
+			update_global_led( "beats", off );
+			break;
+		default:
+			ostringstream os;
+			os << "Unknown Anytime::Type " << _timecode_type;
+			throw runtime_error( os.str() );
+	}
+}
+
 void MackieControlProtocol::update_global_button( const string & name, LedState ls )
 {
-	if ( surface().controls_by_name.find( name ) !=surface().controls_by_name.end() )
+	if ( surface().controls_by_name.find( name ) != surface().controls_by_name.end() )
 	{
 		Button * button = dynamic_cast<Button*>( surface().controls_by_name[name] );
 		mcu_port().write( builder.build_led( button->led(), ls ) );
@@ -523,6 +518,21 @@ void MackieControlProtocol::update_global_button( const string & name, LedState
 	}
 }
 
+void MackieControlProtocol::update_global_led( const string & name, LedState ls )
+{
+	if ( surface().controls_by_name.find( name ) != surface().controls_by_name.end() )
+	{
+		Led * led = dynamic_cast<Led*>( surface().controls_by_name[name] );
+		mcu_port().write( builder.build_led( *led, ls ) );
+	}
+	else
+	{
+#ifdef DEBUG
+		cout << "Led " << name << " not found" << endl;
+#endif
+	}
+}
+
 // send messages to surface to set controls to correct values
 void MackieControlProtocol::update_surface()
 {
@@ -538,16 +548,13 @@ void MackieControlProtocol::update_surface()
 		// update strip from route
 		master_route_signal->notify_all();
 		
-		// turn off the led ring, for bcf emulation mode
-		if ( mcu_port().emulation() == MackiePort::bcf2000 )
-		{
-			Control & control = *surface().controls_by_name["jog"];
-			mcu_port().write( builder.build_led_ring( dynamic_cast<Pot &>( control ), off ) );
-		}
+		// sometimes the jog wheel is a pot
+		surface().blank_jog_ring( mcu_port(), builder );
 		
 		// update global buttons and displays
 		notify_record_state_changed();
 		notify_transport_state_changed();
+		update_smpte_beats_led();
 	}
 }
 
@@ -576,34 +583,46 @@ void MackieControlProtocol::connect_session_signals()
 void MackieControlProtocol::add_port( MIDI::Port & midi_port, int number )
 {
 #ifdef DEBUG
-		cout << "add port " << midi_port.name() << ", " << midi_port.device() << endl;
+	cout << "add port " << midi_port.name() << ", " << midi_port.device() << ", " << midi_port.type() << endl;
+	cout << "MIDI::Port::ALSA_Sequencer " << MIDI::Port::ALSA_Sequencer << endl;
+	cout << "MIDI::Port::Unknown " << MIDI::Port::Unknown << endl;
 #endif
+	if ( string( midi_port.device() ) == string( "ardour" ) )
+	{
+		throw MackieControlException( "The Mackie MCU driver will not use a port with device=ardour" );
+	}
+	else if ( midi_port.type() == MIDI::Port::ALSA_Sequencer )
+	{
+		throw MackieControlException( "alsa/sequencer ports don't work with the Mackie MCU driver right now" );
+	}
+	else
+	{
+		MackiePort * sport = new MackiePort( *this, midi_port, number );
+		_ports.push_back( sport );
+		
+		connections_back = sport->init_event.connect(
+			sigc::bind (
+				mem_fun (*this, &MackieControlProtocol::handle_port_init)
+				, sport
+			)
+		);
 
-	MackiePort * sport = new MackiePort( *this, midi_port, number );
-	_ports.push_back( sport );
+		connections_back = sport->active_event.connect(
+			sigc::bind (
+				mem_fun (*this, &MackieControlProtocol::handle_port_active)
+				, sport
+			)
+		);
 
-	connections_back = sport->init_event.connect(
-		sigc::bind (
-			mem_fun (*this, &MackieControlProtocol::handle_port_init)
-			, sport
-		)
-	);
-
-	connections_back = sport->active_event.connect(
-		sigc::bind (
-			mem_fun (*this, &MackieControlProtocol::handle_port_active)
-			, sport
-		)
-	);
-
-	connections_back = sport->inactive_event.connect(
-		sigc::bind (
-			mem_fun (*this, &MackieControlProtocol::handle_port_inactive)
-			, sport
-		)
-	);
-	
-	_ports_changed = true;
+		connections_back = sport->inactive_event.connect(
+			sigc::bind (
+				mem_fun (*this, &MackieControlProtocol::handle_port_inactive)
+				, sport
+			)
+		);
+		
+		_ports_changed = true;
+	}
 }
 
 void MackieControlProtocol::create_ports()
@@ -662,15 +681,23 @@ void MackieControlProtocol::initialize_surface()
 	
 	set_route_table_size( strips );
 	
-	switch ( mcu_port().emulation() )
+	// TODO same as code in mackie_port.cc
+	string emulation = ARDOUR::Config->get_mackie_emulation();
+	if ( emulation == "bcf" )
 	{
-		case MackiePort::bcf2000: _surface = new BcfSurface( strips ); break;
-		case MackiePort::mackie: _surface = new MackieSurface( strips ); break;
-		default:
-			ostringstream os;
-			os << "no Surface class found for emulation: " << mcu_port().emulation();
-			throw MackieControlException( os.str() );
+		_surface = new BcfSurface( strips );
 	}
+	else if ( emulation == "mcu" )
+	{
+		_surface = new MackieSurface( strips );
+	}
+	else
+	{
+		ostringstream os;
+		os << "no Surface class found for emulation: " << emulation;
+		throw MackieControlException( os.str() );
+	}
+
 	_surface->init();
 	
 	// Connect events. Must be after route table otherwise there will be trouble
@@ -682,6 +709,12 @@ void MackieControlProtocol::initialize_surface()
 
 void MackieControlProtocol::close()
 {
+	// stop polling, and wait for it...
+	// must be before other shutdown otherwise polling loop
+	// calls methods on objects that are deleted
+	_polling = false;
+	pthread_join( thread, 0 );
+	
 	// TODO disconnect port active/inactive signals
 	// Or at least put a lock here
 	
@@ -739,10 +772,6 @@ void MackieControlProtocol::close()
 		_surface = 0;
 	}
 	
-	// stop polling, and wait for it...
-	_polling = false;
-	pthread_join( thread, 0 );
-	
 	// shut down MackiePorts
 	for( MackiePorts::iterator it = _ports.begin(); it != _ports.end(); ++it )
 	{
@@ -839,7 +868,7 @@ void MackieControlProtocol::handle_control_event( SurfacePort & port, Control &
 			// at which point the fader should just reset itself
 			if ( route != 0 )
 			{
-				route->set_gain( slider_position_to_gain( state.pos ), this );
+				route->gain_control().set_value( state.pos );
 				
 				// must echo bytes back to slider now, because
 				// the notifier only works if the fader is not being
@@ -971,14 +1000,35 @@ void MackieControlProtocol::notify_record_enable_changed( RouteSignal * route_si
 	}
 }
 
-void MackieControlProtocol::notify_gain_changed( RouteSignal * route_signal )
+void MackieControlProtocol::notify_active_changed( RouteSignal * route_signal )
+{
+	try
+	{
+#ifdef DEBUG
+		cout << "MackieControlProtocol::notify_active_changed" << endl;
+#endif
+		refresh_current_bank();
+	}
+	catch( exception & e )
+	{
+		cout << e.what() << endl;
+	}
+}
+	
+void MackieControlProtocol::notify_gain_changed( RouteSignal * route_signal, bool force_update )
 {
 	try
 	{
 		Fader & fader = route_signal->strip().gain();
 		if ( !fader.in_use() )
 		{
-			route_signal->port().write( builder.build_fader( fader, gain_to_slider_position( route_signal->route().effective_gain() ) ) );
+			float gain_value = route_signal->route().gain_control().get_value();
+			// check that something has actually changed
+			if ( force_update || gain_value != route_signal->last_gain_written() )
+			{
+				route_signal->port().write( builder.build_fader( fader, gain_value ) );
+				route_signal->last_gain_written( gain_value );
+			}
 		}
 	}
 	catch( exception & e )
@@ -1006,12 +1056,9 @@ void MackieControlProtocol::notify_name_changed( void *, RouteSignal * route_sig
 				line1 = PBD::short_version( fullname, 6 );
 			}
 			
-			route_signal->port().write_sysex(
-				builder.strip_display( strip, 0, line1 )
-			);
-			route_signal->port().write_sysex(
-				builder.strip_display_blank( strip, 1 )
-			);
+			SurfacePort & port = route_signal->port();
+			port.write( builder.strip_display( port, strip, 0, line1 ) );
+			port.write( builder.strip_display_blank( port, strip, 1 ) );
 		}
 	}
 	catch( exception & e )
@@ -1020,7 +1067,7 @@ void MackieControlProtocol::notify_name_changed( void *, RouteSignal * route_sig
 	}
 }
 
-void MackieControlProtocol::notify_panner_changed( RouteSignal * route_signal )
+void MackieControlProtocol::notify_panner_changed( RouteSignal * route_signal, bool force_update )
 {
 	try
 	{
@@ -1030,7 +1077,17 @@ void MackieControlProtocol::notify_panner_changed( RouteSignal * route_signal )
 		{
 			float pos;
 			route_signal->route().panner()[0]->get_effective_position( pos );
-			route_signal->port().write( builder.build_led_ring( pot, ControlState( on, pos ), MackieMidiBuilder::midi_pot_mode_dot ) );
+			
+			// cache the MidiByteArray here, because the mackie led control is much lower
+			// resolution than the panner control. So we save lots of byte
+			// sends in spite of more work on the comparison
+			MidiByteArray bytes = builder.build_led_ring( pot, ControlState( on, pos ), MackieMidiBuilder::midi_pot_mode_dot );
+			// check that something has actually changed
+			if ( force_update || bytes != route_signal->last_pan_written() )
+			{
+				route_signal->port().write( bytes );
+				route_signal->last_pan_written( bytes );
+			}
 		}
 		else
 		{
@@ -1049,19 +1106,98 @@ void MackieControlProtocol::update_automation( RouteSignal & rs )
 	ARDOUR::AutoState gain_state = rs.route().gain_automation_state();
 	if ( gain_state == Touch || gain_state == Play )
 	{
-		notify_gain_changed( &rs );
+		notify_gain_changed( &rs, false );
 	}
 	
 	ARDOUR::AutoState panner_state = rs.route().panner().automation_state();
 	if ( panner_state == Touch || panner_state == Play )
 	{
-		notify_panner_changed( &rs );
+		notify_panner_changed( &rs, false );
+	}
+	_automation_last.start();
+}
+
+string MackieControlProtocol::format_bbt_timecode( nframes_t now_frame )
+{
+	BBT_Time bbt_time;
+	session->bbt_time( now_frame, bbt_time );
+	
+	// According to the Logic docs
+	// digits: 888/88/88/888
+	// BBT mode: Bars/Beats/Subdivisions/Ticks
+	ostringstream os;
+	os << setw(3) << setfill('0') << bbt_time.bars;
+	os << setw(2) << setfill('0') << bbt_time.beats;
+	
+	// figure out subdivisions per beat
+	const Meter & meter = session->tempo_map().meter_at( now_frame );
+	int subdiv = 2;
+	if ( meter.note_divisor() == 8 && meter.beats_per_bar() == 12.0 || meter.beats_per_bar() == 9.0 || meter.beats_per_bar() == 6.0 )
+	{
+		subdiv = 3;
+	}
+	
+	uint32_t subdivisions = bbt_time.ticks / uint32_t( Meter::ticks_per_beat / subdiv );
+	uint32_t ticks = bbt_time.ticks % uint32_t( Meter::ticks_per_beat / subdiv );
+	
+	os << setw(2) << setfill('0') << subdivisions + 1;
+	os << setw(3) << setfill('0') << ticks;
+	
+	return os.str();
+}
+
+string MackieControlProtocol::format_smpte_timecode( nframes_t now_frame )
+{
+	SMPTE::Time smpte;
+	session->smpte_time( now_frame, smpte );
+
+	// According to the Logic docs
+	// digits: 888/88/88/888
+	// SMPTE mode: Hours/Minutes/Seconds/Frames
+	ostringstream os;
+	os << setw(3) << setfill('0') << smpte.hours;
+	os << setw(2) << setfill('0') << smpte.minutes;
+	os << setw(2) << setfill('0') << smpte.seconds;
+	os << setw(3) << setfill('0') << smpte.frames;
+	
+	return os.str();
+}
+
+void MackieControlProtocol::update_timecode_display()
+{
+	if ( surface().has_timecode_display() )
+	{
+		// do assignment here so current_frame is fixed
+		nframes_t current_frame = session->transport_frame();
+		string timecode;
+		
+		switch ( _timecode_type )
+		{
+			case ARDOUR::AnyTime::BBT:
+				timecode = format_bbt_timecode( current_frame );
+				break;
+			case ARDOUR::AnyTime::SMPTE:
+				timecode = format_smpte_timecode( current_frame );
+				break;
+			default:
+				ostringstream os;
+				os << "Unknown timecode: " << _timecode_type;
+				throw runtime_error( os.str() );
+		}	
+		
+		// only write the timecode string to the MCU if it's changed
+		// since last time. This is to reduce midi bandwidth used.
+		if ( timecode != _timecode_last )
+		{
+			surface().display_timecode( mcu_port(), builder, timecode, _timecode_last );
+			_timecode_last = timecode;
+		}
 	}
 }
 
-void MackieControlProtocol::poll_automation()
+void MackieControlProtocol::poll_session_data()
 {
-	if ( _active )
+	if ( _active && _automation_last.elapsed() >= 20 )
 	{
 		// do all currently mapped routes
 		for( RouteSignals::iterator it = route_signals.begin(); it != route_signals.end(); ++it )
@@ -1070,7 +1206,14 @@ void MackieControlProtocol::poll_automation()
 		}
 		
 		// and the master strip
-		if ( master_route_signal != 0 ) update_automation( *master_route_signal );
+		if ( master_route_signal != 0 )
+		{
+			update_automation( *master_route_signal );
+		}
+		
+		update_timecode_display();
+		
+		_automation_last.start();
 	}
 }
 
@@ -1088,7 +1231,7 @@ LedState MackieControlProtocol::frm_left_press( Button & button )
 	);
 	
 	// allow a quick double to go past a previous mark 
-	if ( elapsed < 500 && loc != 0)
+	if ( session->transport_rolling() && elapsed < 500 && loc != 0 )
 	{
 		Location * loc_two_back = session->locations()->first_location_before ( loc->start() );
 		if ( loc_two_back != 0 )
@@ -1501,7 +1644,7 @@ LedState MackieControlProtocol::marker_release( Button & button )
 	return off;
 }
 
-void jog_wheel_state_display( JogWheel::State state, MackiePort & port )
+void jog_wheel_state_display( JogWheel::State state, SurfacePort & port )
 {
 	switch( state )
 	{
@@ -1547,3 +1690,49 @@ Mackie::LedState MackieControlProtocol::scrub_release( Mackie::Button & )
 		_jog_wheel.jog_wheel_state() == JogWheel::shuttle
 	;
 }
+
+LedState MackieControlProtocol::drop_press( Button & button )
+{
+	session->remove_last_capture();
+	return on;
+}
+
+LedState MackieControlProtocol::drop_release( Button & button )
+{
+	return off;
+}
+
+LedState MackieControlProtocol::save_press( Button & button )
+{
+	session->save_state( "" );
+	return on;
+}
+
+LedState MackieControlProtocol::save_release( Button & button )
+{
+	return off;
+}
+
+LedState MackieControlProtocol::smpte_beats_press( Button & )
+{
+	switch ( _timecode_type )
+	{
+		case ARDOUR::AnyTime::BBT:
+			_timecode_type = ARDOUR::AnyTime::SMPTE;
+			break;
+		case ARDOUR::AnyTime::SMPTE:
+			_timecode_type = ARDOUR::AnyTime::BBT;
+			break;
+		default:
+			ostringstream os;
+			os << "Unknown Anytime::Type " << _timecode_type;
+			throw runtime_error( os.str() );
+	}
+	update_smpte_beats_led();
+	return on;
+}
+
+LedState MackieControlProtocol::smpte_beats_release( Button & )
+{
+	return off;
+}
